package saaps.dbt;

import java.text.DateFormat;
import java.text.NumberFormat;

import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

import java.util.Vector;

import java.io.Serializable;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;

/**
* A class for the AE, AL, AO, and AU indeces.
*
* The class contains the date and data in units of nT.
*
* @author Peter Wintoft
* @version 1.0 2001-03-05
*/
public class AE extends DatabaseObject implements Serializable
{
  // Fields
  protected static int numberOfIndexes = 4;
  private Date d;
  // AE, AL, AO, AU.
  private double[] data = new double[numberOfIndexes];
  
  // Constructors
  public AE()
  {
  }

  /**
   * This constructor is used if you want an "empty" object with this date.
   */
  public AE(Date D) {
    this.d = D;
    for (int i=0;i<this.data.length;i++) {
      this.data[i] = Double.NaN;
    }
  }

  public AE(Date D,double[] data) {
    this.d = D;
    for (int i=0;i<this.data.length;i++) {
      this.data[i] = data[i];
    }
  }

  public AE(long time,double[] data)
  {
    this(new Date(time),data);
  }
  
  // Methods
  public double[] getDataArray()
  {
    return this.data;
  }

  //Implemented for DatabaseObject
  public Date getDate()
  {
    return this.d;
  }

  // This method is the generic form of accessing data.
  public Vector getData()
  {
  	Vector vec = new Vector(this.numberOfIndexes);
        for (int i=0;i<this.numberOfIndexes;i++) {
  	  vec.addElement(new Double(this.data[i]));
        }
  	return vec;
  }
  /**
   * This method returns the size of the data that is to be written to file
   */
  public long getDataSize() throws IOException {

    ByteArrayOutputStream baos = new ByteArrayOutputStream();  
    DataOutputStream    dos  = new DataOutputStream(baos);  
    dos.writeLong(this.d.getTime());
    for (int i=0;i<this.data.length;i++) {
      dos.writeDouble(this.data[i]);
    }
    dos.flush();  
    long size = baos.size();  // now you know how large ....
    return size;
  }

  public void setDate(Date D)
  {
    this.d = D;
  }
  public void setDate(long L)
  {
    this.d = new Date(L);
  }
  public void setAE(double[] data)
  {
    for (int i=0;i<this.data.length;i++) {
      this.data[i] = data[i];
    }
  }

  //Implemented for DatabaseObject getDate
  public int getNumberOfDataIndexes() {
    return (numberOfIndexes);
  }
     
  //Implemented for DatabaseObject 
  public double getData(int index) {
    //Date is not considered a data value.
    if (index<this.numberOfIndexes) {
      return this.data[index];
    }
    else {
      System.out.println("Index out of bounds for AE");
      return Double.NaN;
    }
  }

  public String toString() {
//     // Set the date format.
//     DateFormat df = DateFormat.getDateTimeInstance(DateFormat.DEFAULT,DateFormat.DEFAULT,new Locale("sv","SE")); //en UK
//     df.setTimeZone(TimeZone.getTimeZone("UTC"));

    // Find the selected dates.
    Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
    cal.clear(cal.MILLISECOND);
    cal.setTime(d);

    NumberFormat nf = NumberFormat.getInstance();
    nf.setMinimumIntegerDigits(2);

    return ( cal.get(Calendar.YEAR) + " " + nf.format(cal.get(Calendar.MONTH)+1) + " " + nf.format(cal.get(Calendar.DAY_OF_MONTH)) + " " + nf.format(cal.get(Calendar.HOUR_OF_DAY)) + " " + nf.format(cal.get(Calendar.MINUTE)) + " " + nf.format(cal.get(Calendar.SECOND)) + " " + Double.toString(this.data[0]) + " " + Double.toString(this.data[1]) + " " + Double.toString(this.data[2]) + " " + Double.toString(this.data[3]) );
//    return ( cal.get(Calendar.YEAR) + " " + nf.format(cal.get(Calendar.MONTH)+1) + " " + nf.format(cal.get(Calendar.DAY_OF_MONTH)) + " " + nf.format(cal.get(Calendar.HOUR_OF_DAY)) + " " + nf.format(cal.get(Calendar.MINUTE)) + " " + nf.format(cal.get(Calendar.SECOND)) + " " + Double.toString(getData()) );
    
  }

}
