package saaps.dbt;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;

import saaps.util.*;

/**
* BuildHourlyACE downloads the ACE mag and swe data and stores it in the database.
* FTP address:
*   ftp://sec.noaa.gov/pub/lists/ace2/YYYYMM_ace_mag_1h.txt 
*   ftp://sec.noaa.gov/pub/lists/ace2/YYYYMM_ace_swepam_1h.txt 
*@author Peter Wintoft
*@version 1.0 2000-09-04
*/
public class BuildHourlyACE extends Object{
  //The OMNI list.
  private String urlDir = "ftp://sec.noaa.gov/pub/lists/ace2/";
  //Fields to read.
//  private int[] fields = {1,2,3,9,10,13,14,15,16,17,18,19,20,21,22,23,24,25,
//                  28,29,30,36,37,38,39,40,41,42};
//  private boolean[] useField;
//  private boolean[] useField =
//  {true , true , true , false, false, false, false, false, true , true ,
//   false, false, true , true , true , true , true , true , true , true ,
//   true , true , true , true , true , false, false, true , true , true ,
//   false, false, false, false, false, true , true , true , true , true ,
//   true , true , true , false};
  //Field index.
//  private int fieldIndex = -1;
  //Maximum field index.
//  private int maxFieldIndex = 42;
  //The index to the data array.
//  private int dataIndex = 0;
  //Data gap values for the 25 data fields to be downloaded.
//  private static double[] dataGapValues =
    //F      B      Bx     ByGSE  BzGSE  ByGSM  BzGSM
//    { 999.9, 999.9, 999.9, 999.9, 999.9, 999.9, 999.9,
    //sigmaF sigmaB sigmaBx sigmaBy sigmaBz
//      999.9, 999.9, 999.9,  999.9,  999.9,
    //T          N      V       sigmaT     sigmaN sigmaV
//      9999990.0, 999.9, 9999.0, 9999990.0, 999.9, 9999.0,
    //Dst    P1        P2        P4       P10      P30      P60
//      99999, 99999.90, 99999.90, 9999.99, 9999.99, 9999.99, 9999.99};
  //The stream tokenizer.
  private StreamTokenizer streamTokenizer;
  //Date of one observation.
//  private Date date;
  //Data for one observation.
//  private double[] data;
  private Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone("GMT"));

  public BuildHourlyACE()
  {
/*    //Set up useField.
    useField = new boolean[43];
    for (int i=0;i<useField.length;i++)
      useField[i] = false;
    for (int i=0;i<fields.length;i++)
      useField[fields[i]-1] = true; */
  }

  public String[] getURLList()
  {
    String[] urlList = null;
    URL url = null;
    InputStream inStream = null;
    try
    {
      // Create buffered reader.
      BufferedReader bufInput = getURLBufferedReader(urlDir);
      // Create a vector to hold the text lines.
      Vector vec = new Vector(100);
      String textLine = null;
      while ((textLine = bufInput.readLine()) != null)
        vec.addElement(new String(textLine));
      vec.trimToSize();
      urlList = new String[vec.size()];
      for (int i=0;i<vec.size();i++)
        urlList[i] = new String((String) vec.elementAt(i));
    }
    catch (MalformedURLException e)
    {
      System.err.println(e);
      return null;
    }
    catch (IOException e)
    {
      System.err.println(e);
      return null;
    }
    return urlList;
  }
  
  public BufferedReader getURLBufferedReader(String urlStr) throws MalformedURLException, IOException
  {
    URL url = new URL(urlStr);
    InputStream inStream = url.openStream();
    BufferedReader buf = new BufferedReader(new InputStreamReader(inStream));
    return buf;
  }
  
  public BufferedReader getBufferedReader(String fileStr) throws IOException
  {
    BufferedReader buf = new BufferedReader(new FileReader(fileStr));
    return buf;
  }
  
  public void skipHeader() throws IOException
  {
    boolean cont = true;
    int type1 = 35;   // #
    int type2 = 45;   // -
//    int val = Character.getNumericValue('#');
    while (cont)
    {
      streamTokenizer.nextToken();
      if (streamTokenizer.ttype == type1)
      {
        streamTokenizer.nextToken();
        if (streamTokenizer.ttype == type2)
        {
          while (streamTokenizer.ttype == type2)
            streamTokenizer.nextToken();
          if (streamTokenizer.ttype == StreamTokenizer.TT_NUMBER)
            if ((int) streamTokenizer.nval == 1999 | (int) streamTokenizer.nval == 2000)
              cont = false;
        }
      }
    }
//      System.out.println(streamTokenizer.sval);
//      if (streamTokenizer.ttype == StreamTokenizer.TT_NUMBER)
//        if (streamTokenizer.nval == 1999 | streamTokenizer.nval == 2000)
//          cont = false;
//    }
    streamTokenizer.pushBack();
//    while (streamTokenizer.sval == null)
//      streamTokenizer.nextToken();
//    while (!streamTokenizer.sval.startsWith("#-----------"))
//      streamTokenizer.nextToken();
  }
  
  public void buildOneYearMag(String urlName)
  {
    try
    {
      //Get the stream tokenizer to the URL.
      System.out.println("Reading URL " + urlName);
      streamTokenizer = new StreamTokenizer(getURLBufferedReader(urlName));
      //Skip header.
      System.out.println("Skipping header ...");
      skipHeader();
      System.out.println("Reading mag data ...");
      //Create vector to hold data.
      Vector vec = new Vector(24*366);
      //Go through the tokens and put into vector.
      while ((streamTokenizer.nextToken()) != StreamTokenizer.TT_EOF)
      {
        Date date = readDate();
        double[] mag = readMagData();
        ACEmagHourly aceMag = new ACEmagHourly(date,mag);
        vec.add(aceMag);
//        System.out.println(aceMag.toString());
      }
      vec.trimToSize();
      //Store in database.
      ObjectTimeSeries ots = new ObjectTimeSeries(vec);
      Database db = Database.getInstance("ACEmagHourlyDB");
      db.store(ots);
    }
    catch (IllegalTimeSeriesException itse)
    {
      System.err.println(itse);
    }
    catch (MalformedURLException e)
    {
      System.err.println(e);
    }
    catch (IOException e)
    {
      System.err.println(e);
    }    
    catch (Exception e)
    {
      System.err.println(e);
    }    
  }
  
  public void buildOneYearSwe(String urlName)
  {
    try
    {
      //Get the stream tokenizer to the URL.
      System.out.println("Reading URL " + urlName);
      streamTokenizer = new StreamTokenizer(getURLBufferedReader(urlName));
      //Skip header.
      System.out.println("Skipping header ...");
      skipHeader();
      System.out.println("Reading swe data ...");
      //Create vector to hold data.
      Vector vec = new Vector(24*366);
      //Go through the tokens and put into vector.
      while ((streamTokenizer.nextToken()) != StreamTokenizer.TT_EOF)
      {
        Date date = readDate();
        double[] swe = readSweData();
        ACEsweHourly aceSwe = new ACEsweHourly(date,swe);
        vec.add(aceSwe);
//        System.out.println(aceSwe.toString());
      }
      vec.trimToSize();
      //Store in database.
      ObjectTimeSeries ots = new ObjectTimeSeries(vec);
      Database db = Database.getInstance("ACEsweHourlyDB");
      db.store(ots);
    }
    catch (IllegalTimeSeriesException itse)
    {
      System.err.println(itse);
    }
    catch (MalformedURLException e)
    {
      System.err.println(e);
    }
    catch (IOException e)
    {
      System.err.println(e);
    }    
    catch (Exception e)
    {
      System.err.println(e);
    }    
  }
  
  public void buildLocalMag()
  {
    //File name of the source data.
    String fileName = "/space/saaps/DatabaseSrc/mag.txt";
    try
    {
      //Get the tokens. Order of tokens:
      //year, day-of-year, hour-of-day, B, Bx, By, and Bz, in GSM.
      streamTokenizer = new StreamTokenizer(getBufferedReader(fileName));
      //Create a vector to keep the data.
      Vector vec = new Vector(3*366*24);
      //Go through the tokens and add to vector.
      while((streamTokenizer.nextToken()) != StreamTokenizer.TT_EOF)
      {
        Date d = readLocalDate();
        double[] x = readLocalMagData();
        ACEmagHourly mag = new ACEmagHourly(d,x);
        vec.add(mag);
      }
      vec.trimToSize();
      //Store in database.
      ObjectTimeSeries ots = new ObjectTimeSeries(vec);
      Database db = Database.getInstance("ACEmagHourlyDB");
      db.store(ots);
    }
    catch (IllegalTimeSeriesException itse)
    {
      System.err.println(itse);
    }
    catch (IOException ioe)
    {
      System.err.println(ioe);
    }
    catch (Exception e)
    {
      System.err.println(e);
    }    
  }
  
  public void buildLocalSwe()
  {
    //File name of the source data.
    String fileName = "/space/saaps/DatabaseSrc/swepam.txt";
    try
    {
      //Get the tokens. Order of tokens:
      //year, day-of-year, hour-of-day, n, and V.
      streamTokenizer = new StreamTokenizer(getBufferedReader(fileName));
      //Create a vector to keep the data.
      Vector vec = new Vector(3*366*24);
      //Go through the tokens and add to vector.
      while((streamTokenizer.nextToken()) != StreamTokenizer.TT_EOF)
      {
        Date d = readLocalDate();
        double[] x = readLocalSweData();
        ACEsweHourly swe = new ACEsweHourly(d,x);
        vec.add(swe);
      }
      vec.trimToSize();
      //Store in database.
      ObjectTimeSeries ots = new ObjectTimeSeries(vec);
      Database db = Database.getInstance("ACEsweHourlyDB");
      db.store(ots);
    }
    catch (IllegalTimeSeriesException itse)
    {
      System.err.println(itse);
    }
    catch (IOException ioe)
    {
      System.err.println(ioe);
    }
    catch (Exception e)
    {
      System.err.println(e);
    }    
  }
  
  public String getURLFileName(String name, int start)
  {
    start -= 6;
    int end = name.indexOf(".txt") + 4;
    return new String(urlDir + name.substring(start,end));
  }
      
  public static void main(String[] args)
  {
    BuildHourlyACE ace = new BuildHourlyACE();

    //Start building from the local files.
    System.out.println("Building from local files.");
    System.out.println("Mag data ...");
    ace.buildLocalMag();
    System.out.println("Mag data ...");
    ace.buildLocalSwe();

    System.out.println("Building from remote files.");    
    //Get the directory list of the URL folder.
    String[] dirList = ace.getURLList();

    //Go through the list and download the data for MAG and SWE.
    for (int listItem=0;listItem<dirList.length;listItem++)
    {
      //Is it MAG data?
      int offset = 0;
      if ((offset = dirList[listItem].indexOf("_ace_mag")) > 0)
      {
        String urlFileName = ace.getURLFileName(dirList[listItem],offset);
        System.out.println("Connecting to " + urlFileName);
        ace.buildOneYearMag(urlFileName);
      }

      //Is it SWE data?
      offset = 0;
      if ((offset = dirList[listItem].indexOf("_ace_swepam")) > 0)
      {
        String urlFileName = ace.getURLFileName(dirList[listItem],offset);
        System.out.println("Connecting to " + urlFileName);
        ace.buildOneYearSwe(urlFileName);
      }
    }
  }

  private Date readLocalDate() throws IOException
  {
    //Token 1. = year.
    int year = (int) streamTokenizer.nval;
    //Token 2. = day-of-year.
    streamTokenizer.nextToken();
    int doy = (int) streamTokenizer.nval;
    //Token 3. = hour-of-day.
    streamTokenizer.nextToken();
    int hod = (int) streamTokenizer.nval;
    //Set the date.
    calendar.clear();
    calendar.set(Calendar.YEAR,year);
    calendar.set(Calendar.DAY_OF_YEAR,doy);
    calendar.set(Calendar.HOUR_OF_DAY,hod);
    return calendar.getTime();
  }
  
  private Date readDate() throws IOException
  {
    //Token 1. = year.
    int year = (int) streamTokenizer.nval;
    //Token 2. = month.
    streamTokenizer.nextToken();
    int month = (int) streamTokenizer.nval;
    //Token 3. = day.
    streamTokenizer.nextToken();
    int day = (int) streamTokenizer.nval;
    //Token 4. = hour and minute.
    streamTokenizer.nextToken();
    int hour = (int) streamTokenizer.nval/100;
    //Get the date.
    Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
    //Clear all fields.
    cal.clear();
    //Set year, day of year, and hour of day.
    cal.set(cal.YEAR,year);
    cal.set(cal.MONTH,month-1);
    cal.set(cal.DATE,day);
    cal.set(cal.HOUR_OF_DAY,hour);
    Date date = cal.getTime();
    return date;
    //Reset data index.
 //   dataIndex = 0;
  }

  private double[] readMagData() throws IOException
  {
    //Data gap value.
    double dataGapVal = -999.9;
    // Bx, By, Bz, Bt
    double[] data = new double[4];
    //Julian day. SKIP
    streamTokenizer.nextToken();
    //Seconds of day. SKIP
    streamTokenizer.nextToken();
    //S-flag. SKIP
    streamTokenizer.nextToken();
    //Bx
    streamTokenizer.nextToken();
    data[0] = streamTokenizer.nval;
    //By
    streamTokenizer.nextToken();
    data[1] = streamTokenizer.nval;
    //Bz
    streamTokenizer.nextToken();
    data[2] = streamTokenizer.nval;
    //Bt
    streamTokenizer.nextToken();
    data[3] = streamTokenizer.nval;
    //Latitude. SKIP
    streamTokenizer.nextToken();
    //Longitude. SKIP
    streamTokenizer.nextToken();
    //Replace data gap values with NaNs.
    for (int i=0;i<data.length;i++)
      if (data[i] == dataGapVal)
        data[i] = Double.NaN;
    return data;
  }
  
  private double[] readSweData() throws IOException
  {
    //Data gap value.
    double dataGapVal = -9999.9;
    // n, V
    double[] data = new double[2];
    //Julian day. SKIP
    streamTokenizer.nextToken();
    //Seconds of day. SKIP
    streamTokenizer.nextToken();
    //S-flag. SKIP
    streamTokenizer.nextToken();
    //n
    streamTokenizer.nextToken();
    data[0] = streamTokenizer.nval;
    //V
    streamTokenizer.nextToken();
    data[1] = streamTokenizer.nval;
    //Temperature. SKIP
    streamTokenizer.nextToken();
    while ((streamTokenizer.nextToken())!=StreamTokenizer.TT_NUMBER);
    //Replace data gap values with NaNs.
    for (int i=0;i<data.length;i++)
      if (data[i] == dataGapVal)
        data[i] = Double.NaN;
    return data;
  }
  
  private double[] readLocalMagData() throws IOException
  {
    //Data gap value.
    double dataGapVal = -999.9;
    //B, Bx, By, Bz.
    double[] data = new double[4];
    //B
    streamTokenizer.nextToken();
    data[0] = streamTokenizer.nval;
    //Bx
    streamTokenizer.nextToken();
    data[1] = streamTokenizer.nval;
    //By
    streamTokenizer.nextToken();
    data[2] = streamTokenizer.nval;
    //Bz
    streamTokenizer.nextToken();
    data[3] = streamTokenizer.nval;
    //Replace data gap values with NaNs.
    for (int i=0;i<data.length;i++)
      if (data[i] == dataGapVal)
        data[i] = Double.NaN;
    return data;
  }
  
  private double[] readLocalSweData() throws IOException
  {
    //Data gap value.
    double dataGapVal = -999.9;
    //n, V.
    double[] data = new double[2];
    //n
    streamTokenizer.nextToken();
    data[0] = streamTokenizer.nval;
    //V
    streamTokenizer.nextToken();
    data[1] = streamTokenizer.nval;
    //Replace data gap values with NaNs.
    for (int i=0;i<data.length;i++)
      if (data[i] == dataGapVal)
        data[i] = Double.NaN;
    return data;
  }
  
/*  private void readData() throws IOException
  {
    data = new double[25];
    while (fieldIndex < maxFieldIndex)
    {
      nextToken();
      if (useField[fieldIndex])
      {
        data[dataIndex] = streamTokenizer.nval;
        if (data[dataIndex] == dataGapValues[dataIndex])
          data[dataIndex] = Double.NaN;
        dataIndex++;
      }
    }
  } */
}
