package saaps.dbt;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;

import saaps.util.*;

/**
* BuildOMNI downloads the OMNI particle data.
* FTP address:
*   ftp://nssdc.gsfc.nasa.gov/pub/spacecraft_data/omni/
*@author Peter Wintoft
*@version 1.0 2000-02-22
*/
public class BuildOMNI extends Object{
  //The OMNI list.
//  private String urlDir = "ftp://nssdc.gsfc.nasa.gov/pub/spacecraft_data/omni/";
  private String urlDir = "ftp://nssdcftp.gsfc.nasa.gov/spacecraft_data/omni/";
  //Fields to read.
  private int[] fields = {1,2,3,9,10,13,14,15,16,17,18,19,20,21,22,23,24,25,
                  28,29,30,36,37,38,39,40,41,42};
  private boolean[] useField;
//  private boolean[] useField =
//  {true , true , true , false, false, false, false, false, true , true ,
//   false, false, true , true , true , true , true , true , true , true ,
//   true , true , true , true , true , false, false, true , true , true ,
//   false, false, false, false, false, true , true , true , true , true ,
//   true , true , true , false};
  //Field index.
  private int fieldIndex = -1;
  //Maximum field index.
  private int maxFieldIndex = 42;
  //The index to the data array.
  private int dataIndex = 0;
  //Data gap values for the 25 data fields to be downloaded.
  private static double[] dataGapValues =
    //F      B      Bx     ByGSE  BzGSE  ByGSM  BzGSM
    { 999.9, 999.9, 999.9, 999.9, 999.9, 999.9, 999.9,
    //sigmaF sigmaB sigmaBx sigmaBy sigmaBz
      999.9, 999.9, 999.9,  999.9,  999.9,
    //T          N      V       sigmaT     sigmaN sigmaV
      9999990.0, 999.9, 9999.0, 9999990.0, 999.9, 9999.0,
    //Dst    P1        P2        P4       P10      P30      P60
      99999, 99999.90, 99999.90, 9999.99, 9999.99, 9999.99, 9999.99};
  //The stream tokenizer.
  private StreamTokenizer streamTokenizer;
  //Date of one observation.
  private Date date;
  //Data for one observation.
  private double[] data;

  public BuildOMNI()
  {
    //Set up useField.
    useField = new boolean[43];
    for (int i=0;i<useField.length;i++)
      useField[i] = false;
    for (int i=0;i<fields.length;i++)
      useField[fields[i]-1] = true;
  }

  public static void main(String[] args)
    {
      BuildOMNI bo = new BuildOMNI();
      // First year of data.
      //  int year = 1963;
      int year = 1970;
      String urlName = bo.urlDir + "omni" + year + ".dat";

      //Keep the last data read so that there will
      //be overlap from one year to the next.
      OMNI omniOld = null;

      try {
	// Continue until no more files to read.
	while(bo.createStreamTokenizer(urlName))
	  {
	    System.out.println(year);
	    //Create a vector to hold OMNI objects.
	    Vector vec = new Vector(366*24);

	    //Put in last OMNI data.
	    if (omniOld != null)
	      vec.addElement(omniOld);

	    //Go through the tokens.
	    OMNI omni = null;
	    while (bo.getOneHourOfData())
	      {
		omni = new OMNI(bo.date,bo.data);
		System.out.println(omni.toString());
		vec.addElement(omni);
		//        vec.addElement(new OMNI(bo.date,bo.data));
	      }
            
            /* remove last NaNs */
            for(int i=vec.size()-1;i>-1;i--) {
                boolean allNaN = true;
                for(int j=0;j<((DatabaseObject) vec.elementAt(i)).getNumberOfDataIndexes();j++) {
                    if(!Double.isNaN(((DatabaseObject) vec.elementAt(i)).getData(j))) {
                        allNaN = false;
                        break;
                    }
                }
                if(allNaN) {
                    System.out.println("Will remove:"+((DatabaseObject) vec.elementAt(i)).getDate());
                    vec.remove(i);
                }
                else {
                    System.out.println("BREAK:"+((DatabaseObject) vec.elementAt(i)).getDate());
                    break;
                }                
            }

            vec.trimToSize();

	    //Update last omni.
	    omniOld = omni;

	    //Store in database if the vector contain data.
	    if (!vec.isEmpty())
	      {
		ObjectTimeSeries ts = new ObjectTimeSeries(vec);
		//Database db = Database.getInstance("OMNIDB",vec);
		Database db = Database.getInstance("OMNIDB");
		db.store(ts);
	      }

	    // Next year of data.
	    year++;
	    urlName = bo.urlDir + "omni" + year + ".dat";
	  }
      }
      catch (IllegalTimeSeriesException e) {
	System.err.println(e);
      }
      catch (Exception e) {
	System.err.println(e);
      }
    }

//  private String getUrlName(int y)
//{
//  return new String(urlDir + "omni" + y + ".dat");
//}

//  private boolean openURL(String urlName)
//  {
//  }

  private boolean createStreamTokenizer(String name)
  {
    //Reset field index.
    fieldIndex = -1;
    URL url = null;
    InputStream stream = null;
    try
    {
      // Create the URL to the data file.
      url = new URL(name);
      // Get the input stream.
      stream = url.openStream();
    }
    catch (MalformedURLException e)
    {
      System.err.println(e);
      return false;
    }
    catch (IOException e)
    {
      System.err.println(e);
      return false;
    }
    // Create buffered reader for the stream tokenizer.
    BufferedReader bufInput = new BufferedReader(new InputStreamReader(stream));
    // Create the stream tokenizer.
    streamTokenizer = new StreamTokenizer(bufInput);
    return true;
  }

//  private boolean hasMoreTokens()
//  {
//  }

  private boolean getOneHourOfData()
  {
    //The type of token.
    int type = streamTokenizer.TT_EOF;
    try
    {
      //Token 1. = year
      type = nextToken();
      if (type == streamTokenizer.TT_EOF)
        return false;
      else
      {
        readDate();
        readData();
        return true;
      }
    }
    catch (IOException e)
    {
      System.err.println(e);
      return false;
    }
  }

  private int nextToken() throws IOException
  {
    int type = streamTokenizer.nextToken();
    fieldIndex++;
    if (fieldIndex > maxFieldIndex)
      fieldIndex = 0;
    return type;
  }

  private void readDate() throws IOException
  {
    int year = (int) streamTokenizer.nval;
    //Token 2. = day of year.
    nextToken();
    int day = (int) streamTokenizer.nval;
    //Token 3. = hour of day.
    nextToken();
    int hour = (int) streamTokenizer.nval;
    //Get the date.
    Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
    //Clear all fields.
    cal.clear();
    //Set year, day of year, and hour of day.
    cal.set(cal.YEAR,year);
    cal.set(cal.DAY_OF_YEAR,day);
    cal.set(cal.HOUR_OF_DAY,hour);
    date = cal.getTime();
    //Reset data index.
    dataIndex = 0;
  }

  private void readData() throws IOException
  {
    data = new double[25];
    while (fieldIndex < maxFieldIndex)
    {
      nextToken();
      if (useField[fieldIndex])
      {
        data[dataIndex] = streamTokenizer.nval;
        if (data[dataIndex] == dataGapValues[dataIndex])
          data[dataIndex] = Double.NaN;
        dataIndex++;
      }
    }
  }
}
