package saaps.dbt;

import java.text.DateFormat;
import java.text.NumberFormat;

import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

import java.util.Vector;

import java.io.Serializable;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;

/**
* A class for the Dst index.
*
* The class contains the date and Dst in units of nT.
*
* @author Peter Wintoft
* @version 1.0 2001-03-02
*/
public class Dst extends DatabaseObject implements Serializable
{
  // Fields
  private static int numberOfIndexes = 2;
  private Date d;
  private double dst;
  private double status;
  
  // Constructors
  public Dst()
  {
  }

  /**
   * This constructor is used if you want an "empty" object with this date.
   */
  public Dst(Date D) {
    this.d = D;
    this.dst = Double.NaN;
    this.status = 1.0;
  }

  public Dst(Date D,double data) {
    this.d = D;
    this.dst = data;
    this.status = 1.0;
  }

  public Dst(Date D,double data,double status) {
    this.d = D;
    this.dst = data;
    this.status = status;
  }

  public Dst(long time,double data)
  {
    this(new Date(time),data);
  }

  public Dst(long time,double data,double status)
  {
    this(new Date(time),data,status);
  }

 /**
  * Construct a Dst object by giving the date and the two data field
  * values. The data fields are Dst and status.
  *
  *@param D The date.
  *@param data Array with the the two data field values.
  */
  public Dst(Date D, double[] data) {
    this.d = D;
    this.dst = data[0];
    this.status = data[1];
  }

  // Methods
  public double getDst()
  {
    return this.dst;
  }

  public double getStatus()
  {
    return this.status;
  }

  //Implemented for DatabaseObject
  public Date getDate()
  {
    return this.d;
  }

  // This method is the generic form of accessing data.
  public Vector getData()
  {
  	Vector vec = new Vector(numberOfIndexes);
  	vec.addElement(new Double(this.dst));
  	vec.addElement(new Double(this.status));
  	return vec;
  }
  /**
   * This method returns the size of the data that is to be written to file
   */
  public long getDataSize() throws IOException {

    ByteArrayOutputStream baos = new ByteArrayOutputStream();  
    DataOutputStream    dos  = new DataOutputStream(baos);  
    dos.writeLong(this.d.getTime());
    dos.writeDouble(this.dst);
    dos.writeDouble(this.status);
    dos.flush();  
    long size = baos.size();  // now you know how large ....
    return size;
  }

  public void setDate(Date D)
  {
    this.d = D;
  }
  public void setDate(long L)
  {
    this.d = new Date(L);
  }
  public void setDst(double dst)
  {
    this.dst = dst;
  }

  public void setStatus(double status) {
      this.status = status;
  }

  //Implemented for DatabaseObject getDate
  public int getNumberOfDataIndexes() {
    return (numberOfIndexes);
  }
     
  //Implemented for DatabaseObject 
  public double getData(int index) {
    //Date is not considered a data value
    switch (index) {
    case 0 :
      return(this.dst);
    case 1 :
      return(this.status);
    default :
      System.out.println("Index out of bounds for Dst");
      return (0.0);
    }
  }

  public String toString() {
//     // Set the date format.
//     DateFormat df = DateFormat.getDateTimeInstance(DateFormat.DEFAULT,DateFormat.DEFAULT,new Locale("sv","SE")); //en UK
//     df.setTimeZone(TimeZone.getTimeZone("UTC"));

    // Find the selected dates.
    Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
    cal.clear(cal.MILLISECOND);
    cal.setTime(d);

    NumberFormat nf = NumberFormat.getInstance();
    nf.setMinimumIntegerDigits(2);

    return ( cal.get(Calendar.YEAR) + " " + nf.format(cal.get(Calendar.MONTH)+1) + " " + nf.format(cal.get(Calendar.DAY_OF_MONTH)) + " " + nf.format(cal.get(Calendar.HOUR_OF_DAY)) + " " + nf.format(cal.get(Calendar.MINUTE)) + " " + nf.format(cal.get(Calendar.SECOND)) + " " + Double.toString(this.dst) + " " + Double.toString(this.status) );
//    return ( cal.get(Calendar.YEAR) + " " + nf.format(cal.get(Calendar.MONTH)+1) + " " + nf.format(cal.get(Calendar.DAY_OF_MONTH)) + " " + nf.format(cal.get(Calendar.HOUR_OF_DAY)) + " " + nf.format(cal.get(Calendar.MINUTE)) + " " + nf.format(cal.get(Calendar.SECOND)) + " " + Double.toString(getData()) );
    
  }

}
