package saaps.dbt;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;

import saaps.util.ObjectTimeSeries;
import saaps.util.IllegalTimeSeriesException;

/**
* LatestACEmag downloads the latest ACE Bx, By, Bz, Bt, latitude and longitude data from SEC
* and stores it in the database.
* FTP address: ftp://ftp.sel.noaa.gov/pub/lists/ace/ace_mag_1m.txt and
*	ftp://ftp.sel.noaa.gov/pub/lists/ace/ace_swepam_1m.txt
*
*@author Peter Wintoft
*@version 1.1 1999-12-21
*@version 1.0 990913
*/
public class LatestACEmag
{
	private static double dataGapValue = -999.9;
	public static void main(String[] args)
	{
		String ftpSrcStr="ftp://ftp.sel.noaa.gov/pub/lists/ace/ace_mag_1m.txt";
//		String ftpSrcStrSwe="ftp://ftp.sel.noaa.gov/pub/lists/ace/ace_swepam_1m.txt";
		DateFormat df;
		Date d;
		Vector acev = null;
		try
		{
			// Open connection
//			System.out.println("Open URL ...");
			URL url = new URL(ftpSrcStr);
			URLConnection urlConn = url.openConnection();
			// Input stream.
			BufferedReader streamIn = new BufferedReader(
			                            new InputStreamReader(
			                              urlConn.getInputStream()));
			String textLine = "";
			textLine = streamIn.readLine();
			// Go through the file for all lines starting with
			// "#", ":".
//			while (!textLine.startsWith("1") && !textLine.startsWith("2"))
			while (textLine.charAt(0)=='#' || textLine.charAt(0)==':')
			{
//				System.out.println("Skipping: " + textLine);
				textLine = streamIn.readLine();
			}
			// Create a Vector of ACE objects.
			acev = new Vector(120);
			while ((textLine = streamIn.readLine()) != null)
			{
//				System.out.println("Reading: " + textLine);
				// Tokenize the string.
				StringTokenizer st = new StringTokenizer(textLine);
				// Create the date string.
				String dateStr = st.nextToken() + "-" +		// Year
													st.nextToken() + "-" +	// Month
													st.nextToken() + " ";		// Day
				String timeStr = st.nextToken();					// Hour and minute
				// Insert a ":" between hour and minute.
				timeStr = timeStr.substring(0,2) + ":" + timeStr.substring(2,4);
				dateStr = dateStr + timeStr;
				// Read the date using - as the field separator.
//				String dateStr = textLine.substring(0,4) + "-" +
//				               		textLine.substring(5,7) + "-" +
//				               		textLine.substring(8,10) + " " +
//				               		textLine.substring(11,13) + ":" +
//				               		textLine.substring(13,15);
				// Use UTC time and get the Date object.
				df = DateFormat.getDateTimeInstance(DateFormat.SHORT,DateFormat.SHORT,new Locale("sv","SE"));
				df.setTimeZone(TimeZone.getTimeZone("UTC"));
				d = df.parse(dateStr);
				// Skip over until Bx field.
				st.nextToken();
				st.nextToken();
				st.nextToken();
				// Read Bx, By, Bz, Bt, latitude, and longitude.
				double bx = Double.valueOf(st.nextToken()).doubleValue();
				double by = Double.valueOf(st.nextToken()).doubleValue();
				double bz = Double.valueOf(st.nextToken()).doubleValue();
				double bt = Double.valueOf(st.nextToken()).doubleValue();
				double latitude = Double.valueOf(st.nextToken()).doubleValue();
				double longitude = Double.valueOf(st.nextToken()).doubleValue();
				// Replace data gaps with NaNs.
				if (bx == -999.9)
				  bx = Double.NaN;
				if (by == -999.9)
				  by = Double.NaN;
				if (bz == -999.9)
				  bz = Double.NaN;
				if (bt == -999.9)
				  bt = Double.NaN;
				if (latitude == -999.9)
				  latitude = Double.NaN;
				if (longitude == -999.9)
				  longitude = Double.NaN;
				// Add data to vector.
				acev.addElement(new ACEmag(d,bx,by,bz,bt,latitude,longitude));
			}
			// Trim the vector.
			acev.trimToSize();
			// Close input stream.
			streamIn.close();
			// Store the data in the database

			//Database db = Database.getInstance("ACEmagDB",acev);
			ObjectTimeSeries ts = new ObjectTimeSeries(acev);
			Database db = Database.getInstance("ACEmagDB");

			if(!db.isSet())
			  System.out.println("MetaData for ACEmagDB is unavailable \nWill try to create it.");
			db.store(ts);


		}
		catch (IOException e)
		{
			System.err.println("URL Connection error: " + e);
		}
		catch (NumberFormatException e)
		{
			System.err.println("Number format error: " + e);
		}
		catch (ParseException e)
		{
			System.err.println("Parse error: " + e);
		}
		catch (IllegalTimeSeriesException e) {
		  System.err.println(e);
		}
                catch (Exception e)
		{
			System.err.println(e);
		}

	}
}
