package saaps.dbt;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;

import saaps.dbt.*;
import saaps.util.ObjectTimeSeries;
import saaps.util.IllegalTimeSeriesException;

/**
* LatestPrelKp downloads the latest preliminary Kp from SEC
* and stores it in the SAAPS database.
* FTP address: ftp://ftp.sel.noaa.gov/pub/latest/DGD.txt
*
* The application uses the single parameter databas class
* DatabaseSP and its related classes.
*
*@author Peter Wintoft
*@version 2.1 Sets the status field to 2.
*@version 2.0 1999-09-22
*@version 1.0 990901
*/
public class LatestPrelKp
{
  private static double dataGapValue = -1.0;

	public static void main(String[] args)
	{
		String ftpSrcStr="ftp://ftp.sel.noaa.gov/pub/latest/DGD.txt";
		DateFormat df;
		Date d;
		try
		{
			// Open connection

		  //System.out.println("Will try to open connection");
		  URL url = new URL(ftpSrcStr);
		  URLConnection urlConn = url.openConnection();
		  // Input stream.
		  BufferedReader streamIn = new BufferedReader(
						  new InputStreamReader(
			                           urlConn.getInputStream()));
		  //System.out.println("Got input stream");
			String textLine = "";
			textLine = streamIn.readLine();
			//System.out.println("read first line");
			// Go through the file for all lines starting with
			// "#", ":", or " ".
			while (!textLine.startsWith("1") && !textLine.startsWith("2"))
			{
				textLine = streamIn.readLine();
				//System.out.println("read line");
			}
			// Create a Vector of Kp objects.
			Vector kpv = new Vector(8*30);
			//System.out.println("Made our vector");
			while ((textLine = streamIn.readLine()) != null)
			{
				// Read the date using - as the field separator.
				String dateStr = textLine.substring(0,4) + "-" +
				               		textLine.substring(5,7) + "-" +
				               		textLine.substring(8,10);
				df = DateFormat.getDateInstance(DateFormat.SHORT,new Locale("sv","SE"));
				df.setTimeZone(TimeZone.getTimeZone("UTC"));
				d = df.parse(dateStr);
				// Assume that observations are at 00, 03, 06, ...
				int h;
				Calendar cal = Calendar.getInstance();
				cal.setTimeZone(df.getTimeZone());
				for (int i=0;i<8;i++)
				{
					h=3*i;
					cal.setTime(d);
					cal.add(Calendar.HOUR,h);
					double tmpKp = Double.valueOf(textLine.substring(63+2*i,65+2*i).trim()).doubleValue();

					//System.out.println("Adding this:" + tmpKp);
					if (tmpKp == dataGapValue ) {
					  tmpKp = Double.NaN;
					}
					kpv.addElement(new Kp(cal.getTime().getTime(),tmpKp,2.0));
					// Integer.valueOf(textLine.substring(63+2*i,65+2*i).trim()).intValue()));
				}
			}
			// Trim the vector.
			kpv.trimToSize();
			// Remove from the end elements that have NaNs in the Kp field.
			boolean cont = true;
			while (cont) {
			    Kp lastKp = (Kp) kpv.lastElement();
			    if (Double.isNaN(lastKp.getKp())) {
			        kpv.removeElementAt(kpv.size()-1);
			    }
			    else {
			        cont = false;
			    }
			}
			// Close input stream.
			streamIn.close();
			// Store the data in the database
			// Database db = Database.getInstance("KpDB",kpv);
			//System.out.println("Closed stream");

			ObjectTimeSeries ts = new ObjectTimeSeries(kpv);

			Database db = Database.getInstance("KpDB");

			if(!db.isSet())
			  System.out.println("MetaData for Kp is unavailable \nWill try to create it.");
			
			//System.out.println("Got instance of DB will do store");
			db.store(ts);
			//System.out.println("Stored data, finished with this session");

		}
		catch (IOException e)
		{
			System.err.println("URL Connection error: " + e);
		}
		catch (NumberFormatException e)
		{
			System.err.println("Number format error: " + e);
		}
		catch (ParseException e)
		{
			System.err.println("Parse error: " + e);
		}
		catch (IllegalTimeSeriesException e) {
		  System.err.println(e);
		}
                catch (Exception e) {
                    System.err.println(e);
                }
        }
}
