package saaps.dbt;

import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;
import java.util.Vector;
import java.text.DateFormat;
import java.text.NumberFormat;

import java.io.Serializable;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.lang.Integer;

/**
* A class for OMNI objects.
*
* The class contains
* the date and OMNI data fields. There are methods to read
* and write date and the fields.
*
* @author Peter Wintoft
* @version 1.0 2000-04-14
*/
public class OMNI extends DatabaseObject implements Serializable
{
  // Fields
  private static int numberOfIndexes = 25;
  private Date d;
  private double[] data = new double[numberOfIndexes];
/*  private double F;
  private double B;
  private double Bx;
  private double ByGSE;
  private double BzGSE;
  private double ByGSM;
  private double BzGSM;
  private double sigmaF;
  private double sigmaB;
  private double sigmaBx;
  private double sigmaBy;
  private double sigmaBz;
  private double T;
  private double N;
  private double V;
  private double sigmaT;
  private double sigmaN;
  private double sigmaV;
  private double Dst;
  private double P1;
  private double P2;
  private double P4;
  private double P10;
  private double P30;
  private double P60; */

  // Constructors
  public OMNI()
  {
  }
  public OMNI(Date D, double[] DATA)
  {
    d = D;
    data = DATA;
  }
  public OMNI(long L, double[] DATA)  // 990718
  {
    d = new Date(L);
    data = DATA;
  }

  //Implemented for DatabaseObject getDate
  public int getNumberOfDataIndexes() {
    return (numberOfIndexes);
  }
  //Implemented for DatabaseObject getDate
  public Date getDate()
  {
    return d;
  }

  public Vector getData()
  {
    Vector vec = new Vector(numberOfIndexes);
    for (int i=0;i<numberOfIndexes;i++)
      vec.addElement(new Double(data[i]));
    return vec;
  }

  //Implemented for DatabaseObject getData
  public double getData(int index) {
    return data[index];
/*    switch (index) { //No breks allowed due to return statement
    case 0 :
      return(kp);
    default :
      System.out.println("Index out of bounds for Kp");
      return (0.0);
    }   */
  }

  /**
   * This method returns the size of the data that is to be written to file
   */
  public long getDataSize() throws IOException {

    ByteArrayOutputStream baos = new ByteArrayOutputStream();  
    DataOutputStream    dos  = new DataOutputStream(baos);  

    dos.writeLong(d.getTime());
    for (int i=0;i<numberOfIndexes;i++)
      dos.writeDouble(data[i]);  

    dos.flush();  
    long size = baos.size();  // now you know how large ....
    return size;
  }

  public void setDate(Date D)
  {
    d = D;
  }
  public void setDate(long L)  // 990718
  {
    d = new Date(L);
  }
  public void setData(double[] DATA)
  {
    data = DATA;
  }
  public void setData(double DATA, int index)
  {
    data[index] = DATA;
  }

  public String toString() {
    // Set the date format.
//     DateFormat df = DateFormat.getDateTimeInstance(DateFormat.DEFAULT,DateFormat.DEFAULT,new Locale("sv","SE"));
//     df.setTimeZone(TimeZone.getTimeZone("UTC"));

    // Find the selected dates.
    Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
    cal.clear(cal.MILLISECOND);
    cal.setTime(d);

    NumberFormat nf = NumberFormat.getInstance();
    nf.setMinimumIntegerDigits(2);

    StringBuffer sb = new StringBuffer(cal.get(Calendar.YEAR) + " " + nf.format(cal.get(Calendar.MONTH)+1) + " " + nf.format(cal.get(Calendar.DAY_OF_MONTH)) + " " + nf.format(cal.get(Calendar.HOUR_OF_DAY)) + " " + nf.format(cal.get(Calendar.MINUTE)) + " " + nf.format(cal.get(Calendar.SECOND)));
    for (int i=0;i<numberOfIndexes;i++)
      sb.append(" " + Double.toString(data[i]));

    return ( sb.toString() );

  }

}
