package saaps.network;

import Jama.Matrix;
import saaps.math.Math;

/**
The Layer class consist of a collection of Neuron objects.

@author Peter Wintoft
@version 2000-01-18
*/

public class Layer
{
	// Array of neurons.
	private Neuron[] neuron;
	// The output from the layer.
	private Matrix output;
	
	/**
	A layer is created from the weight matrix, the bias matrix
	and the activation function. All neurons in the layer have
	the same activation functions.
	
	@param w weight matrix
	@param b bias matrix
	@param af activation function
	*/
	public Layer(Matrix w, Matrix b, int af)
	{
		neuron = new Neuron[w.getRowDimension()];
		for (int i=0;i<w.getRowDimension();i++)
		{
			neuron[i] = 
				new Neuron(w.getMatrix(i,i,0,w.getColumnDimension()-1),b.get(i,0),af);
		}
	}
	
	/**
	A layer is created from the weight matrix and the bias matrix
	using the default activation function (TANSIG). All neurons
	in the layer have the same activation functions.
	
	@param w weight matrix
	@param b bias matrix
	*/
	public Layer(Matrix w, Matrix b)
	{
		this(w,b,Neuron.TANSIG);
	}
	
	/**
	Calculate the output from the layer.
	
	@param input the input matrix 
	*/
	public Matrix sim(Matrix input)
	{
		output = new Matrix(neuron.length,input.getColumnDimension());
		for (int i=0;i<output.getRowDimension();i++)
		{
			output.setMatrix(i,i,0,input.getColumnDimension()-1,neuron[i].sim(input));
		}
		return output;
	}
	
	/**
	Get the number of neurons in the layer.
	*/
	public int getNumOfNeurons()
	{
		return neuron.length;
	}
}