package saaps.network;

import Jama.Matrix;
import saaps.math.Math;

/**
The Network class consist of one or several Layer objects.

@author Peter Wintoft
@version 2000-01-19
*/
public class Network
{
	// An array of layers.
	private Layer[] layer;
	// The output from the network.
	private Matrix output;
	
	/**
	Construct a network from a NetworkParam object.
	
	@param np the NetworkParam object
	*/
	public Network(NetworkParam np)
	{
		int l = np.getNumOfLayers();
		Matrix[] w = np.getWeight();
		Matrix[] b = np.getBias();
		int[] af = np.getActFun();
		layer = new Layer[l];
		for (int i=0;i<l;i++)
		{
			layer[i] = new Layer(w[i],b[i],af[i]);
		}
	}

	/**
	Construct a one-layer network.
	<p>
	May be removed in the future.
	
	@param w1 weights
	@param b1 bias
	@param f1 activation function
	*/
	public Network(Matrix w1, Matrix b1, int f1)
	{
		layer = new Layer[1];
		layer[0] = new Layer(w1,b1,f1);
	}

	/**
	Construct a one-layer network using the default activation
	function (TANSIG).
	<p>
	May be removed in the future.
	
	@param w1 weights
	@param b1 bias
	*/
	public Network(Matrix w1, Matrix b1)
	{
		this(w1,b1,Neuron.TANSIG);
	}

	/**
	Construct a two-layer network.
	<p>
	May be removed in the future.
	
	@param w1 layer one weights
	@param b1 layer one bias
	@param f1 layer one activation function
	@param w2 layer two weights
	@param b2 layer two bias
	@param f2 layer two activation function
	*/
	public Network(Matrix w1, Matrix b1, int f1, Matrix w2, Matrix b2, int f2)
	{
		layer = new Layer[2];
		layer[0] = new Layer(w1,b1,f1);
		layer[1] = new Layer(w2,b2,f2);
	}

	/**
	Construct a two-layer network using the default activation
	function (TANSIG).
	<p>
	May be removed in the future.
	
	@param w1 layer one weights
	@param b1 layer one bias
	@param w2 layer two weights
	@param b2 layer two bias
	*/
	public Network(Matrix w1, Matrix b1, Matrix w2, Matrix b2)
	{
		this(w1,b1,Neuron.TANSIG,w2,b2,Neuron.TANSIG);
	}

	/**
	Run the network from an input matrix. The output will be
	a Matrix object with the number of rows equal to the number
	of outputs and the number of columns equal to the number
	of columns in the input matrix.
	
	@param input the input matrix
	*/	
	public Matrix sim(Matrix input)
	{
		output = new Matrix(layer[0].getNumOfNeurons(),input.getColumnDimension());
		Matrix inp = input;
		for (int i=0;i<layer.length;i++)
		{
			output = layer[i].sim(inp);
			inp = output;
		}
		return output;
	}

	/**
	Get the output from the network.
	*/
	public Matrix getOutput()
	{
		return output;
	}
}