package saaps.network;

import java.util.Date;
import saaps.dbt.Database;
import saaps.util.TimeSeries;
import Jama.Matrix;

/**

@author Peter Wintoft
@version 2000-02-15
*/
public class NetworkModel
{
	Network net;
	NormParam normPar;
	
	/**
	Initialize the network from the NetworkParam and
	NormParam objects.
	
	@param netp Network description.
	@param normp Normalization description.
	*/
	public NetworkModel(NetworkParam netp, NormParam normp)
	{
		net = new Network(netp);
		normPar = normp;
	}
	
	/**
	Run the NetworkModel on a TimeSeries object.
	
	@param ts A time series.
	*/
	public TimeSeries sim(TimeSeries ts)
	{
		// Create daily sums.
		TimeSeries tsSum = ts.dailySum();
		
		// Get the dates.
		Date[] dates = tsSum.getDates();
		for (int i=0;i<dates.length;i++)
			System.out.println("SumKpNetwork: dates(" + i + ")=" + dates[i]);
		
		// Normalize and create time delay line.
		Matrix input = tsSum.normalize(normPar).timeDelay(normPar);
		
		// Run network
		input.print(3,1);//today
		Matrix output = net.sim(input).transpose();
		output.print(3,1);
		
		//Return result.
		return new TimeSeries(dates,output);
	}
	
}