package saaps.network;

import Jama.Matrix;
import java.io.RandomAccessFile;
import java.io.IOException;
import java.io.Serializable;
//import java.lang.String;

/**
A NetworkParam object contain the weights, biases and
activation functions for a feedforward network. The
object is created by reading the data from a file.

@author Peter Wintoft
@version 2000-01-21
*/
public class NetworkParam implements Serializable
{
  private int numOfLayers;
  private Matrix[] weight;
  private Matrix[] bias;
  private int[] actFun;

  public NetworkParam()
  {
  }

  public NetworkParam(String fileName)
  {
    readParam(fileName);
  }

  private void readParam(String fileName)
  {
    RandomAccessFile raf;
    try
    {
      int rows;
      int cols;
      raf = new RandomAccessFile("Models/" + fileName,"r");
      numOfLayers = raf.readInt();
      System.out.println("NetworkParam.numOfLayers=" + numOfLayers);
      weight = new Matrix[numOfLayers];
      bias = new Matrix[numOfLayers];
      actFun = new int[numOfLayers];
      for (int i=0;i<numOfLayers;i++)
      {
        // Weight matrix.
        rows = raf.readInt();
        cols = raf.readInt();
        weight[i] = new Matrix(rows,cols);
        for (int c=0;c<cols;c++)
        {
          for (int r=0;r<rows;r++)
          {
            weight[i].set(r,c,raf.readDouble());
          }
        }
        // Bias matrix.
        bias[i] = new Matrix(rows,1);
        for (int r=0;r<rows;r++)
        {
          bias[i].set(r,0,raf.readDouble());
        }
        // Activation function.
/*        String afStr = raf.readUTF();
        System.out.println("afStr=" + afStr);
        if (afStr.equals("purelin"))
          actFun[i] = Neuron.PURELIN;
        if (afStr.equals("logsig"))
          actFun[i] = Neuron.LOGSIG;
        if (afStr.equals("tansig"))
          actFun[i] = Neuron.TANSIG;
          */
        actFun[i] = raf.readInt();
      }
      raf.close();
    }
    catch (IOException e)
    {
      System.err.println(e);
    }
  }

  public int getNumOfInputs()
  {
    return weight[0].getColumnDimension();
  }
  public int getNumOfLayers()
  {
    return numOfLayers;
  }
  
  public void setNumOfLayers(int n)
  {
    numOfLayers = n;
  }
  
  public Matrix[] getWeight()
  {
    return weight;
  }

  public void setWeight(Matrix[] m)
  {
    weight = m;
  }

  public Matrix[] getBias()
  {
    return bias;
  }

  public void setBias(Matrix[] b)
  {
    bias = b;
  }

  public int[] getActFun()
  {
    return actFun;
  }

  public void setActFun(int[] a)
  {
    actFun = a;
  }
}
