/*
 * AnomalyPlotterFieldData.java
 *
 * Created on August 20, 2001, 4:29 PM
 */

package saaps.saam;

import java.util.Calendar;
import java.util.Date;
import java.util.StringTokenizer;
import java.util.TimeZone;
import java.util.Vector;

import saaps.util.TimeSeries;

/**
 *
 * @author  root
 * @version 
 */
public class AnomalyPlotterFieldData extends java.lang.Object {

    //The field types.
    private static int YEAR = 0;
    private static int MONTH = 1;
    private static int DAY = 2;
    private static int HOUR = 3;
    private static int MINUTE = 4;
    private static int LOCALTIME = 5;
    private static int RADIUS = 6;
    private static int LONGITUDE = 7;
    private static int LATITUDE = 8;
    private String[] typeString = {"Year","Month","Day","Hour","Minute","Local_Time","Radius","Longitude","Latitude"};
        
    private int[] mapFieldToType;
    private int[] mapTypeToField;
            
    //The anomaly data.
    //The first array index corresponds to the rows.
    //The second array index corresponds to the fields.
    private double[][] anomalyData;
    
    //The anomaly data string.
    private String anomalyDataString = "";
    
    //A calendar object for time calculations.
    private Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
        
    public AnomalyPlotterFieldData() {
        
        this.mapFieldToType = null;

        this.mapTypeToField = new int[this.typeString.length];
        for (int i=0;i<this.mapTypeToField.length;i++) {
            this.mapTypeToField[i] = -1;
        }
    }
    
    public void setFieldToTypeMap(String[] map) {
        
        for (int i=0;i<this.mapFieldToType.length;i++) {
            this.mapFieldToType[i] = -1;
        }
        for (int i=0;i<this.mapTypeToField.length;i++) {
            this.mapTypeToField[i] = -1;
        }
        
        if (map != null) {
            for (int i=0;i<map.length;i++) {
                
                StringTokenizer st = new StringTokenizer(map[i]);
                int field = Integer.valueOf(st.nextToken()).intValue() - 1;

                String typeStr = st.nextToken();
                int type = -1;
                while (!typeStr.equalsIgnoreCase(this.typeString[++type]));
                
                this.mapFieldToType[field] = type;
                this.mapTypeToField[type] = field;
            }
        }
    }
    
    public String[] getFields() {
        String[] s = new String[this.mapFieldToType.length];
        for (int i=0;i<s.length;i++) {
            s[i] = Integer.toString(i+1);
        }
        return s;
    }
    
    public String[] getFieldNotUsed() {
        String[] s = null;
        
        int sLength = 0;
        for(int i=0;i<this.mapFieldToType.length;i++) {
            if (this.mapFieldToType[i] == -1) sLength++;
        }
        
        if (sLength>0) {
            s = new String[sLength];
            int j = 0;
            for (int i=0;i<this.mapFieldToType.length;i++) {
                if (this.mapFieldToType[i]==-1) {
                    s[j++] = Integer.toString(i+1);
                }
            }
        }
                        
        return s;
    }
    
    public String[] getTypeNotUsed() {
        String[] s = null;
        
        int sLength = 0;
        for(int i=0;i<this.mapTypeToField.length;i++) {
            if (this.mapTypeToField[i] == -1) sLength++;
        }
        
        if (sLength>0) {
            s = new String[sLength];
            int j = 0;
            for (int i=0;i<this.mapTypeToField.length;i++) {
                if (this.mapTypeToField[i]==-1) {
                    s[j++] = this.typeString[i];
                }
            }
        }
                
        return s;
    }
    
    public String[] getTypeUsed() {
        String[] s = null;
        
        int sLength = 0;
        for(int i=0;i<this.mapTypeToField.length;i++) {
            if (this.mapTypeToField[i] > -1) sLength++;
        }
        
        if (sLength>0) {
            s = new String[sLength];
            int j = 0;
            for (int i=0;i<this.mapTypeToField.length;i++) {
                if (this.mapTypeToField[i]>-1) {
                    s[j++] = this.typeString[i];
                }
            }
        }
        
        return s;
    }
    
    public String[] getTypes() {
        return this.typeString;
    }
    
    public String[] getMapFieldToType() {
        String[] s = null;
        
        int sLength = 0;
        for (int i=0;i<this.mapFieldToType.length;i++) {
            if (this.mapFieldToType[i]>-1) sLength++;
        }
        
        if (sLength>0) {
            s = new String[sLength];
            int j = 0;
            for (int field=0;field<this.mapFieldToType.length;field++) {
                if (this.mapFieldToType[field]>-1) {
                    s[j] = Integer.toString(field+1) + " ";
                    s[j++] += this.typeString[this.mapFieldToType[field]];
                }
            }
        }
        
        return s;
    }
    
    public String getDataString() {
        return this.anomalyDataString;
    }
    
    public void setDataString(String s) {
        this.anomalyDataString = s;
    }
    
    public String getSampleLine() {
        String s = "";
        for (int i=0;i<this.anomalyData[0].length;i++) {
            if (Double.isNaN(this.anomalyData[0][i])) {
                s += "NaN ";
            }
            else {
                if (Math.rint(this.anomalyData[0][i]) == this.anomalyData[0][i]) {
                    s += Integer.toString((int) this.anomalyData[0][i]) + " ";
                }
                else {
                    s += Double.toString(this.anomalyData[0][i]) + " ";
                }
            }
        }
        return s;
    }
        
    public void parseData() {
        //Tokenize with respect to newlines.
        StringTokenizer st = new StringTokenizer(anomalyDataString,"\n");
        //Number of rows.
        int numOfRows = st.countTokens();
        //Tokenize one row.
        StringTokenizer stRow = new StringTokenizer(st.nextToken());
        //Number of columns.
        int numOfCols = stRow.countTokens();
        //Init data.
        this.anomalyData = new double[numOfRows][numOfCols];
        this.mapFieldToType = new int[numOfCols];
        //Read the first line of data.
        for (int col=0;col<numOfCols;col++) {
            try {
                this.anomalyData[0][col] = Double.valueOf(stRow.nextToken()).doubleValue();
            }
            catch (NumberFormatException e) {
                this.anomalyData[0][col] = Double.NaN;
            }
            this.mapFieldToType[col] = -1;
        }
        //Read the rest of the data.
        for (int row=1;row<numOfRows;row++) {
            stRow = new StringTokenizer(st.nextToken());
            for (int col=0;col<numOfCols;col++) {
                try {
                    this.anomalyData[row][col] = Double.valueOf(stRow.nextToken()).doubleValue();
                }
                catch (NumberFormatException e) {
                    this.anomalyData[0][col] = Double.NaN;
                }
            }
        }
    }
        
    public boolean isAllSet() {
        return (this.anomalyData != null & this.mapFieldToType != null);
    }
    
    public TimeSeries getLTLongTS() {
        //Data field names.
        String[] fieldName = {"Local Time","Longitude"};
        //Data field units.
        String[] fieldUnit = {"Hour","Degrees East"};
        //Create some dummy date. We do not need the dates.
        Date[] date = new Date[this.anomalyData.length];
        //Init the data.
        double[][] data = new double[this.anomalyData.length][2];
        //The time series.
        TimeSeries ts = null;
        //True if year, month, and day are present.
        boolean dateExist = (this.mapTypeToField[this.YEAR]>-1) &
                            (this.mapTypeToField[this.MONTH]>-1) &
                            (this.mapTypeToField[this.DAY]>-1);
        //True if local time exist.
        boolean ltExist = (this.mapTypeToField[this.LOCALTIME]>-1);
        //True if longitude exist.
        boolean longExist = (this.mapTypeToField[this.LONGITUDE]>-1);
        //True if hour exist.
        boolean hourExist = (this.mapTypeToField[this.HOUR]>-1);
        //Both local time and longitude exist.
        if (ltExist & longExist) {
            for (int i=0;i<data.length;i++) {
                //Local time.
                data[i][0] = this.anomalyData[i][this.mapTypeToField[this.LOCALTIME]];
                //Longitude.
                data[i][1] = this.anomalyData[i][this.mapTypeToField[this.LONGITUDE]];
            }
            ts = new TimeSeries(date,data,fieldName,fieldUnit);
        }
        //Local time does not exist, but date, hour, and longitude exist.
        if (!ltExist & dateExist & hourExist & longExist) {
            double lt;
            for (int i=0;i<data.length;i++) {
                //Hour
                lt = this.anomalyData[i][this.mapTypeToField[this.HOUR]];
                //Minute
                if (this.mapTypeToField[this.MINUTE] > -1) {
                    lt += this.anomalyData[i][this.mapTypeToField[this.MINUTE]] / 60.0;
                }
                //Get the local time.
                lt += this.anomalyData[i][this.mapTypeToField[this.LONGITUDE]]*24.0/360.0;
                //Make sure that 0 <= lt < 24.
                if (lt < 0.0) {
                    int n = (int) (-lt/24.0);
                    lt += (n+1)*24.0;
                }
                if (lt >= 24.0) {
                    int n = (int) (lt/24.0);
                    lt -= n*24.0;
                }
                data[i][0] = lt;
                //Longitude
                data[i][1] = this.anomalyData[i][this.mapTypeToField[this.LONGITUDE]];
            }
            ts = new TimeSeries(date,data,fieldName,fieldUnit);
        }
        return ts;
    }
    
    public TimeSeries getLTLatTS() {
        //Data field names.
        String[] fieldName = {"Local Time","Latitude"};
        //Data field units.
        String[] fieldUnit = {"Hour","Degrees North"};
        //Create some dummy date. We do not need the dates.
        Date[] date = new Date[this.anomalyData.length];
        //Init the data.
        double[][] data = new double[this.anomalyData.length][2];
        //The time series.
        TimeSeries ts = null;
        //True if year, month, and day are present.
        boolean dateExist = (this.mapTypeToField[this.YEAR]>-1) &
                            (this.mapTypeToField[this.MONTH]>-1) &
                            (this.mapTypeToField[this.DAY]>-1);
        //True if local time exist.
        boolean ltExist = (this.mapTypeToField[this.LOCALTIME]>-1);
        //True if longitude exist.
        boolean longExist = (this.mapTypeToField[this.LONGITUDE]>-1);
        //True if latitude exist.
        boolean latExist = (this.mapTypeToField[this.LATITUDE]>-1);
        //True if hour exist.
        boolean hourExist = (this.mapTypeToField[this.HOUR]>-1);
        //Both local time and latitude exist.
        if (ltExist & latExist) {
            for (int i=0;i<data.length;i++) {
                //Local time.
                data[i][0] = this.anomalyData[i][this.mapTypeToField[this.LOCALTIME]];
                //Longitude.
                data[i][1] = this.anomalyData[i][this.mapTypeToField[this.LATITUDE]];
            }
            ts = new TimeSeries(date,data,fieldName,fieldUnit);
        }
        //Local time does not exist, but date, hour, and longitude exist.
        if (!ltExist & dateExist & hourExist & longExist & latExist) {
            double lt;
            for (int i=0;i<data.length;i++) {
                //Hour
                lt = this.anomalyData[i][this.mapTypeToField[this.HOUR]];
                //Minute
                if (this.mapTypeToField[this.MINUTE] > -1) {
                    lt += this.anomalyData[i][this.mapTypeToField[this.MINUTE]] / 60.0;
                }
                //Get the local time.
                lt += this.anomalyData[i][this.mapTypeToField[this.LONGITUDE]]*24.0/360.0;
                //Make sure that 0 <= lt < 24.
                if (lt < 0.0) {
                    int n = (int) (-lt/24.0);
                    lt += (n+1)*24.0;
                }
                if (lt >= 24.0) {
                    int n = (int) (lt/24.0);
                    lt -= n*24.0;
                }
                data[i][0] = lt;
                //Latitude
                data[i][1] = this.anomalyData[i][this.mapTypeToField[this.LATITUDE]];
            }
            ts = new TimeSeries(date,data,fieldName,fieldUnit);
        }
        return ts;
    }
    
    public TimeSeries getLocalTimeTS() {
        //Data field name.
        String fieldName = "Local Time";
        //Data field unit.
        String fieldUnit = "Hour";
        //Init the date and data.
        Date[] date = new Date[this.anomalyData.length];
        double[] data = new double[date.length];
        TimeSeries ts = null;
        //True if year, month, and day are present.
        boolean dateExist = (this.mapTypeToField[this.YEAR]>-1) &
                            (this.mapTypeToField[this.MONTH]>-1) &
                            (this.mapTypeToField[this.DAY]>-1);
        boolean ltExist = (this.mapTypeToField[this.LOCALTIME]>-1);
        boolean longExist = (this.mapTypeToField[this.LONGITUDE]>-1);
        boolean hourExist = (this.mapTypeToField[this.HOUR]>-1);
        if (dateExist & ltExist) {
            //Clear the time.
            cal.clear();
            int t;
            for (int i=0;i<date.length;i++) {
                //Get the time.
                //Year
                t = (int) this.anomalyData[i][this.mapTypeToField[this.YEAR]];
                cal.set(Calendar.YEAR,t);
                //Month
                t = (int) this.anomalyData[i][this.mapTypeToField[this.MONTH]];
                cal.set(Calendar.MONTH,t);
                //Day
                t = (int) this.anomalyData[i][this.mapTypeToField[this.DAY]];
                cal.set(Calendar.DATE,t);
                //Hour
                if (this.mapTypeToField[this.HOUR] > -1) {
                    t = (int) this.anomalyData[i][this.mapTypeToField[this.HOUR]];
                    cal.set(Calendar.HOUR_OF_DAY,t);
                }
                //Minute
                if (this.mapTypeToField[this.MINUTE] > -1) {
                    t = (int) this.anomalyData[i][this.mapTypeToField[this.MINUTE]];
                    cal.set(Calendar.MINUTE,t);
                }
                date[i] = cal.getTime();
                //Get the local time.
                data[i] = this.anomalyData[i][this.mapTypeToField[this.LOCALTIME]];
            }
            ts = new TimeSeries(date,data,fieldName,fieldUnit);
        }
        //If the local time field is not present check
        //if hour and longitude fields are present.
        if ( dateExist & !ltExist & longExist & hourExist) {
            //Clear the time.
            cal.clear();
            int t;
            double lt;
            for (int i=0;i<date.length;i++) {
                //Get the time.
                //Year
                t = (int) this.anomalyData[i][this.mapTypeToField[this.YEAR]];
                cal.set(Calendar.YEAR,t);
                //Month
                t = (int) this.anomalyData[i][this.mapTypeToField[this.MONTH]];
                cal.set(Calendar.MONTH,t);
                //Day
                t = (int) this.anomalyData[i][this.mapTypeToField[this.DAY]];
                cal.set(Calendar.DATE,t);
                //Hour
                t = (int) this.anomalyData[i][this.mapTypeToField[this.HOUR]];
                cal.set(Calendar.HOUR_OF_DAY,t);
                lt = t;
                //Minute
                if (this.mapTypeToField[this.MINUTE] > -1) {
                    t = (int) this.anomalyData[i][this.mapTypeToField[this.MINUTE]];
                    cal.set(Calendar.MINUTE,t);
                    lt += t/60.0;
                }
                date[i] = cal.getTime();
                //Get the local time.
                lt += this.anomalyData[i][this.mapTypeToField[this.LONGITUDE]]*24.0/360.0;
                //Make sure that 0 <= lt < 24.
                if (lt < 0.0) {
                    int n = (int) (-lt/24.0);
                    lt += (n+1)*24.0;
                }
                if (lt >= 24.0) {
                    int n = (int) (lt/24.0);
                    lt -= n*24.0;
                }
                data[i] = lt;
            }
            ts = new TimeSeries(date,data,fieldName,fieldUnit);
        }
        return ts;
    }
    
    public TimeSeries getLocalTimeHistogram() {
        //Data field names.
        String[] fieldName = {"Local Time","Number of events"};
        //Data field units.
        String[] fieldUnit = {"Hour",""};
        //Create some dummy date.
        Date[] date = new Date[24];
        //The time series.
        TimeSeries ts = null;
        //Create the data and set the local time field.
        double[][] data = new double[24][2];
        for (int lt=0;lt<data.length;lt++) {
            data[lt][0] = lt;
        }
        //Is the local time field present?
        if (this.mapTypeToField[this.LOCALTIME] > -1) {
            //Extract the data.
            for (int i=0;i<this.anomalyData.length;i++) {
                int lt = (int) Math.floor(this.anomalyData[i][this.mapTypeToField[this.LOCALTIME]]);
                data[lt][1]++;
            }
            ts = new TimeSeries(date,data,fieldName,fieldUnit);
        }
        //If the local time field is not present check
        //if hour and longitude fields are present.
        if ( this.mapTypeToField[this.LOCALTIME] == -1 &
        this.mapTypeToField[this.LONGITUDE] > -1 &
        this.mapTypeToField[this.HOUR] > -1 ) {
            //Extract the data.
            for (int i=0;i<this.anomalyData.length;i++) {
                int lt = (int) Math.floor(this.anomalyData[i][this.mapTypeToField[this.HOUR]] + this.anomalyData[i][this.mapTypeToField[this.LONGITUDE]] * 24.0 / 360.0);
                //Make sure that 0 <= lt < 24.
                if (lt < 0) {
                    int n = -lt/24;
                    lt += (n+1)*24;
                }
                if (lt > 23) {
                    int n = lt/24;
                    lt -= n*24;
                }
                data[lt][1]++;
            }
            ts = new TimeSeries(date,data,fieldName,fieldUnit);
        }
        //Return the time series object.
        return ts;
    }
    
    /**
    * @param args the command line arguments
    */
    public static void main (String args[]) {
    }

}
