package saaps.saam;

import java.applet.AppletContext;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.util.Date;
import java.util.Calendar;
import java.util.TimeZone;
import java.util.Vector;
import javax.swing.*;
import saaps.dbt.DatabaseRMI;
import saaps.network.NetworkParam;
import saaps.network.NormParam;
import saaps.util.TimeSeries;
import saaps.util.YMDJComboBox;

/**
 * This applet plots the data in the SAAPS database.
 * It lists the database objects from which the user
 * can select an object. The start and end date choices
 * are initiated based on the chosen database object.
 * After the user select the dates the data can be loaded
 * to the applet. The data field that should be plotted on
 * the x-axis is selected, and one or more fields for the
 * y-axis are slected. The plot symbol can be chosen and
 * when the plot button is pushed the plot is produced.
 *
 * @author  Peter Wintoft
 * @version 2000-02-18
 */
public class DataPlotterApplet extends java.applet.Applet {

  private TimeSeries ts;
  /** Initializes the Form */
  public void init () {
    initComponents ();
    //Initialize the combo boxes.
    for (int i=1970;i<2010;i++)
    {
      jComboBoxSY.addItem(new Integer(i));
      jComboBoxEY.addItem(new Integer(i));
    }
    for (int i=1;i<12;i++)
    {
      jComboBoxSM.addItem(new Integer(i));
      jComboBoxEM.addItem(new Integer(i));
    }
    for (int i=1;i<31;i++)
    {
      jComboBoxSD.addItem(new Integer(i));
      jComboBoxED.addItem(new Integer(i));
    }
    //Get the database objects list.
    DatabaseRMI dbrmi = null;
    Vector dbList = null;
    try
    {
      dbrmi = (DatabaseRMI) Naming.lookup("//" + getCodeBase().getHost() +
              ":2001" + "/DatabaseRMIServer");
      dbList = dbrmi.getDatabaseList();
    }
    catch(NotBoundException nbe)
    {
      System.err.println(nbe);
    }
    catch(MalformedURLException mfurle)
    {
      System.err.println(mfurle);
    }
    catch(RemoteException re)
    {
      System.err.println(re);
    }
    //Show the database list.
    jListDatabase.setListData(dbList);
//    for (int i=0;i<dbList.size();i++)
//      jListDatabase.add(dbList.elementAt(i));
  }

  /** This method is called from within the init() method to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the FormEditor.
   */
  private void initComponents () {//GEN-BEGIN:initComponents
    setLayout (new java.awt.GridBagLayout ());
    java.awt.GridBagConstraints gridBagConstraints1;

    jLabelTitle = new javax.swing.JLabel ();
    jLabelTitle.setText ("Data Plotter");
    jLabelTitle.setFont (new java.awt.Font ("SansSerif", 1, 18));
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 12;
    gridBagConstraints1.gridy = 0;
    gridBagConstraints1.gridwidth = 9;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.insets = new java.awt.Insets (10, 30, 0, 0);
    add (jLabelTitle, gridBagConstraints1);

    jListDatabase = new javax.swing.JList ();
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 0;
    gridBagConstraints1.gridy = 3;
    gridBagConstraints1.gridwidth = 10;
    gridBagConstraints1.gridheight = 2;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.ipadx = 100;
    gridBagConstraints1.ipady = 120;
    gridBagConstraints1.insets = new java.awt.Insets (3, 30, 0, 0);
    add (jListDatabase, gridBagConstraints1);

    jLabelStart = new javax.swing.JLabel ();
    jLabelStart.setText ("Start");
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 0;
    gridBagConstraints1.gridy = 6;
    gridBagConstraints1.gridwidth = 2;
    gridBagConstraints1.gridheight = 2;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.ipadx = 36;
    gridBagConstraints1.ipady = 3;
    gridBagConstraints1.insets = new java.awt.Insets (20, 30, 0, 0);
    add (jLabelStart, gridBagConstraints1);

    jComboBoxSY = new javax.swing.JComboBox ();
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 0;
    gridBagConstraints1.gridy = 8;
    gridBagConstraints1.gridwidth = 3;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.ipadx = -50;
    gridBagConstraints1.ipady = -7;
    gridBagConstraints1.insets = new java.awt.Insets (0, 30, 0, 0);
    add (jComboBoxSY, gridBagConstraints1);

    jComboBoxSM = new javax.swing.JComboBox ();
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 0;
    gridBagConstraints1.gridy = 9;
    gridBagConstraints1.gridwidth = 3;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.ipadx = -50;
    gridBagConstraints1.ipady = -7;
    gridBagConstraints1.insets = new java.awt.Insets (10, 30, 0, 0);
    add (jComboBoxSM, gridBagConstraints1);

    jComboBoxSD = new javax.swing.JComboBox ();
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 0;
    gridBagConstraints1.gridy = 10;
    gridBagConstraints1.gridwidth = 3;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.ipadx = -50;
    gridBagConstraints1.ipady = -7;
    gridBagConstraints1.insets = new java.awt.Insets (10, 30, 0, 0);
    add (jComboBoxSD, gridBagConstraints1);

    jLabel1 = new javax.swing.JLabel ();
    jLabel1.setText ("End");
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 0;
    gridBagConstraints1.gridy = 12;
    gridBagConstraints1.gridheight = 2;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.insets = new java.awt.Insets (10, 30, 0, 0);
    add (jLabel1, gridBagConstraints1);

    jComboBoxEY = new javax.swing.JComboBox ();
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 0;
    gridBagConstraints1.gridy = 14;
    gridBagConstraints1.gridwidth = 3;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.ipadx = -50;
    gridBagConstraints1.ipady = -7;
    gridBagConstraints1.insets = new java.awt.Insets (3, 30, 0, 0);
    add (jComboBoxEY, gridBagConstraints1);

    jComboBoxEM = new javax.swing.JComboBox ();
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 0;
    gridBagConstraints1.gridy = 15;
    gridBagConstraints1.gridwidth = 3;
    gridBagConstraints1.gridheight = 2;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.ipadx = -50;
    gridBagConstraints1.ipady = -7;
    gridBagConstraints1.insets = new java.awt.Insets (10, 30, 0, 0);
    add (jComboBoxEM, gridBagConstraints1);

    jComboBoxED = new javax.swing.JComboBox ();
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 0;
    gridBagConstraints1.gridy = 17;
    gridBagConstraints1.gridwidth = 3;
    gridBagConstraints1.gridheight = 2;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.ipadx = -50;
    gridBagConstraints1.ipady = -7;
    gridBagConstraints1.insets = new java.awt.Insets (10, 30, 0, 0);
    add (jComboBoxED, gridBagConstraints1);

    jButtonGetData = new javax.swing.JButton ();
    jButtonGetData.setText ("Get Data");
    jButtonGetData.addActionListener (new java.awt.event.ActionListener () {
        public void actionPerformed (java.awt.event.ActionEvent evt) {
          jButtonGetDataActionPerformed (evt);
        }
      }
    );
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 0;
    gridBagConstraints1.gridy = 19;
    gridBagConstraints1.gridwidth = 9;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.insets = new java.awt.Insets (20, 30, 0, 0);
    add (jButtonGetData, gridBagConstraints1);

    jSeparator1 = new javax.swing.JSeparator ();
    jSeparator1.setOrientation (javax.swing.SwingConstants.VERTICAL);
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 11;
    gridBagConstraints1.gridy = 1;
    gridBagConstraints1.gridheight = 21;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.ipadx = 8;
    gridBagConstraints1.ipady = 440;
    gridBagConstraints1.insets = new java.awt.Insets (16, 7, 0, 0);
    add (jSeparator1, gridBagConstraints1);

    jLabelDatabase = new javax.swing.JLabel ();
    jLabelDatabase.setText ("Database Objects");
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 0;
    gridBagConstraints1.gridy = 1;
    gridBagConstraints1.gridwidth = 11;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.insets = new java.awt.Insets (16, 20, 0, 0);
    add (jLabelDatabase, gridBagConstraints1);

    jLabelDataFields = new javax.swing.JLabel ();
    jLabelDataFields.setText ("Data Fields");
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 12;
    gridBagConstraints1.gridy = 1;
    gridBagConstraints1.gridwidth = 5;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.insets = new java.awt.Insets (16, 10, 0, 0);
    add (jLabelDataFields, gridBagConstraints1);

    jListXAxis = new javax.swing.JList ();
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 12;
    gridBagConstraints1.gridy = 4;
    gridBagConstraints1.gridwidth = 7;
    gridBagConstraints1.gridheight = 2;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.ipadx = 100;
    gridBagConstraints1.ipady = 90;
    gridBagConstraints1.insets = new java.awt.Insets (3, 10, 0, 0);
    add (jListXAxis, gridBagConstraints1);

    jRadioButtonPoints = new javax.swing.JRadioButton ();
    jRadioButtonPoints.setText ("Points");
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 12;
    gridBagConstraints1.gridy = 15;
    gridBagConstraints1.gridwidth = 4;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.insets = new java.awt.Insets (0, 10, 0, 0);
    add (jRadioButtonPoints, gridBagConstraints1);

    jRadioButtonLines = new javax.swing.JRadioButton ();
    jRadioButtonLines.setText ("Lines");
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 12;
    gridBagConstraints1.gridy = 17;
    gridBagConstraints1.gridwidth = 3;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.insets = new java.awt.Insets (0, 10, 0, 0);
    add (jRadioButtonLines, gridBagConstraints1);

    jButtonPlot = new javax.swing.JButton ();
    jButtonPlot.setText ("Plot");
    jButtonPlot.addActionListener (new java.awt.event.ActionListener () {
        public void actionPerformed (java.awt.event.ActionEvent evt) {
          jButtonPlotActionPerformed (evt);
        }
      }
    );
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 12;
    gridBagConstraints1.gridy = 19;
    gridBagConstraints1.gridwidth = 6;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.insets = new java.awt.Insets (20, 30, 0, 0);
    add (jButtonPlot, gridBagConstraints1);

    jPanelPlot = new javax.swing.JPanel ();
    jPanelPlot.setLayout (new java.awt.FlowLayout ());

    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 20;
    gridBagConstraints1.gridy = 3;
    gridBagConstraints1.gridwidth = 2;
    gridBagConstraints1.gridheight = 10;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.ipadx = 350;
    gridBagConstraints1.ipady = 270;
    gridBagConstraints1.insets = new java.awt.Insets (3, 30, 0, 0);
    add (jPanelPlot, gridBagConstraints1);

    jLabelXAxis = new javax.swing.JLabel ();
    jLabelXAxis.setText ("X-Axis");
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 12;
    gridBagConstraints1.gridy = 3;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.insets = new java.awt.Insets (23, 10, 0, 0);
    add (jLabelXAxis, gridBagConstraints1);

    jLabelYAxis = new javax.swing.JLabel ();
    jLabelYAxis.setText ("Y-Axis");
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 12;
    gridBagConstraints1.gridy = 6;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.insets = new java.awt.Insets (20, 10, 0, 0);
    add (jLabelYAxis, gridBagConstraints1);

    jListYAxis = new javax.swing.JList ();
    gridBagConstraints1 = new java.awt.GridBagConstraints ();
    gridBagConstraints1.gridx = 12;
    gridBagConstraints1.gridy = 8;
    gridBagConstraints1.gridwidth = 7;
    gridBagConstraints1.gridheight = 4;
    gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints1.ipadx = 100;
    gridBagConstraints1.ipady = 90;
    gridBagConstraints1.insets = new java.awt.Insets (0, 10, 0, 0);
    add (jListYAxis, gridBagConstraints1);

  }//GEN-END:initComponents

  private void jButtonGetDataActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonGetDataActionPerformed
    // Add your handling code here:
    //Load the time series.
    DatabaseRMI dbrmi = null;
    ts = null;
    YMDJComboBox ymd = new YMDJComboBox(jComboBoxSY,jComboBoxSM,jComboBoxSD,0,0,0);
    Date sd = ymd.getDate();
    ymd.setJComboBox(jComboBoxEY,jComboBoxEM,jComboBoxED,23,59,59);
    Date ed = ymd.getDate();
    String dbObject = (String) jListDatabase.getSelectedValue();
    try
    {
      dbrmi = (DatabaseRMI)
        Naming.lookup("//" + getCodeBase().getHost() +
        ":2001" + "/DatabaseRMIServer");
      ts = dbrmi.getTimeSeries(dbObject,sd,ed,true);
    }
    catch(NotBoundException nbe)
    {
      System.err.println(nbe);
    }
    catch(MalformedURLException murle)
    {
      System.err.println(murle);
    }
    catch(RemoteException re)
    {
      System.err.println(re);
    }
    catch(java.io.IOException ioe)
    {
      System.err.println(ioe);
    }
    catch(saaps.dbt.NoDataFoundException ndfe) {
      System.err.println(ndfe);
//      new ErrorDialog(getAppletFrame(),ndfe.getMessage());
    }

  }//GEN-LAST:event_jButtonGetDataActionPerformed

  private void jButtonPlotActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonPlotActionPerformed
    // Add your handling code here:
    int xField = -1;
    int[] yFields = {0};
    DataPlotterPlot plot = new DataPlotterPlot(ts,xField,yFields,false,false,null,null,false,false);
    if (jPanelPlot.getComponentCount() > 0)
      jPanelPlot.remove(0);
    jPanelPlot.add(plot.getPlot());
    jPanelPlot.validate();
  }//GEN-LAST:event_jButtonPlotActionPerformed


// Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JLabel jLabelTitle;
  private javax.swing.JList jListDatabase;
  private javax.swing.JLabel jLabelStart;
  private javax.swing.JComboBox jComboBoxSY;
  private javax.swing.JComboBox jComboBoxSM;
  private javax.swing.JComboBox jComboBoxSD;
  private javax.swing.JLabel jLabel1;
  private javax.swing.JComboBox jComboBoxEY;
  private javax.swing.JComboBox jComboBoxEM;
  private javax.swing.JComboBox jComboBoxED;
  private javax.swing.JButton jButtonGetData;
  private javax.swing.JSeparator jSeparator1;
  private javax.swing.JLabel jLabelDatabase;
  private javax.swing.JLabel jLabelDataFields;
  private javax.swing.JList jListXAxis;
  private javax.swing.JRadioButton jRadioButtonPoints;
  private javax.swing.JRadioButton jRadioButtonLines;
  private javax.swing.JButton jButtonPlot;
  private javax.swing.JPanel jPanelPlot;
  private javax.swing.JLabel jLabelXAxis;
  private javax.swing.JLabel jLabelYAxis;
  private javax.swing.JList jListYAxis;
// End of variables declaration//GEN-END:variables

}
