package saaps.saam;

import java.rmi.*;
import java.util.Date;

import saaps.dbt.*;
import saaps.math.*;
import saaps.util.*;

/**
*
* @author
* @version
*/
public class SEADataThread extends Object implements Runnable{
  //The uploaded time series.
  private TimeSeries sea;
  //The database connection.
  private DatabaseRMI dbRMI;
  //The event dates.
  private Date[] eventDates;
  //Start and end windows.
  private long startWindow, endWindow;
  //Name of the database object.
  private String dboName;
  //Field to use.
  private int field;
  //The data dialog.
  SEADataDialog seaDD;
  //The thread to this object.
  Thread thread;
  //Flag if thread is running.
  boolean isRunning;

  public SEADataThread(Date[] ed, long sw, long ew, DatabaseRMI dbr, String dbo, int f, SEADataDialog seadd)
  {
    eventDates = ed;
    startWindow = sw;
    endWindow = ew;
    dbRMI = dbr;
    dboName = dbo;
    field = f;
    seaDD = seadd;
    // Go through each event and load data.
    // Get first event.
  }

  private void getAllTimeSeries()
  {
    long eventTime = eventDates[0].getTime();
    Date start = new Date(eventTime + startWindow);
    Date end = new Date(eventTime + endWindow);
    TimeSeries epoch = getTimeSeries(start,end);
    Jama.Matrix epochData = epoch.getData();
    Jama.Matrix oneField = epochData.getMatrix(0,epochData.getRowDimension()-1,field,field);
    Jama.Matrix data = new Jama.Matrix(epoch.getLength(),eventDates.length,Double.NaN);
    data.setMatrix(0,epoch.getLength()-1,0,0,oneField);
    // Get the rest of the events.
    for (int i=1;i<eventDates.length;i++)
    {
      eventTime = eventDates[i].getTime();
      start = new Date(eventTime + startWindow);
      end = new Date(eventTime + endWindow);
      epoch = getTimeSeries(start,end);
      epochData = epoch.getData();
      oneField = epochData.getMatrix(0,epochData.getRowDimension()-1,field,field);
      data.setMatrix(0,epoch.getLength()-1,i,i,oneField);
    }
    // Create the SEA time series.
    Date[] seaDates = new Date[epoch.getLength()];
    for (int i=0;i<seaDates.length;i++)
      seaDates[i] = new Date((endWindow-startWindow)/seaDates.length*i + startWindow);
    sea = new TimeSeries(seaDates,data);
  }

  private TimeSeries getTimeSeries(Date sd, Date ed)
  {
    TimeSeries res = null;
    if (!seaDD.isCancelled())
    {
      try
      {
        res = dbRMI.getTimeSeries(dboName,sd,ed);
      }
      catch(RemoteException re)
      {
        System.err.println(re);
        return null;
      }
    }
 //   System.out.println("Read " + res.getLength() + " data points ...");
    return res;
  }

  public TimeSeries getTimeSeries()
  {
    return sea;
  }

  public void start()
  {
    thread = new Thread(this);
    thread.start();
  }

  public void run()
  {
    isRunning = true;
    getAllTimeSeries();
    isRunning = false;
  }

  public boolean isRunning()
  {
    return isRunning;
  }

}
