/*
 * AnomModelApplet.java
 *
 * Created on June 29, 2001, 2:32 PM
 */

package saaps.sapm;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

import saaps.dbt.DatabaseRMI;
import saaps.util.InfoDialog;
import saaps.util.Model;
import saaps.util.ModelList;
import saaps.util.TimeSeries;
import saaps.saam.ErrorDialog;

/**
 *
 * @author  root
 * @version 
 */
public class AnomModelApplet extends java.applet.Applet {
        
    //A time series with predicted anomalies.
    private TimeSeries anomPred;
    
    //The anomaly plot.
    private AnomModelPlot anomPlot;
    
    //The threaded object to create the plot.
    private PlotCreator pc;

    //The minimum and maximum values of the y-axis.
    private double minY = 0.0;
    private double maxY = 100.0;
    
    //The data loading info.
    private java.awt.Label labelLoadingData = new java.awt.Label("Loading data ...", java.awt.Label.CENTER);
    
    //Calendar object for date calculations.
    private Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));

    /** Initializes the applet AnomModelApplet */
    public void init () {
        initComponents ();
        this.labelLoadingData.setFont(new java.awt.Font("Helvetica",java.awt.Font.BOLD,24));
        this.panelPlot.add(this.labelLoadingData);
        this.initModelList();
        this.setDateTextFields(new Date());
        this.pc = new PlotCreator();
        (new Thread(this.pc)).start();
        this.setSize(600,450);
    }
    
    private void initModelList() {
        DatabaseRMI dbrmi = null;
        String [] myModelList = null;
        
        try {
            dbrmi = (DatabaseRMI)
            Naming.lookup("//" + getCodeBase().getHost() +
            ":2001" + "/DatabaseRMIServer");
            myModelList  = dbrmi.getModelListString();
        }
        catch(NotBoundException nbe) {
            System.err.println(nbe);
        }
        catch(MalformedURLException murle) {
            System.err.println(murle);
        }
        catch(RemoteException re) {
            System.err.println(re);
        }
        if(myModelList != null) {
            this.listModel.removeAll();
            for (int i=0;i<myModelList.length;i++) {
                this.listModel.add(myModelList[i]);
            }
            this.listModel.select(0);
        }
        else {
            new ErrorDialog(this.getAppletFrame(),"Cannot read the model list from server!");
        }
    }
    
    private java.awt.Frame getAppletFrame() {
        java.awt.Component curr = this;
        while (!(curr instanceof java.awt.Frame))
            curr = curr.getParent();
        return (java.awt.Frame) curr;
    }
    
    private void getAnomPred(Date d) {
        DatabaseRMI dbrmi = null;
        int modelIndex = this.listModel.getSelectedIndex();
        try {
            dbrmi = (DatabaseRMI)
            Naming.lookup("//" + getCodeBase().getHost() +
            ":2001" + "/DatabaseRMIServer");
            anomPred = dbrmi.getModelTS(modelIndex,d);
        }
        catch(NotBoundException nbe) {
            System.err.println(nbe);
            //      new ErrorDialog(getAppletFrame(),nbe.getMessage());
        }
        catch(MalformedURLException murle) {
            System.err.println(murle);
            //      new ErrorDialog(getAppletFrame(),murle.getMessage());
        }
        catch(RemoteException re) {
            System.err.println(re);
            //      new ErrorDialog(getAppletFrame(),re.getMessage());
        }
    }
    
    private void setDateTextFields(Date d) {
        this.cal.setTime(d);
        this.textFieldYear.setText(String.valueOf(cal.get(cal.YEAR)));
        this.textFieldMonth.setText(String.valueOf(cal.get(cal.MONTH)+1));
        this.textFieldDay.setText(String.valueOf(cal.get(cal.DATE)));
    }
    
    private void setDateTextFields() {
        Date d = this.anomPred.getLastDate();
        setDateTextFields(d);
    }
    
    private void setCalendar() throws NumberFormatException {
        this.cal.set(Calendar.YEAR,Integer.parseInt(this.textFieldYear.getText()));
        this.cal.set(Calendar.MONTH,Integer.parseInt(this.textFieldMonth.getText())-1);
        this.cal.set(Calendar.DATE,Integer.parseInt(this.textFieldDay.getText()));
        this.cal.set(Calendar.HOUR_OF_DAY,23);
        this.cal.set(Calendar.MINUTE,55);
        this.cal.set(Calendar.SECOND,55);
        this.cal.set(Calendar.MILLISECOND,00);
    }
    
    class PlotCreator implements Runnable {
        Date plotDate;
        //    double yl;
        //    double rl;
        
        PlotCreator() {
            plotDate = new Date();
            //      yl = 5e7;
            //      rl = 5e8;
            anomPlot = new AnomModelPlot(400,300);
        }
        
        public void setDate(Date d) {
            plotDate = d;
        }
        
        public Date getDate() {
            return plotDate;
        }
        
/*    public void setYellowLevel(double y)
    {
      yl = y;
    }
 
    public void setRedLevel(double r)
    {
      rl = r;
    }*/
        
        public void run() {
            getAnomPred(plotDate);
            setDateTextFields();
            anomPlot.setData(anomPred,minY,maxY);
            panelPlot.remove(labelLoadingData);
            panelPlot.add(anomPlot);
            anomPlot.repaint();
        }
    }
    
    /** This method is called from within the init() method to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        labelTitle = new java.awt.Label();
        panelModel = new java.awt.Panel();
        labelModel = new java.awt.Label();
        listModel = new java.awt.List();
        panelDate = new java.awt.Panel();
        labelYear = new java.awt.Label();
        textFieldYear = new java.awt.TextField();
        labelMonth = new java.awt.Label();
        textFieldMonth = new java.awt.TextField();
        labelDay = new java.awt.Label();
        textFieldDay = new java.awt.TextField();
        panelPlot = new java.awt.Panel();
        buttonToday = new java.awt.Button();
        buttonPlot = new java.awt.Button();
        buttonModelInfo = new java.awt.Button();
        buttonHelp = new java.awt.Button();
        setLayout(null);
        
        labelTitle.setFont(new java.awt.Font ("Dialog", 1, 14));
        labelTitle.setBackground(new java.awt.Color (204, 204, 204));
        labelTitle.setForeground(java.awt.Color.black);
        labelTitle.setText("Anomaly Predictions");
        
        add(labelTitle);
        labelTitle.setLocation(10, 10);
        labelTitle.setSize(labelTitle.getPreferredSize());
        
        
        panelModel.setLayout(new java.awt.BorderLayout());
        panelModel.setFont(new java.awt.Font ("Dialog", 0, 11));
        panelModel.setName("panel3");
        panelModel.setBackground(new java.awt.Color (204, 204, 204));
        panelModel.setForeground(java.awt.Color.black);
        
        labelModel.setFont(new java.awt.Font ("Dialog", 0, 11));
          labelModel.setBackground(new java.awt.Color (204, 204, 204));
          labelModel.setForeground(java.awt.Color.black);
          labelModel.setText("Model");
          panelModel.add(labelModel, java.awt.BorderLayout.NORTH);
          
          
        listModel.setFont(new java.awt.Font ("Courier", 0, 11));
          listModel.setBackground(new java.awt.Color (255, 247, 233));
          listModel.setForeground(java.awt.Color.black);
          panelModel.add(listModel, java.awt.BorderLayout.CENTER);
          
          
        add(panelModel);
        panelModel.setBounds(10, 50, 120, 180);
        
        
        panelDate.setLayout(new java.awt.GridLayout(3, 2));
        panelDate.setFont(new java.awt.Font ("Dialog", 0, 11));
        panelDate.setName("panel4");
        panelDate.setBackground(new java.awt.Color (204, 204, 204));
        panelDate.setForeground(java.awt.Color.black);
        
        labelYear.setFont(new java.awt.Font ("Dialog", 0, 11));
          labelYear.setBackground(new java.awt.Color (204, 204, 204));
          labelYear.setForeground(java.awt.Color.black);
          labelYear.setText("Year");
          panelDate.add(labelYear);
          
          
        textFieldYear.setBackground(new java.awt.Color (255, 247, 233));
          textFieldYear.setFont(new java.awt.Font ("Courier", 0, 11));
          textFieldYear.setColumns(4);
          textFieldYear.setForeground(java.awt.Color.black);
          textFieldYear.setText("2001");
          panelDate.add(textFieldYear);
          
          
        labelMonth.setFont(new java.awt.Font ("Dialog", 0, 11));
          labelMonth.setBackground(new java.awt.Color (204, 204, 204));
          labelMonth.setForeground(java.awt.Color.black);
          labelMonth.setText("Month");
          panelDate.add(labelMonth);
          
          
        textFieldMonth.setBackground(new java.awt.Color (255, 247, 233));
          textFieldMonth.setFont(new java.awt.Font ("Courier", 0, 11));
          textFieldMonth.setColumns(4);
          textFieldMonth.setForeground(java.awt.Color.black);
          textFieldMonth.setText("06");
          panelDate.add(textFieldMonth);
          
          
        labelDay.setFont(new java.awt.Font ("Dialog", 0, 11));
          labelDay.setBackground(new java.awt.Color (204, 204, 204));
          labelDay.setForeground(java.awt.Color.black);
          labelDay.setText("Day");
          panelDate.add(labelDay);
          
          
        textFieldDay.setBackground(new java.awt.Color (255, 247, 233));
          textFieldDay.setFont(new java.awt.Font ("Courier", 0, 11));
          textFieldDay.setColumns(4);
          textFieldDay.setForeground(java.awt.Color.black);
          textFieldDay.setText("29");
          panelDate.add(textFieldDay);
          
          
        add(panelDate);
        panelDate.setBounds(10, 280, 120, 100);
        
        
        panelPlot.setFont(new java.awt.Font ("Dialog", 0, 11));
        panelPlot.setName("panel6");
        panelPlot.setBackground(new java.awt.Color (204, 204, 204));
        panelPlot.setForeground(java.awt.Color.black);
        
        add(panelPlot);
        panelPlot.setBounds(160, 40, 400, 340);
        
        
        buttonToday.setFont(new java.awt.Font ("Dialog", 0, 11));
        buttonToday.setLabel("Today");
        buttonToday.setName("button7");
        buttonToday.setBackground(new java.awt.Color (204, 204, 204));
        buttonToday.setForeground(java.awt.Color.black);
        buttonToday.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonTodayActionPerformed(evt);
            }
        }
        );
        
        add(buttonToday);
        buttonToday.setLocation(10, 400);
        buttonToday.setSize(buttonToday.getPreferredSize());
        
        
        buttonPlot.setFont(new java.awt.Font ("Dialog", 0, 11));
        buttonPlot.setLabel("Plot");
        buttonPlot.setName("button8");
        buttonPlot.setBackground(new java.awt.Color (204, 204, 204));
        buttonPlot.setForeground(java.awt.Color.black);
        buttonPlot.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonPlotActionPerformed(evt);
            }
        }
        );
        
        add(buttonPlot);
        buttonPlot.setLocation(80, 400);
        buttonPlot.setSize(buttonPlot.getPreferredSize());
        
        
        buttonModelInfo.setFont(new java.awt.Font ("Dialog", 0, 11));
        buttonModelInfo.setLabel("Model Info ...");
        buttonModelInfo.setName("button9");
        buttonModelInfo.setBackground(new java.awt.Color (204, 204, 204));
        buttonModelInfo.setForeground(java.awt.Color.black);
        buttonModelInfo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonModelInfoActionPerformed(evt);
            }
        }
        );
        
        add(buttonModelInfo);
        buttonModelInfo.setLocation(30, 240);
        buttonModelInfo.setSize(buttonModelInfo.getPreferredSize());
        
        
        buttonHelp.setFont(new java.awt.Font ("Dialog", 0, 11));
        buttonHelp.setLabel("Help");
        buttonHelp.setName("button60");
        buttonHelp.setBackground(new java.awt.Color (204, 204, 204));
        buttonHelp.setForeground(java.awt.Color.black);
        buttonHelp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonHelpActionPerformed(evt);
            }
        }
        );
        
        add(buttonHelp);
        buttonHelp.setLocation(140, 400);
        buttonHelp.setSize(buttonHelp.getPreferredSize());
        
    }//GEN-END:initComponents

  private void buttonHelpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonHelpActionPerformed
// Add your handling code here:
    java.applet.AppletContext ac = getAppletContext();
    java.net.URL url = getDocumentBase();
    try
    {
      java.net.URL urlDoc = new java.net.URL(url,"AnomModelAppletHelp.html");
      ac.showDocument(urlDoc,"Anomaly Model Prediction Help");
    }
    catch (java.net.MalformedURLException murle)
    {
      System.err.println(murle);
    }
  }//GEN-LAST:event_buttonHelpActionPerformed

  private void buttonTodayActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonTodayActionPerformed
// Add your handling code here:
        this.setDateTextFields(new Date());
  }//GEN-LAST:event_buttonTodayActionPerformed

  private void buttonPlotActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonPlotActionPerformed
      try {
          //Get the selected model.
          //      int m = this.listModel.getSelectedIndex();
          //Get the end date from the text fields and set the calendar object.
          this.setCalendar();
          //Remove the old plot.
          this.panelPlot.remove(this.anomPlot);
          //Display data loading info.
          this.panelPlot.add(this.labelLoadingData);
          //Set the date in the plot creator.
          this.pc.setDate(this.cal.getTime());
          //Start the plot creator.
          (new Thread(this.pc)).start();
      }
      catch (NumberFormatException e) {
          new saaps.saam.ErrorDialog(this.getAppletFrame(),"Incorrect date entered");
      }
  }//GEN-LAST:event_buttonPlotActionPerformed

  private void buttonModelInfoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonModelInfoActionPerformed
      DatabaseRMI dbrmi = null;
      //Get the selected model.
      int m = this.listModel.getSelectedIndex();
      Model model = null;
      try {
          dbrmi = (DatabaseRMI)
          Naming.lookup("//" + getCodeBase().getHost() +
          ":2001" + "/DatabaseRMIServer");
          model  = dbrmi.getModel(m);
      }
      catch(NotBoundException nbe) {
          System.err.println(nbe);
      }
      catch(MalformedURLException murle) {
          System.err.println(murle);
      }
      catch(RemoteException re) {
          System.err.println(re);
      }
      if(model!= null) {
          //Model model = this.modelList.getModel(m);
          new InfoDialog(this.getAppletFrame(), model.getName(), model.getInfo(),40).setVisible(true);
      }
      else {
          new ErrorDialog(this.getAppletFrame(),"Cannot get the model from server!");
      }
  }//GEN-LAST:event_buttonModelInfoActionPerformed
  
  
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private java.awt.Label labelTitle;
  private java.awt.Panel panelModel;
  private java.awt.Label labelModel;
  private java.awt.List listModel;
  private java.awt.Panel panelDate;
  private java.awt.Label labelYear;
  private java.awt.TextField textFieldYear;
  private java.awt.Label labelMonth;
  private java.awt.TextField textFieldMonth;
  private java.awt.Label labelDay;
  private java.awt.TextField textFieldDay;
  private java.awt.Panel panelPlot;
  private java.awt.Button buttonToday;
  private java.awt.Button buttonPlot;
  private java.awt.Button buttonModelInfo;
  private java.awt.Button buttonHelp;
  // End of variables declaration//GEN-END:variables
  
}
