/*
 * ElFluence.java
 *
 * Created on November 21, 2000, 8:22 PM
 */
 
package saaps.sapm;

import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;
import java.util.Vector;

import saaps.dbt.ODIDatabase;
//import saaps.dbt.Database;
//import saaps.dbt.GOES08partDB;
import saaps.util.TimeSeries;

/** 
 *
 * @author  saaps
 * @version 
 */
public class ElFluence extends Object {

  //The full resolution data.
  private TimeSeries flux;
  //The 24 hour fluence.
  private TimeSeries fluence;
  private static Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
  
  /** Creates new ElFluence */
  public ElFluence() {
    this(new Date());
  }
  
  public ElFluence(Date d)
  {
    //Get the latest 8 days of >2 MeV electron flux.
    getLatest8Days(d);
    //Calculate the fluence.
    calcFluence();
  }
  
  private void getLatest8Days(Date d2)
  {
    //Eight days back.
    cal.setTime(d2);
    cal.add(cal.DATE,-8);
    cal.set(cal.HOUR_OF_DAY,0);
    cal.set(cal.MINUTE,10);
    cal.set(cal.SECOND,0);
    cal.set(cal.MILLISECOND,0);
    Date d1 = cal.getTime();
    //Connect to the database.
//    GOES08partDB db = (GOES08partDB) Database.getInstance("GOES08partDB",true);
    ODIDatabase db = new ODIDatabase();
    //Get the data.
//    try
//    {
//      flux = db.retrieveTS(d1,d2,true);
    flux = db.retrieveTS("KPAP_3h",d1,d2); // Should be changed to GOES_G11_5 when data exist.
//    }
//    catch(IOException ioe)
//    {
//      System.err.println(ioe);
//    }
  }
  
  private void calcFluence()
  {
    //Create vector. Max 7*24 hourly values over one week.
    Vector dateVec = new Vector(7*24);
    Vector fluenceVec = new Vector(7*24);
    //Find the point that starts one day ahead.
    cal.setTime(flux.getDate(0));
    cal.add(cal.HOUR_OF_DAY,24);
    long startDate = cal.getTime().getTime();
    int fluxStart = 0;
    cal.setTime(flux.getDate(fluxStart));
    while(cal.getTime().getTime()<startDate)
    {
      fluxStart++;
      cal.setTime(flux.getDate(fluxStart));
    }
    //Go through the flux data from the last point and
    //calculate daily sums with one hour sample rate.
    int fluxIndex = flux.getLength();
    while(fluxIndex>=fluxStart)
    {
      double data = 0;
      int j2 = fluxIndex;
//      int j1 = j2 - 24*12;  //Number of 5 minute data points in one day.
      int j1 = j2 - 8;  //Number of 3 hour data points in one day.
      int jn = 0;
      for(int j=j1;j<j2;j++)
      {
//        if(!Double.isNaN(flux.getData(j,7)))
        if(!Double.isNaN(flux.getData(j,0)))
        {
          jn++;
//          data += flux.getData(j,7);
          data += flux.getData(j,0);
        }
      }
      if(jn>0)
//        data *= 5*60*24*12/jn;
        data *= 5*60*24*12/jn;
      else
        data = Double.NaN;
      fluenceVec.addElement(new Double(data));
      dateVec.addElement(new Date(flux.getDate(j2-1).getTime()));
      fluxIndex -= 12;
    }
    //Create the time series object.
    dateVec.trimToSize();
    fluenceVec.trimToSize();
    Date[] dates = new Date[dateVec.size()];
    double[] data = new double[dates.length];
    for(int i=0;i<dates.length;i++)
    {
      int j = dates.length-1-i;
      Date d = (Date) dateVec.elementAt(j);
      Double x = (Double) fluenceVec.elementAt(j);
      dates[i] = new Date(d.getTime());
      data[i] = x.doubleValue();
    }
    fluence = new TimeSeries(dates,data);
  }
  
  public TimeSeries getFluence()
  {
    return fluence;
  }
  
  /**
  * @param args the command line arguments
  */
  public static void main (String args[]) {
    ElFluence.cal.setTime(new Date());
    ElFluence.cal.add(Calendar.DATE,-30);
    ElFluence.cal.set(Calendar.HOUR_OF_DAY,16);
    ElFluence.cal.set(Calendar.MINUTE,55);
    ElFluence flu = new ElFluence(ElFluence.cal.getTime());
    TimeSeries ts = flu.getFluence();
    for(int i=0;i<ts.getLength();i++)
      System.out.println(ts.getDate(i) + " " + ts.getData(i,0));
  }

}
