/*
 * ElFluencePlot3.java
 *
 * Created on November 24, 2000, 1:57 PM
 */
 
package saaps.sapm;

import java.awt.*;
import java.awt.geom.AffineTransform;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

import saaps.util.TimeSeries;

/** 
 *
 * @author  saaps
 * @version 
 */
public class ElFluencePlot3 extends Canvas {

  //The physical data to plotted.
  private Data data;
  //The reference frame.
  private RefFrame refFrame;
  //The axes.
  private Axes axes;
  //The transforms.
  private Transform trans;
  //The plot.
  private Plot plot;
  
  private Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));

  /** Creates new ElFluencePlot3 */
/*  public ElFluencePlot3(TimeSeries ts, double yl, double rl, int w, int h) {
    //Create the data.
    data = new Data(ts,yl,rl);
    //Create the reference frame.
    refFrame = new RefFrame(w,h);
    //Create the transform.
    trans = new Transform();
    //Create the plot.
    plot = new Plot();
    //Create the axes.
    axes = new Axes();
  }
  
  public ElFluencePlot3(TimeSeries ts, double yl, double rl, int w, int h, double min, double max) {
    //Create the data.
    data = new Data(ts,yl,rl,min,max);
    //Create the reference frame.
    refFrame = new RefFrame(w,h);
    //Create the transform.
    trans = new Transform();
    //Create the plot.
    plot = new Plot();
    //Create the axes.
    axes = new Axes();
  }*/
  
  public ElFluencePlot3(int w, int h)
  {
    //Create the reference frame.
    refFrame = new RefFrame(w,h);
    setSize(w,h);
  }
  
  public void setYellowLevel(double level)
  {
    data.setYellowLevel(level);
    plot.updateYellow();
  }
  
  public void setRedLevel(double level)
  {
    data.setRedLevel(level);
    plot.updateRed();
  }
  
  public void setData(TimeSeries ts, double yl, double rl, double min, double max)
  {
    //Create data.
    data = new Data(ts,yl,rl,min,max);
    //Create transform.
    trans = new Transform();
    //Create plot.
    plot = new Plot();
    //Create axes.
    axes = new Axes();
  }
  
  public void paint(Graphics gr)
  {
    plot.draw(gr);
    axes.draw(gr);
  }
  
  class Data
  {
    //The zero date.
    Date zeroDate;
    //The first date.
    Date firstDate;
    //The last date.
    Date lastDate;
    //The number of days from zero time.
    double[] x;
    //The log values of the fluence.
    double[] y;
    //The min and max values;
    double minX;
    double maxX;
    double minY;
    double maxY;
    //The log values of the levels.
    double yellowLevel;
    double redLevel;

    public Data(TimeSeries ts, double yl, double rl)
    {
      yellowLevel = yl;
      redLevel = rl;
      x = getXData(ts);
      y = getYData(ts);
      yellowLevel = getYData(yl);
      redLevel = getYData(rl);
      firstDate = ts.getDate(0);
      lastDate = ts.getDate(ts.getLength()-1);
    }
    
    public Data(TimeSeries ts, double yl, double rl, double min, double max)
    {
      yellowLevel = yl;
      redLevel = rl;
      x = getXData(ts);
      y = getYData(ts,min,max);
      yellowLevel = getYData(yl);
      redLevel = getYData(rl);
      firstDate = ts.getDate(0);
      lastDate = ts.getDate(ts.getLength()-1);
    }
    
    private void setYellowLevel(double level)
    {
      yellowLevel = getYData(level);
    }
    
    private void setRedLevel(double level)
    {
      redLevel = getYData(level);
    }
    
    private double[] getXData(TimeSeries ts)
    {
      minX = Double.POSITIVE_INFINITY;
      maxX = Double.NEGATIVE_INFINITY;
      double[] x = new double[ts.getLength()];
      cal.setTime(ts.getLastDate());
      cal.set(cal.HOUR_OF_DAY,0);
      cal.set(cal.MINUTE,0);
      cal.set(cal.SECOND,0);
      cal.set(cal.MILLISECOND,0);
      zeroDate = cal.getTime();
      long zeroTime = zeroDate.getTime();
      //Number of milliseconds per day.
      double timeDiv = 1000*60*60*24;
      for(int i=0;i<x.length;i++)
      {
        x[i] = (ts.getDate(i).getTime() - zeroTime)/timeDiv;
        minX = Math.min(x[i],minX);
        maxX = Math.max(x[i],maxX);
      }
      return x;
    }
    
    private double[] getYData(TimeSeries ts)
    {
      double[] y = new double[ts.getLength()];
      for(int i=0;i<y.length;i++)
      {
        y[i] = log10(ts.getData(i,0));
        minY = Math.min(y[i],minY);
        maxY = Math.max(y[i],maxY);
      }
      return y;
    }
    
    private double[] getYData(TimeSeries ts, double min, double max)
    {
      minY = log10(min);
      maxY = log10(max);
      double[] y = new double[ts.getLength()];
      for(int i=0;i<y.length;i++)
        y[i] = Math.max(Math.min(log10(ts.getData(i,0)),maxY),minY);
      return y;
    }
    
    private double getYData(double a)
    {
      return log10(a);
    }
    
    public double getMinX()
    {
      return minX;
    }

    public double getMaxX()
    {
      return maxX;
    }
    
    public double getMinY()
    {
      return minY;
    }
    
    public double getMaxY()
    {
      return maxY;
    }
  }
  
  class RefFrame
  {
    int x0 = 30;
    int y0 = 22;
    int width;
    int height;

    RefFrame(int w, int h)
    {
//      width = w - 2*x0;
//      height = h - 2*y0;
      width = w - x0 - 1;
      height = h - y0 - 30;
    }
      
  }
  
  class Axes
  {
    //The axes origin.
//    int x0 = 10;
//    int y0 = 10;
    //The axes border.
//    int width;
//    int height;
    //The axes limits in physical coordinates: days and log10(fluence).
    double[] xLim = new double[2];
    double[] yLim = new double[2];
    //The tick lines.
    XTick xTick;
    YTick yTick;
    //The labels.
    XLabel xLabel;
    YLabel yLabel;
    
    Axes()
    {
      //Default limits.
      xLim[0] = data.getMinX();
      xLim[1] = data.getMaxX();
      yLim[0] = data.getMinY();
      yLim[1] = data.getMaxY();
//      yLim[0] = 1e4;
//      yLim[1] = 1e10;
//      width = w - 2*x0;
//      height = h - 2*y0;
      xTick = new XTick();
      yTick = new YTick();
      xLabel = new XLabel();
      yLabel = new YLabel();
    }
    
    private void draw(Graphics g)
    {
      xTick.draw(g);
      yTick.draw(g);
      drawBorder(g);
      xLabel.draw(g);
      yLabel.draw(g);
    }
    
    private void drawBorder(Graphics g)
    {
      g.setColor(new Color(0,0,0));
      g.drawRect(refFrame.x0,refFrame.y0,refFrame.width,refFrame.height);
    }
    
    class XTick
    {
      //The positions in physical coordinates.
      double[] v;
      //The positions in device coordinates.
      int[] x;
      //The length.
      int y1;
      int y2;
      
      XTick()
      {
        //The number of x-tick lines equals the number of days.
        int n = (int) (Math.ceil(xLim[1])-Math.floor(xLim[0])) + 1;
        v = new double[n];
        v[0] = Math.floor(xLim[0]);
        for(int i=1;i<n;i++)
        {
          v[i] = v[i-1] + 1.0;
        }
        //Transform the data.
        x = trans.transformX(v);
        y1 = refFrame.y0;
        y2 = refFrame.y0 + refFrame.height + 8;
      }
      
      void draw(Graphics gr)
      {
        gr.setColor(new Color(100,100,100));
        for(int i=0;i<x.length;i++)
          gr.drawLine(x[i],y1,x[i],y2);
      }
    }

    class YTick
    {
      //The positions in physical coordinates.
      double[] w;
      //The positions in device coordinates.
      int[] y;
      //The length.
      int x1;
      int x2;
      
      YTick()
      {
        //The number of x-tick lines equals the number of days.
        int n = (int) (Math.ceil(yLim[1])-Math.floor(yLim[0])) + 1;
        w = new double[n];
        w[0] = Math.floor(yLim[0]);
        for(int i=1;i<n;i++)
        {
          w[i] = w[i-1] + 1.0;
        }
        //Transform the data.
        y = trans.transformY(w);
        x1 = refFrame.x0 - 8;
        x2 = refFrame.x0 + refFrame.width;
      }
      
      void draw(Graphics gr)
      {
        gr.setColor(new Color(100,100,100));
        for(int i=0;i<y.length;i++)
          gr.drawLine(x1,y[i],x2,y[i]);
      }
    }
    
    class XLabel
    {
      //The labels.
      private String[] labels;
      private String text;
      //The positions.
      private int[] x;
      private int y = refFrame.y0+refFrame.height+12;
      private int textX;
      private int textY = y + 12;
      
      XLabel()
      {
        labels = new String[xTick.x.length-1];
        x = new int[labels.length];
        for(int i=0;i<labels.length;i++)
        {
          cal.setTime(data.zeroDate);
          cal.add(cal.DATE,(int) xTick.v[i]);
          labels[i] = String.valueOf(cal.get(cal.DATE));
          x[i] = (xTick.x[i] + xTick.x[i+1])/2 - 12;
        }
        
        textX = (xTick.x[0] + xTick.x[xTick.x.length-1])/2 - 48;
        
        cal.setTime(data.firstDate);
        int firstMonth = cal.get(cal.MONTH);
        int firstYear = cal.get(cal.YEAR);
        cal.setTime(data.lastDate);
        int lastMonth = cal.get(cal.MONTH);
        int lastYear = cal.get(cal.YEAR);
        if(firstMonth==lastMonth)
          text = getText(firstMonth,firstYear);
        else
          text = getText(firstMonth,firstYear,lastMonth,lastYear);
      }
      
      private String getText(int fm, int fy)
      {
        String t = getMonth(fm) + " " + String.valueOf(fy);
        return t;
      }
      
      private String getText(int fm, int fy, int lm, int ly)
      {
        String t = null;
        if(fy==ly)
          t = getMonth(fm) + "/" + getMonth(lm) + " " + String.valueOf(fy);
        else
        {
          t = getMonth(fm) + " " + String.valueOf(fy) + " / " +
              getMonth(lm) + " " + String.valueOf(ly);
        }
        return t;
      }
      
      private String getMonth(int m)
      {
        String t = null;
        switch(m)
        {
          case 0:
            t = "Jan";
            break;
          case 1:
            t = "Feb";
            break;
          case 2:
            t = "Mar";
            break;
          case 3:
            t = "Apr";
            break;
          case 4:
            t = "May";
            break;
          case 5:
            t = "June";
            break;
          case 6:
            t = "July";
            break;
          case 7:
            t = "Aug";
            break;
          case 8:
            t = "Sep";
            break;
          case 9:
            t = "Oct";
            break;
          case 10:
            t = "Nov";
            break;
          case 11:
            t = "Dec";
            break;
        }
        return t;
      }
      
      public void draw(Graphics gr)
      {
        for(int i=0;i<labels.length;i++)
          gr.drawString(labels[i],x[i],y);
        gr.drawString(text,textX,textY);
      }
      
    }
    
    class YLabel
    {
      //The labels.
      private String[] labels;
      private String text = "log(f) where f is the fluence in cm^-2 sr^-1";
      //The positions.
      private int[] x;
      private int[] y;
      private int textX = 0;
      private int textY = 12;
      
      YLabel()
      {
        labels = new String[yTick.y.length];
        x = new int[labels.length];
        y = new int[labels.length];
        for(int i=0;i<labels.length;i++)
        {
          labels[i] = String.valueOf((int) yTick.w[i]);
          x[i] = 12 - ((int) yTick.w[i]/10)*6;
          y[i] = yTick.y[i] + 6;
        }
//        textY = (yTick.y[0] + yTick.y[yTick.y.length-1])/2 + 72;
      }

      public void draw(Graphics gr)
      {
        for(int i=0;i<labels.length;i++)
          gr.drawString(labels[i],x[i],y[i]);
        gr.drawString(text,textX,textY);
/*        Font f = gr.getFont();
        AffineTransform a = f.getTransform();
        a.setToRotation(0.1);
        System.out.println(a.getType());
        Font fr = f.deriveFont(a);
        FontRenderContext frc = new FontRenderContext(a,false,false);
        TextLayout tl = new TextLayout(text,fr,frc);
//        gr.setFont(f);
//        gr.drawString(text,textX,textY);
        gr.setFont(fr);
        gr.drawString(text,textX,textY);
        gr.setFont(f);*/
      }
      
    }
    
  }
  
  class Transform
  {
    //Physical limits.
    double v1;
    double v2;
    double w1;
    double w2;
    //Device limits.
    int x1;
    int x2;
    int y1;
    int y2;
        
    Transform()
    {
      v1 = Math.floor(data.getMinX());
      v2 = Math.ceil(data.getMaxX());
      w1 = Math.floor(data.getMinY());
      w2 = Math.ceil(data.getMaxY());
      x1 = refFrame.x0;
      x2 = refFrame.x0 + refFrame.width;
      y1 = refFrame.y0;
      y2 = refFrame.y0 + refFrame.height;
    }
    
    int[] transformX(double[] v)
    {
      int[] x = new int[v.length];
      for(int i=0;i<x.length;i++)
        x[i] = ((int) ((v[i]-v1)/(v2-v1)*(x2-x1))) + x1;
      return x;
    }

    int[] transformY(double[] w)
    {
      int[] y = new int[w.length];
      for(int i=0;i<y.length;i++)
        y[i] = ((int) ((w[i]-w1)/(w2-w1)*(y1-y2))) + y2;
      return y;
    }
    
    int transformY(double w)
    {
      return ((int) ((w-w1)/(w2-w1)*(y1-y2))) + y2;
    }
  }
  
  class Plot
  {
    private double[] v;
    private double[] w;
    private int[] x;
    private int[] yGreen;
    private int[] yYellow;
    private int[] yRed;
    private Color cGreen = new Color(0,255,0);
    private Color cYellow = new Color(255,255,0);
    private Color cRed = new Color(255,0,0);
    
    Plot()
    {
      v = new double[data.x.length+3];
      for(int i=0;i<data.x.length;i++)
        v[i] = data.x[i];
      v[v.length-3] = v[v.length-4];
      v[v.length-2] = v[0];
      v[v.length-1] = v[0];
      x = trans.transformX(v);
      
      updateYellow();
      
      updateRed();
      
      for(int i=0;i<data.y.length;i++)
        w[i] = data.y[i];
      w[w.length-3] = trans.w1;
      w[w.length-2] = trans.w1;
      w[w.length-1] = w[0];
      yRed = trans.transformY(w);
    }
    
    public void updateYellow()
    {
      w = new double[data.y.length+3];
      for(int i=0;i<data.y.length;i++)
        w[i] = Math.min(data.y[i],data.yellowLevel);
      w[w.length-3] = trans.w1;
      w[w.length-2] = trans.w1;
      w[w.length-1] = w[0];
      yGreen = trans.transformY(w);
    }
    
    public void updateRed()
    {
      for(int i=0;i<data.y.length;i++)
        w[i] = Math.min(data.y[i],data.redLevel);
      w[w.length-3] = trans.w1;
      w[w.length-2] = trans.w1;
      w[w.length-1] = w[0];
      yYellow = trans.transformY(w);
    }
    
    private void draw(Graphics gr)
    {
      drawRed(gr);
      drawYellow(gr);
      drawGreen(gr);
      drawLevels(gr);
    }
    
    private void drawGreen(Graphics gr)
    {
      gr.setColor(cGreen);
      gr.fillPolygon(x,yGreen,x.length);
    }

    private void drawYellow(Graphics gr)
    {
      gr.setColor(cYellow);
      gr.fillPolygon(x,yYellow,x.length);
    }

    private void drawRed(Graphics gr)
    {
      gr.setColor(cRed);
      gr.fillPolygon(x,yRed,x.length);
    }
    
    private void drawLevels(Graphics gr)
    {
      //Red line.
      int level = trans.transformY(data.redLevel);
      gr.setColor(cRed);
      gr.fillRect(x[0],level-1,x[x.length-4]-x[0],2);
//      gr.drawLine(x[0],level,x[x.length-4],level);
      //Yellow line.
      level = trans.transformY(data.yellowLevel);
      gr.setColor(cYellow);
      gr.fillRect(x[0],level-1,x[x.length-4]-x[0],2);
//      gr.drawLine(x[0],level,x[x.length-4],level);
    }
    
  }
  
  private double log10(double x)
  {
    return Math.log(x)/Math.log(10.0);
  }
  
}