/*
 * EventList.java
 *
 * Created on June 8, 2001, 10:10 AM
 */

package saaps.util;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.StringTokenizer;
import java.util.TimeZone;
import java.util.Vector;
import java.util.Locale;
import java.io.Serializable;

/**
 *
 * @author  root
 * @version 
 */
public class EventList extends java.lang.Object implements Serializable {

    //Objects for date calculations and parsing.
    private Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));

    private SimpleDateFormat format = new SimpleDateFormat("yyyy MM dd HH mm ss");

    //The calendar date fields.
    private int[] dateFields = {Calendar.YEAR, Calendar.MONTH, Calendar.DATE,
                        Calendar.HOUR_OF_DAY, Calendar.MINUTE, Calendar.SECOND};
    
    //The event dates.
    private Vector eventDates = null;
    
//    //Fields to use.
//    private boolean[] useField;
    
    /** Creates new EventList */
    public EventList() {
        this.eventDates = new Vector();
    }
    
    public EventList(String eventString) {
//        System.out.println(eventString);
        this.parseEventString(eventString);
    }
    
//    public EventList(String eventString, boolean[] useField) {
//        this.useField = useField;
//        this.parseEventString(eventString);
//    }
    
    public EventList(Date[] dates) {
        this.eventDates = new Vector(dates.length);
        for (int i=0;i<dates.length;i++) {
            this.eventDates.addElement(dates[i]);
        }
    }

    public void parseEventString(String eventString) {
        // Tokenize with respect to newline.
        StringTokenizer stLine = new StringTokenizer(eventString,"\n");
        int numOfLines = stLine.countTokens();
        //Set the number of events.
        this.eventDates = new Vector(numOfLines);
//    Date[] res = new Date[numOfLines];
        //Tokenize one line at a time.
//        int row = 0;
        for (int i=0;i<numOfLines;i++)
        {
            // Parse one line.
            this.parseEvent(stLine.nextToken());
//            row++;
        }
    }
    
    public void parseEventFile(String fileName) {
        //Open file for reading.
        try {
            FileReader fr = new FileReader(fileName);
            BufferedReader reader = new BufferedReader(fr);
            //Set the number of events to a default value.
            this.eventDates = new Vector(100);
            //Read one line at a time and parse.
            String oneLine = null;
            while ((oneLine = reader.readLine()) != null) {
                this.parseEvent(oneLine);
            }
            this.eventDates.trimToSize();
        }
        catch (FileNotFoundException e) {
            System.err.println(e);
        }
        catch (IOException e) {
            System.err.println(e);
        }
    }
    
    private void parseEvent(String eventString) {
        // Tokenize the event string.
        StringTokenizer st = new StringTokenizer(eventString);
        int field = 0;
        this.cal.clear();
//        while (st.hasMoreTokens() & field<this.useField.length)
        while (st.hasMoreTokens() & field<6)
        {
            String token = st.nextToken();
//            System.out.println(token);
            int t = 0;
            //Try to parse as an integer.
            try {
                t = Integer.parseInt(token);
            }
            //If it can not parse then use defult t=0.
            catch (NumberFormatException e) {
            }
            // The month number is the month minus 1.
            if (field==1) t--;
//            if (this.useField[field]) {
//                this.cal.set(this.dateFields[field],t);
//            }
            this.cal.set(this.dateFields[field],t);
            field++;
         }
         this.eventDates.addElement(cal.getTime());
    }
    
    /**
     *Use only the year, month, and date to describe the events.
     *The other fields are set to zero.
     *Remove repeated events.
     */
    public void useOnlyDates() {
        //Get first date.
        int i=0;
        cal.setTime((Date) this.eventDates.elementAt(i));
        cal.set(Calendar.HOUR_OF_DAY,0);
        cal.set(Calendar.MINUTE,0);
        cal.set(Calendar.SECOND,0);
        cal.set(Calendar.MILLISECOND,0);
        Date newDate = cal.getTime();
        this.eventDates.setElementAt(newDate,i);
        //Go through the events.
        while (++i<this.eventDates.size()) {
            cal.setTime((Date) this.eventDates.elementAt(i));
            cal.set(Calendar.HOUR_OF_DAY,0);
            cal.set(Calendar.MINUTE,0);
            cal.set(Calendar.SECOND,0);
            cal.set(Calendar.MILLISECOND,0);
            newDate = cal.getTime();
            if (newDate.compareTo((Date) this.eventDates.elementAt(i-1))==0) {
                this.eventDates.removeElementAt(i--);
            }
            else {
                this.eventDates.setElementAt(newDate,i);
            }
        }
    }

    public Vector getEvents() {
        return this.eventDates;
    }
    
    public Date getEvent(int i) {
        return (Date) this.eventDates.elementAt(i);
    }
    
    public int getNumOfEvents() {
        return this.eventDates.size();
    }
    
    public String toString() {
        if (this.eventDates == null) {
            return "";
        }
        StringBuffer buffer = new StringBuffer();
        for (int i=0;i<this.eventDates.size();i++) {
            cal.setTime((Date) this.eventDates.elementAt(i));
            buffer.append(this.calToString());
//            buffer.append(this.format.format((Date) this.eventDates.elementAt(i)) + "\n");
        }
        return buffer.toString();
    }
    
    private String calToString() {
        StringBuffer buf = new StringBuffer();
        buf.append(this.cal.get(Calendar.YEAR) + " ");
        int x = this.cal.get(Calendar.MONTH) + 1;
        if (x<10) buf.append("0" + x + " ");
        else buf.append(x + " ");
        x = this.cal.get(Calendar.DATE);
        if (x<10) buf.append("0" + x + " ");
        else buf.append(x + " ");
        x = this.cal.get(Calendar.HOUR_OF_DAY);
        if (x<10) buf.append("0" + x + " ");
        else buf.append(x + " ");
        x = this.cal.get(Calendar.MINUTE);
        if (x<10) buf.append("0" + x + " ");
        else buf.append(x + " ");
        x = this.cal.get(Calendar.SECOND);
        if (x<10) buf.append("0" + x + "\n");
        else buf.append(x + "\n");
        return buf.toString();
    }
}
