/*
 * ModelList.java
 *
 * Created on June 18, 2001, 1:08 PM
 */

package saaps.util;

import java.awt.List;
import java.util.Vector;
import java.util.Date;
import java.util.Calendar;
import java.util.Locale;

import saaps.sapm.Anom001Model;
import saaps.sapm.Anom002Model;
import saaps.sapm.Anom003Model;

/**
 * This class contain the available models.
 *
 * @author  Peter Wintoft
 * @version 1.0
 */
public class ModelList extends java.lang.Object {
    
    //The models.
    private Vector models;

    /** Creates new ModelList */
    public ModelList() {
        this(true);
    }
    
    public ModelList(boolean readFromFile) {
        this.models = new Vector(3);
        this.models.addElement(new Anom001Model(readFromFile));
        this.models.addElement(new Anom002Model(readFromFile));
        this.models.addElement(new Anom003Model(readFromFile));
        this.models.trimToSize();
    }

    /**
     *Return the number of models.
     *
     *@return Number of models.
     */
    public int getNumOfModels() {
        return this.models.size();
    }
    
    /**
     *Get one model.
     *
     *@param i The model index
     *@return The model
     */
    public Model getModel(int i) {
        return (Model) this.models.elementAt(i);
    }

    /**
     *Gets a TimeSeries from input data. 
     *
     *@param i The model index
     *@param d1 start date
     *@param d2 end date
     *@return The TimeSeries
     */
    public TimeSeries getModelTS(int i,Date d1, Date d2) {
//System.err.println("d1: " + d1 + " d2: " + d2);
	Model model = (Model) this.models.elementAt(i);
	model.readTimeSeries(d1,d2);
	model.run();
	return model.getOutput();
    }

    /**
     * Gets a TimeSeries from the Model with this prediction date
     * and 7 days back.
     *
     *@param i The model index
     *@param predDate prediction date
     *@return The TimeSeries
     */
    public TimeSeries getModelTS(int i,Date predDate) {
        //Go back 8 days to ensure that 7 full days of data are loaded.
//        Date d1 = new Date(predDate.getTime() - (8L * 24L * 60L * 60L * 1000L));
        Date d1 = new Date(predDate.getTime() - (15L * 24L * 60L * 60L * 1000L));
	return getModelTS(i,d1,predDate);
    }
    
    /**
     *Return a AWT list of the models.
     *
     *@return The models as an List object.
     */
    public List getList() {
        List list = new List();
        for (int m=0;m<this.models.size();m++) {
            list.add(((Model) models.elementAt(m)).getName());
        }
        return list;
    }
    
    public String[] getListString() {
        String[] str = new String[this.models.size()];
        for (int i=0;i<str.length;i++) {
            str[i] = ((Model) this.models.elementAt(i)).getName();
        }
        return str;
    }
    
    /**
     *Sort the models using an array of doubles. The double array is also sorted.
     *
     *@param x Array of doubles.
     */
    public void sort(double[] x) {
        Vector vx = new Vector(x.length);
        Vector oldModels = new Vector(this.models.size());
        for (int i=0;i<x.length;i++) {
            vx.addElement(new Double(x[i]));
            oldModels.addElement((Model) this.models.elementAt(i));
        }

        for (int j1=0;j1<x.length;j1++) {
            int jmax = 0;
            double max = ((Double) vx.elementAt(jmax)).doubleValue();
            for (int j2=0;j2<vx.size();j2++) {
                if (((Double) vx.elementAt(j2)).doubleValue() > max) {
                    max = ((Double) vx.elementAt(j2)).doubleValue();
                    jmax = j2;
                }
            }
            x[j1] = max;
            this.models.set(j1,(Model) oldModels.elementAt(jmax));
            vx.remove(jmax);
            oldModels.remove(jmax);
        }
    }
    
    /**
    * @param args the command line arguments
    */
    public static void main (String args[]) {
        ModelList ml = new ModelList();
        double[] x = {2.0 , 3.0};
        for (int i=0;i<x.length;i++) { 
            System.out.println(x[i]);
            System.out.println(ml.getModel(i).getName());
        }
        ml.sort(x);
        System.out.println();
        for (int i=0;i<x.length;i++) { 
            System.out.println(x[i]);
            System.out.println(ml.getModel(i).getName());
        }
    }

}
