PRO odi_ascii, out_name, dataset_name, varnames=varnames, $
               epochrange=epochrange, recordrange=recordrange

  dataset_name = STRSPLIT(dataset_name, 'SYSTEM!', /EXTRACT, /REGEX)

  IF (STRPOS(out_name, '.txt') LT 0) THEN out_name += '.txt'
  FILE_DELETE, out_name, /ALLOW_NONEXISTENT

;
; Construct the epoch or time range in SQL
;
  SQL_range, epochrange=epochrange, recordrange=recordrange, sqlrange

; Connect to the database
  DBConnect, oDB

; Find the dataset id
  oRS = OBJ_NEW('IDLDBRecordSet', oDB, $
                SQL="SELECT id, name, LEFT(majority, 20) AS majority " + $
                    "FROM dataset WHERE " + $
                    "sedat_name='" + STRUPCASE(dataset_name) + "'")
  IF (oRS->CurrentRecord() GE 0) THEN BEGIN
    ds = oRS->GetRecord()
    OBJ_DESTROY, oRS
  ENDIF ELSE $
    error_exit, ' Dataset ' + dataset_name + ' not found!'

  OPENW, 1, out_name

;
; Retrieve and store variable definitions from table variable_metadata
;
  IF (N_ELEMENTS(varnames) EQ 0) THEN varnames = '*'

  var = odi_vardefs(varnames, ds.id, oDB)

  ind = WHERE(var.rvar EQ 'F', count)
  PRINTF, 1, '# NRV (non-record varying variables):', count

  nvar = N_ELEMENTS(var)
  tformat = STRARR(nvar)
  tndims = INTARR(nvar)
  tdims = INTARR(nvar, 100)
  tunits = STRARR(nvar)
  theader = STRARR(nvar)
  FOR i=0,nvar-1 DO BEGIN
    header = STRING(var[i].name, var[i].sedat_type, FORMAT='(A-25,1X,A1)')
    IF (STRUPCASE(var[i].name) EQ 'EPOCH') THEN BEGIN
        tform = 'F21.4'
    ENDIF ELSE IF (var[i].sedat_type EQ 'I') THEN BEGIN
      tform = 'I14'
    ENDIF ELSE IF (var[i].sedat_type EQ 'R') THEN BEGIN
      tform = 'E16.6'
    ENDIF ELSE IF (var[i].sedat_type EQ 'D') THEN BEGIN
      tform = 'E20.10'
    ENDIF ELSE BEGIN
      tform = 'A' + STRTRIM(STRING(var[i].number_elements), 2)
    ENDELSE
; Read units if present
    sql = "SELECT LEFT(value, 100) AS units FROM variable_attribute " + $
          "WHERE name='UNITS' AND variable_metadata_id=" + $
          STRING(var[i].id)
    oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=sql)
  IF (oRS->CurrentRecord() GE 0) THEN BEGIN
      rec = oRS->GetRecord()
      OBJ_DESTROY, oRS
      units = rec.units
    ENDIF ELSE units = ' '
    IF (var[i].dims EQ 0) THEN BEGIN
      ndims = 1
      dims = [1]
    ENDIF ELSE IF (var[i].dims EQ 1) THEN BEGIN
      ndims = 1
      dims = var[i].sizes
    ENDIF ELSE BEGIN
      ndims = var[i].dims
      dims = STRSPLIT(var[i].sizes, ',', /EXTRACT)
    ENDELSE
    header += STRING(tform, units, ndims, $
                     FORMAT='(1X,A-10,1X,A-20,1X,I2,1H:)')
    FOR j=0,ndims-1 DO $
      header += STRING(dims[j], FORMAT='(1Hx,I3)')

    IF (var[i].rvar EQ 'F') THEN BEGIN
; Read NRV values
      sql = "SELECT LEFT(value, 500) AS value FROM variable_nrv " + $
            "WHERE variable_metadata_id=" + STRING(var[i].id) + " ORDER BY ind ASC"
      oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=sql)
      IF (oRS->CurrentRecord() GE 0) THEN BEGIN
        nrv = oRS->GetRecord()
        WHILE (oRS->MoveCursor(/NEXT)) DO $
          nrv = [nrv, oRS->GetRecord()]
        OBJ_DESTROY, oRS
      ENDIF ELSE $
        error_exit, ' Error reading NRV variables!'
      header += ' :'
      FOR j=0,N_ELEMENTS(nrv)-1 DO $
        header += STRING(nrv[j].value, FORMAT='(1X,'+tform+')')
      PRINTF, 1, header
    ENDIF
    tformat[i] = tform
    tndims[i] = ndims
    tdims[i,0:ndims-1] = dims
    tunits[i] = units
    theader[i] = header
  ENDFOR

;
; Read the record varying variables from the database and
; write them to the ASCII file
;

; Filter out the record varying variables
  ind = WHERE(var.rvar EQ 'T', count)
  PRINTF, 1, '# RV (record varying variables):', count
  IF (count GT 0) THEN BEGIN
    var = var[ind]
; Add variable declaration and SQL strings to structure vars
    FOR i=0,count-1 DO BEGIN
      odi_descvar, var[i], ds.majority, vtype, sql
      var[i].vtype = vtype
      var[i].sql = sql
    ENDFOR

    tformat = tformat[ind]
    tndims = tndims[ind]
    tdims = tdims[ind]
    theader = theader[ind]
    FOR i=0,N_ELEMENTS(var)-1 DO PRINTF, 1, theader[i]

; Create the SQL query to retrieve the data
    sql = odi_rvsql(var) + " FROM " + ds.name + sqlrange

    oRS = OBJ_NEW('IDLDBRecordSet', oDB, N_BUFFERS=10000, SQL=sql)

    nrec = 1L
    IF (oRS->CurrentRecord() GE 0) THEN status = 1 ELSE status = -1
    WHILE (status GT 0) DO BEGIN
      data = oRS->GetRecord()
      nr = 0
      header = '#' + STRING(nrec, FORMAT='(I10)')
      FOR i=0,N_ELEMENTS(var)-1 DO BEGIN
        res = EXECUTE("dvar = " + var[i].vtype)
        nd = SIZE(dvar)
        nd = nd[N_ELEMENTS(nd)-1]
        header += ' :'
        FOR j=0,nd-1 DO header += STRING(data.(nr+j), FORMAT='(1H ,'+tformat[i]+')')
        nr += nd
      ENDFOR
      PRINTF, 1, header
      nrec += 1
      status = oRS->MoveCursor(/NEXT)
    ENDWHILE
    OBJ_DESTROY, oRS
  ENDIF

  CLOSE, 1

  OBJ_DESTROY, oDB

END