; $Id: sedat_close.pro,v 1.7 2001/12/06 12:46:43 stamper Exp $
; 
; sedat_close
; 
; Closes a dataset and releases the STPDF resources associated with it
; 
; Arguments
;   meta          I  A meta-data structure for the file to be closed
;   status        O  Status code - 0 is success
;   cmn           IO SEDAT execution environment information
;
; External routines 
;   libidl_stpdf.so STPDF library
;
; $Log: sedat_close.pro,v $
; Revision 1.7  2001/12/06 12:46:43  stamper
; Use idl_stp_relcd for input files.
; Commented thoroughly.
;
; Revision 1.6  2001/05/09 08:37:33  stamper
; Use the new meta.io flag to determine whether the file is an input or
; output file, then look up the file details using the meta.idsid or
; meta.odsid arrays, indexed by meta.id.
;
; Revision 1.5  2000/12/11 15:03:09  stamper
; Added comments and trapped non-structure arguments
;

pro sedat_close, meta, STATUS=status, CMN=cmn

; Error if the meta argument is not a structure

    if (type_of(meta) ne 'STRUCT') then begin
        status = 1
    endif else begin

    ; Check if it is a system/ODI dataset
    IF ( OBJ_VALID(meta.ODI_DB) ) THEN BEGIN
       OBJ_DESTROY, meta.ODI_DB
       cmn.odsid[meta.id] = -1
       RETURN
    ENDIF


; Check whether the data file was opened for input or output
        if (meta.io eq 'I') then begin

;         For input close with idl_stp_relcd
          status = call_external('libidl_stpdf.so','idl_stp_relcd', $
	                 	  cmn.idsid[meta.id])
          cmn.idsid[meta.id] = -1
        endif else if (meta.io eq 'O') then begin
;         For output close with idl_chp_relcd
          status = call_external('libidl_stpdf.so','idl_chp_relcd', $
	                         cmn.odsid[meta.id]) 

;         H.Evans 2003/12/02
;         quick hack to correct a bug where the cdf.chp file is misnamed CDF.chp
;         This seems to occur when fields are appended to those from an existing file and 
;         the new file has a CDF.chp support file instead of cdf.chp.

	   fmask = cmn.ds_dir+cmn.ods[meta.id]
	   oldchp = fmask + '/CDF.chp'
          if file_test( oldchp) then begin
		newchp = fmask + '/cdf.chp' 
		spawn,'mv ' + oldchp  + ' ' + newchp
	   endif
          cmn.odsid[meta.id] = -1
        endif
    endelse

    return
end
