; $Id: sedat_make_dmeta.pro,v 1.3 2001/12/06 12:48:43 stamper Exp $
; 
; sedat_make_dmeta
; 
; Make a metadata structure for a whole dataset
; 
; Arguments
;   n_recs    I    The number of records (if known)
;   varmeta   I    A structure where each tag points to a variable metadata
;                  structure
;   dmeta     I    Set to a variable where the result is returned
; Keywords         
;   status    O    To return a status value
;   cmn       IO   To pass in/return file information.
;
; Returns
;   A status code to indicate success/failure; 0 is success
;
; $Log: sedat_make_dmeta.pro,v $
; Revision 1.3  2001/12/06 12:48:43  stamper
; Commented thoroughly.
;
; Revision 1.2  2000/12/11 15:24:07  stamper
; Extended comments.
; Changed logging message to refer to 'file' not 'output file'
;

pro sedat_make_dmeta, n_recs, varmeta, dmeta, nrv_vars, $
    status = status, cmn = cmn

    false = 0 eq 1
    true  = 1 eq 1

    if (n_params() ne 4) then begin
	status = 1
	print, systime(0), $
             ' Failed creating dataset meta structure - '+ $
             'need 3 parameters in function call'
	return
    endif
	
    ; Check that varmeta is defined

    if (n_elements(varmeta) eq 0) then begin
	print, systime(0), $
	     ' Failed creating dataset meta structure - '+ $
             'no variable metadata'
	status = 2
	return
    endif

    n_vars = n_tags(varmeta);

    dmeta = {name:'',$
             id:0L,$
             n_vars:n_vars[0],$
             n_recs:n_recs[0],$
             rec:0L,$
             var:varmeta,$
             nrv_vars:nrv_vars,$
             only_rec_vary: false, $
             odi_db: OBJ_NEW(),$
             odi_query: ''}
    if (!QUIET eq 0) then print, systime(0),' Created dataset structure for file'
    status = 0
end




