; $Id: sedat_open_read.pro,v 1.11 2002/12/05 16:35:54 stamper Exp $
; 
; sedat_open_read
; 
; Open a dataset for reading
; 
; Arguments
;   file_number   I    The number of the file to open
;   meta	  O    The dataset metadata structure returned
;   
; Keywords
;   status        O    Status code - 0 is success
;   cmn           IO   SEDAT execution environment information
;
; External routines 
;   libidl_stpdf.so STPDF library
;    
; $Log: sedat_open_read.pro,v $
; Revision 1.11  2002/12/05 16:35:54  stamper
; Corrected spelling mistake in comments.
;
; Revision 1.10  2001/12/06 12:48:04  stamper
; Make construction of the structure form of the axes_desc descriptor
; conditional on finding a !DEPEND_j parameter.
; Commented thoroughly.
;
; Revision 1.9  2001/05/09 08:41:24  stamper
; Correct the setting of the units metadata item.
; Set the meta.id element to index into the arrays of file details, and set
; the meta.io element to 'I' to indicate an input dataset.
; Set the meta.name field to the file name known to the user.
;
; Revision 1.8  2000/12/11 16:00:37  stamper
; Added comment header.
; Layout changes.
; Set initial values for meta.id and meta.rec for return after opening a file
;

pro sedat_open_read, file_number, meta, STATUS=status, CMN=cmn

; The 'file' parameter should be some way of uniquely identifying a file.
; When used within a tool, it can simply be a number, indicating the 
; input number for that tool, and the CMN data structure will have the 
; information needed to convert that to a real file name
;
;!QUIET=0
; Check that the file_number is within range
    if file_number le 0 or file_number gt n_elements(cmn.idsf) or $
                           file_number gt n_elements(cmn.idsd) then begin
        status = 1
        return
    endif

; Open the file and retrieve the dataset metadata to a structure
    file = cmn.idsf[file_number-1]
    dir  = cmn.idsd[file_number-1]

; Add the database connection and the record set to meta
  oDB = OBJ_NEW()
  sql_query = ''

  nrv_vars = -1

  IF (STRPOS(file, "SYSTEM!") GE 0) THEN BEGIN
    sedat_open_read_odi, STRMID(file, 7), numents, varmeta, status, oDB, sql_query, nrv_vars, cmn=cmn
    if status ne 0 then return

    type = "LRD"
    id = 0L
  ENDIF ELSE BEGIN
    if dir eq 'LRD' then begin
        dir = ''
        type = 'LRD'
    endif else begin
        type = 'CDF'
    endelse

    id = 0L
    numents = 0L
    numcols = 0L
    numpars = 0L
    status = call_external('libidl_stpdf.so','idl_stp_getcd',$
                	   file,dir,type,id,numents,numcols,numpars);

    if status ne 0 then return
    id = long(id)
; Get the column and parameter names   
    tempnames = strarr(numcols);
    status = call_external('libidl_stpdf.so','idl_chp_getcnn',$
              	           id,tempnames,numcols)
    if status ne 0 then return
    colnames = ''+tempnames;

    parnames = ''
    if numpars gt 0 then begin
        tempnames = strarr(numpars);
        status = call_external('libidl_stpdf.so','idl_chp_getpnn',$
                 	       id,tempnames,numpars)
        if status ne 0 then return
    endif
    parnames = parnames+tempnames;
    
; Set up the dataset specific metadata by querying each variable
; in turn
    
    varmeta = 0

    for i=0,numcols-1 do begin
        datatype = ' '
        ndims    = 0L
        dims     = lonarr(7)
        partype  = ' '
        temp     = ''
        temp1    = ''

; Get the details for column number i, including type, dimensions and units

    	colname = colnames[i]
        status = call_external('libidl_stpdf.so','idl_chp_getocd',$
                  	       id,colname,temp,ndims,dims,temp1)
        if status ne 0 then return

; Save data type and units
        datatype = ''+temp
        units    = ''+temp1

; Save column dimensions
    	dimensions = dims[0:(ndims-1)]
        num_elements = 1
        for j=0,ndims-1 do num_elements = num_elements * dims[j]

; If no units specified directly, try looking up the colname!units 
; parameter for the units

        if units eq 'none' then begin
            units = ''
            parname = colname+'!units'	
            status = call_external('libidl_stpdf.so','idl_chp_getpar',$
                                   id,parname,units)
            if status eq 0 then units = ''+units
        endif

; Use the colname!FILLVAL parameter to provide a null value

	parname = colname+'!FILLVAL'	
        status = call_external('libidl_stpdf.so','idl_chp_getopd', $
	                       id,parname,partype)
        if status eq 0 then begin
            sedat_make_variable, [1], partype, tnull
            status = call_external('libidl_stpdf.so','idl_chp_getpar',$
                   	           id,parname,tnull)
            if status ne 0 then return
            null = tnull
        endif else begin
            null = '*'
        endelse

; Use the colname!SI_conversion parameter to provide details of how to 
; convert units to SI units

        si_rel = ''
	parname = colname+'!SI_conversion'	
        status = call_external('libidl_stpdf.so','idl_chp_getpar',$
                               id,parname,si_rel)
        if status eq 0 then si_rel = ''+si_rel

; If a colname!LABLAXIS parameter exists then the value of this is retrieved
; to provide the axes_desc (axes description) details

        axis_label = '' 
	parname = colname+'!LABLAXIS'
        status = call_external('libidl_stpdf.so','idl_chp_getopd', $
	                       id,parname,partype)
    	single_label = (status eq 0)
    	if (single_label) then begin
            status = call_external('libidl_stpdf.so','idl_chp_getpar',$
                   	           id,parname,axis_label)
            if status ne 0 then return
            axis_label = ''+axis_label
    	endif else axis_label = colname

        axes_desc = axis_label
 
; The colname!VAR_TYPE parameter defines the type of the variable/column,
; whether data, support data or metadata

	var_type = ''
	parname = colname+'!VAR_TYPE'
        status = call_external('libidl_stpdf.so','idl_chp_getpar',$
               	               id,parname,temp)
        if status eq 0 then var_type = var_type+temp

; For array-valued 'data' columns
	if (var_type eq 'data' and num_elements gt 1) then begin
    	    for j=ndims,1,-1 do begin
    		jstr = strtrim(string(j),2)
    		axis_name = 'axis'+jstr

; Look for a colname!DEPEND_j variable
		axis_values = ''
                parname = colname+'!DEPEND_'+jstr
                status = call_external('libidl_stpdf.so','idl_chp_getpar',$
               	                        id,parname,temp)

; If found then set up axes_desc as a structure rather than a string
                if status eq 0 then begin
		    axis_values = axis_values+temp

; If a !LABLAXIS parameter was found then we have a simple label with 
; a pointer to another column given by the !DEPEND_j parameter for values 
; of dimension positions

    		    if (single_label) then begin
    			axes_desc = join_struct(axes_desc, $
    					create_struct(axis_name, $
    						      {label:axis_label, $
    						       values:axis_values}))
; Otherwise there should be a !LABL_PTR_j parameter which points to another
; column which contains labels for the dimension positions

    		    endif else begin
    			axis_labels = ''
    			parname = colname + '!LABL_PTR_'+jstr
    			status = call_external('libidl_stpdf.so','idl_chp_getpar',$
    					       id,parname,temp)
    			if status eq 0 then axis_labels = axis_labels+temp
    
    			axes_desc = join_struct(axes_desc, $
    						create_struct(axis_name, $
    							      {labels:axis_labels, $
    							       values:axis_values}))
    		    endelse
		end
    	    end
        endif

; Create the metadata structure for this column

    	sedat_make_meta, colname, datatype, ndims, dimensions, null, meta, $
    	    var_type=var_type, units=units, si_rel=si_rel, $
    	    axes_desc=axes_desc, $
    	    status=status, cmn=cmn

; Add the metadata for this column to the structure of all column metadata    

    	if status ne 0 then begin
            print, 'Failed to create metadata for variable '+colname+' in file '+file
    	endif else begin
            varmeta = join_struct(varmeta,create_struct(colname,meta))
    	endelse
    endfor

  ENDELSE

; Now create the dataset metadata

    status = 0
    sedat_make_dmeta, numents, varmeta, meta, nrv_vars, status=status, cmn=cmn

    if status ne 0 then begin
        print, 'Failed to create metadata for file '+file
        meta = 0
    endif
    meta.odi_db = oDB
    meta.odi_query = sql_query


; Set the id number to the SEDAT file number and the initial record number to 1

    meta.id  = file_number-1
    meta.rec = 1

; Mark the metadata structure as for an input file and save the STPDF id

    meta = create_struct(meta,'io','I');
    cmn.idsid[meta.id] = id

; Set the dataset name, to either the LRD name or the end of the dataset
; directory (the filename is always 'data' and the directory gives the name
; by which it is known to SEDAT users).

    if (type eq 'LRD') then begin
	 meta.name = file
    endif else begin
	 bits = str_sep(dir,'/')
	 meta.name = bits[size(bits,/n_elements)-1]
    endelse

    return

end
