<?php

/**
* php functions for the ODI database implementation in SPENVIS. 
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noordwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 31 May, 2010 
*
* @version $Id: odi.library.php 294 2010-04-06 11:20:55Z daniel $
*
* @package ODI
* @subpackage SPENVIS
*/

  require_once(realpath(dirname( __FILE__ ).'/../other/php/putils.php'));

  require_once(realpath(dirname( __FILE__ ).'/../other/php/SpenvisProperties.php'));

  require_once(realpath(dirname( __FILE__ )."/dbtools.php"));

  $link = connect_to_database();

  $dHandle = putils_fopen($p_type="dbc", $p_ext="txt", $p_mode="w");

//
// Retrieve the name of the selected dataset from the dblist namelist
//
  $nml = putils_get_namelist();
  $nml->set_line_regex('^([a-z][a-z0-9_#-]*)(\([1-9][0-9,]*\))? *= *([^ ].*)$');
  $nml->read_block("ODI");
  $primary = $nml->getvalues("PRIMARY");
  if ($primary == "")
    putils_err_write("No dataset was selected!");

//
// Retrieve metadata for the dataset
//
  $query = "SELECT ds.id, ds.name, " .
           "ds.epoch_start AS tmin, ds.epoch_end AS tmax, " .
           "dm.value AS description " .
           "FROM dataset AS ds LEFT JOIN dataset_metadata AS dm " .
           "ON (ds.id=dm.dataset_id) WHERE ds.name='" . $primary .
           "' AND dm.attribute='Logical_source_description'";
  $result = mysqli_query($link, $query);
  $row = mysqli_fetch_array($result, MYSQLI_ASSOC);
  $ds_id = $row["id"];
  $ds_desc = $row["description"];
  mysqli_free_result($result);

  fprintf($dHandle, "<H4>%s</H4>" . LF, $ds_desc);

//
// Start and stop time input boxes
//
  $sepoch = cdfepoch_to_date($row["tmin"]);
  $eepoch = cdfepoch_to_date($row["tmax"]);
  fprintf($dHandle, "<P><B>Time range:</B>" . LF);
  fprintf($dHandle, "<INPUT name='SEPOCH' size='20' value='" .
                    $sepoch[0] . "'> &ndash;" . LF);
  fprintf($dHandle, "<INPUT name='EEPOCH' size='20' value='" .
                    $eepoch[0] . "'>" . LF);

  fprintf($dHandle, "%s" . LF, "<P><TABLE border='0' cellspacing='2' celltabbing='4'>");

//
// Build a list of all record varying variables, excluding EPOCH
// variable descriptions are stored in $vars
// variable names are also stored in $channels and written to a hidden INPUT field
// 
//
  $query = "SELECT id, name, dims, sizes " .
           "FROM variable_metadata AS vm " .
           "WHERE UPPER(name)!='EPOCH' AND " .
           "dataset_id=" . $ds_id .
           " AND record_variance='T'";
  $result = mysqli_query($link, $query);
  $vars = array();
  while ($row = mysqli_fetch_array($result, MYSQLI_ASSOC))
  {
    if ($row["dims"] <= 1)
    {
      if (!isset($channels))
        $channels = strtolower($row["name"]);
      else
        $channels .= "," . strtolower($row["name"]);
      $vars[] = $row;
    }
  }
  mysqli_free_result($result);

  fprintf($dHandle, "%s" . LF, "<INPUT type='hidden' name='db_channels' " .
          "value='" . $channels . "'>");

  foreach ($vars as $var)
  {
    $fname = "";
    $ffill = "";
    $funits = "";
    $query = "SELECT name, value FROM variable_attribute " .
             "WHERE variable_metadata_id=" . $var["id"] .
             " AND (UPPER(LEFT(name, 8))='FIELDNAM' OR " .
             "UPPER(name)='FILLVAL' OR UPPER(name)='UNITS') " .
             "AND CHAR_LENGTH(TRIM(value))>0 " .
             "ORDER BY name";
    $result = mysqli_query($link, $query);
    while ($row = mysqli_fetch_array($result, MYSQLI_ASSOC))
    {
      if (strtoupper($row["name"]) == "FIELDNAM")
        $fname = $row["value"];
      else if (strtoupper($row["name"]) == "FILLVAL")
        $ffill = $row["value"];
      else if (strtoupper($row["name"]) == "UNITS")
        $funits = $row["value"];
    }
    mysqli_free_result($result);
    if ($fname == "") $fname = $var["name"];
    if ($ffill == "") $ffill = -1.0e20;

// Variable id is used as checkbox value to flag selected variables
    fprintf($dHandle, "%s" . LF, "<TR><TD align='left' valign='top'>" .
            "<INPUT type=\"checkbox\" name=\"" . strtolower($var["name"]) .
            "\" value=\"" . $var["id"] . "\" datatype=\"integer\"></TD>");
    fprintf($dHandle, "%s%s", "<TD align='left'>", $fname);
    if (strlen(trim($funits)) > 0)
      fprintf($dHandle, "%s", " [" . $funits . "]");
    fprintf($dHandle, LF);
// Output variable dims, units and fillval to namelist
    fprintf($dHandle, "%s" . LF, "<INPUT type=\"hidden\" name=\"" .
                      strtolower($var["name"]) . "_dims\" value=\"" .
                      $var["dims"] . "\" datatype=\"integer\">");
    fprintf($dHandle, "%s" . LF, "<INPUT type='hidden' name='" .
                      strtolower($var["name"]) . "_units' value='" .
                      $funits . "'>");
    fprintf($dHandle, "%s" . LF, "<INPUT type='hidden' name='" .
                      strtolower($var["name"]) . "_fill' value='" .
                      $ffill . "'>");

    $ic = 0;
    if ($var["dims"] == 0)
    {
// For scalars, write LABLAXIS, LABL_PTR_1 or FIELDNAM as variable label
      $flabel = $fname;
// Retrieve LABLAXIS if it is defined
      $query = "SELECT value FROM variable_attribute " .
               "WHERE name='LABLAXIS' AND " .
               "variable_metadata_id=" . $var["id"];
      $result = mysqli_query($link, $query);
      if (mysqli_num_rows($result) > 0)
      {
        $row = mysqli_fetch_array($result, MYSQLI_ASSOC);
        $flabel = $row["value"];
        mysqli_free_result($result);
      }
      else
      {
        mysqli_free_result($result);
// Retrieve the name of the LABEL_PTR_1 variable
        $query = "SELECT value FROM variable_attribute " .
                 "WHERE name='LABL_PTR_1' AND " .
                 "variable_metadata_id=" . $var["id"];
        $result = mysqli_query($link, $query);
        if (mysqli_num_rows($result) == 0)
          mysqli_free_result($result);
        else
        {
          $row = mysqli_fetch_array($result, MYSQLI_ASSOC);
          mysqli_free_result($result);
          $vname = $row["value"];
// Read the LABL_PTR_1 labels
          $query = "SELECT vn.value FROM variable_nrv AS vn " .
                   "LEFT JOIN variable_metadata AS vm ON " .
                   "(vn.variable_metadata_id=vm.id) " .
                   "WHERE vm.name='" . $vname . "' AND " .
                   "vm.dataset_id=" . $ds_id .
                   " ORDER BY vn.ind";
          $result = mysqli_query($link, $query);
          $row = mysqli_fetch_array($result, MYSQLI_ASSOC);
          mysqli_free_result($result);
          $flabel = $row["value"];
        }
      }
      fprintf($dHandle, "%s" . LF, "<INPUT type='hidden' name='" .
                        strtolower($var["name"]) . "_label' value='" .
                        remove_quotes($flabel) . "'>");
    }
    else if ($var["dims"] == 1)
    {
//
// For array variables, SIZES and and label for each index (LABL_PTR_1) need to be stored
//
      fprintf($dHandle, "%s" . LF, "<INPUT type=\"hidden\" name=\"" .
                        strtolower($var["name"]) . "_sizes\" value=\"" .
                        $var["sizes"] . "\" datatype=\"integer\">");
// Retrieve the name of the LABEL_PTR_1 variable
      $query = "SELECT value FROM variable_attribute " .
               "WHERE name='LABL_PTR_1' AND " .
               "variable_metadata_id=" . $var["id"];
      $result = mysqli_query($link, $query);
      if (mysqli_num_rows($result) == 0)
        mysqli_free_result($result);
      else
      {
        $row = mysqli_fetch_array($result, MYSQLI_ASSOC);
        mysqli_free_result($result);
        $vname = $row["value"];
// Read the LABL_PTR_1 labels
        $query = "SELECT vn.value FROM variable_nrv AS vn " .
                 "LEFT JOIN variable_metadata AS vm ON " .
                 "(vn.variable_metadata_id=vm.id) " .
                 "WHERE vm.name='" . $vname . "' AND " .
                 "vm.dataset_id=" . $ds_id .
                 " ORDER BY vn.ind";
        $result = mysqli_query($link, $query);
        while ($row = mysqli_fetch_array($result, MYSQLI_ASSOC))
        {
//
// Identify the array index for labels by $ic
// Use the variable name and _$ic for checkbox names and store
// LABL_PTR_1 values as variable labels
          $ic++;
          fprintf($dHandle, "%s" . LF, "<BR>" .
                  "<INPUT type=\"checkbox\" name=\"" .
                  strtolower($var["name"]) . "_" . $ic .
                  "\" value=\"1\" datatype=\"integer\">");
          fprintf($dHandle, "%s" . LF, $row["value"]);
          fprintf($dHandle, "%s" . LF,
                  "<INPUT type='hidden' name='" .
                  strtolower($var["name"]) . "_label_" . $ic .
                  "' value='" . remove_quotes($row["value"]) . "'>");
        }
        mysqli_free_result($result);
      }
    }
    fprintf($dHandle, "%s" . LF, "</TD></TR>");
  }

  fprintf($dHandle, "%s" . LF, "</TABLE><P>");

// Add the dataset description to the namelist through a hidden INPUT field
  fprintf($dHandle, "<INPUT type='hidden' name='DATA_DESC' value='%s'>" . LF,
                    $ds_desc);
  fclose($dHandle);

  mysqli_close($link);

?>