<?php

/**
* php functions for the ODI database implementation in SPENVIS. 
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noordwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 31 May, 2010 
*
* @version $Id: odi.library.php 294 2010-04-06 11:20:55Z daniel $
*
* @package ODI
* @subpackage SPENVIS
*/

  require_once(realpath(dirname( __FILE__ ).'/../other/php/putils.php'));

  require_once(realpath(dirname( __FILE__ ).'/../other/php/SpenvisProperties.php'));

  require_once(realpath(dirname( __FILE__ )."/dbtools.php"));

  $nml = putils_get_namelist();
  $nml->set_line_regex('^([a-z][a-z0-9_#-]*)(\([1-9][0-9,]*\))? *= *([^ ].*)$');
  $nml->read_block("ODI");
  $channels = $nml->getvalues("db_channels");
  $channels = explode(",", $channels);

  $nmlvars = $nml->list_vars();

//
// Read the channel information from the namelist.
// All channels are always listed in the namelist, but only does that
// were selected by the user will be added to $channels.
// Selected channels are identified by a namelist entry like
//    b_meas_vec = 104
// i.e., each selected channel is written to the namelist with it's
// id number from the database.
//
  $nvars = 0;
  $dvars = array();
  $dims = array();
  $sizes = array();
  $units = array();
  $labels = array();
  $cols = array();
  $fills = array();
  foreach ($channels as $channel)
  {
    if ($nml->getvalues($channel) > 0)
    {
      $dvars[] = $channel;
      $dim = $nml->getvalues($channel . "_dims");
      $dims[] = $dim;
      if ($dim == 0)
      {
        $sizes[] = 1;
        $labels[] = $nml->getvalues($channel . "_label");
        $cols[] = array(strtolower($channel));
      }
      else
      {
        $size = $nml->getvalues($channel . "_sizes");
        $dlabel = array();
        $dcol = array();
        for ($i = 1; $i <= $size; $i++)
        {
          if (in_array(strtolower($channel) . "_" . $i, $nmlvars))
          {
            $dlabel[] = $nml->getvalues($channel . "_label" . "_" . $i);
            $dcol[] = strtolower($channel) . "_" . $i;
          }
        }
        $labels[] = $dlabel;
        $cols[] = $dcol;
        $sizes[] = count($dcol);
      }
      $units[] = $nml->getvalues($channel . "_units");
      $fills[] = $nml->getvalues($channel . "_fill");
    }
  }

//
// Build a variable list
//
  $nv = 1;
  $nc = 1;
  $nl = 3;
  $vars = array();
  for ($i = 0; $i < count($dvars); $i++)
  {
    if ($sizes[$i] > 0)
    {
      $vars[] = array("name" => $dvars[$i],
                      "dim" => $dims[$i],
                      "size" => $sizes[$i],
                      "units" => $units[$i],
                      "fills" => $fills[$i],
                      "labels" => $labels[$i],
                      "cols" => $cols[$i]);
      $nv++;
      $nc += $sizes[$i];
      if ($dims[$i] > 0)
        $nl++;
    }
  }

  if (count($vars) == 0)
  {
    $wHandle = fopen("spenvis.war", "w");
    fprintf($wHandle, "<P><FONT color='red'>" .
                      "<B>No channels were selected!</B></FONT>" . LF);
    fclose($wHandle);
  }

// Open the output file spenvis_dbo.txt
  $oHandle = putils_fopen($p_type="dbo", $p_ext="txt", $p_mode="w");

// Write the fixed file headers
  fprintf($oHandle, "'*'" . str_repeat(",%6d", 8) . LF,
          2+$nl+$nv, 1, $nl, 0, $nv, $nc, -1, 0);
  fprintf($oHandle, "'SPENVIS - %s'" . LF, date("d-M-Y H:i:s"));
  fprintf($oHandle, "'PRJ_DEF', -1,'%s'" . LF,
          $nml->getvalues("PRODEF"));
  fprintf($oHandle, "'PLT_HDR', -1,'%s'" . LF,
          $nml->getvalues("DATA_DESC"));

// Build a list of fill values
  $fills = sprintf("'FILLVALS', %d, -1.0E31", $nc);
  foreach ($vars as $var)
  {
    $fills .= "," . implode(",", array_fill(0, $var["size"], strval($var["fills"])));
    if ($var["dim"] > 0)
      fprintf($oHandle, "'%s', -%d,'%s'" . LF,
              $var["name"] . "_labels", $var["size"],
              implode("','", $var["labels"]));
  }
  fprintf($oHandle, "%s" . LF, $fills);

// Epoch is always output
  fprintf($oHandle, "'MJD','days', 1,'Modified Julian Day'" . LF);

// Write the header line for each variable
  foreach ($vars as $var)
  {
    fprintf($oHandle, "'%s','%s', %d,",
            $var["name"], $var["units"], $var["size"]);
    if ($var["dim"] > 0)
      fprintf($oHandle, "'','%s'" . LF, $var["name"] . "_labels");
    else
      fprintf($oHandle, "'%s'" . LF, $var["labels"]);
  }


  $link = connect_to_database();

//
// Build the query to retrieve the selected data fields
// A maximum of 2500 records is retrieved (this can be changed by
// the LIMIT parameter at the end of the query
//
  $query = "SELECT cdf_epoch";
  foreach ($vars as $var)
    $query .= ',' . implode(",", $var["cols"]);
  $query .= " FROM " . $nml->getvalues("DATA_TABLE") .
            " WHERE cdf_epoch>='" . date_to_cdfepoch($nml->getvalues("SEPOCH"), 0) .
            "' AND cdf_epoch<='" . date_to_cdfepoch($nml->getvalues("EEPOCH"), 0) .
            "' ORDER BY cdf_epoch LIMIT 2500";

  $result = mysqli_query($link, $query);
  if (mysqli_num_rows($result) == 0)
  {
    $wHandle = fopen("spenvis.war", "a");
    fprintf($wHandle, "<P><FONT color='red'>" .
                      "<B>No data were found in the specified epoch range!</B></FONT>" . LF);
    fclose($wHandle);
  }

// Convert cdf epochs to AMJD
  $cdfepoch_1950 = 61536067200000.0;
  while ($row = mysqli_fetch_array($result, MYSQLI_NUM))
  {
    $amjd = ($row[0]-$cdfepoch_1950) / 86400000.0;
    fprintf($oHandle, "%14.6f", $amjd);
    for ($i = 1; $i < count($row); $i++)
      fprintf($oHandle, ",%14.5e", $row[$i]);
    fprintf($oHandle, LF);
  }
  mysqli_free_result($result);

  fprintf($oHandle, "'End of File'" . LF);

  fclose($oHandle);

  mysqli_close($link);


// Write channel information for inclusion in db_out.html template
  $dHandle = putils_fopen($p_type="dbf", $p_ext="txt", $p_mode="w");

  fprintf($dHandle, "<SELECT name='VARNAME'>" . LF);
  foreach ($vars as $var)
  {
    if ($var["dim"] == 0)
      fprintf($dHandle, "  <OPTION value='" . $var["name"] . "'>" .
                        $var["labels"] . "</OPTION>" . LF);
    else
      for ($i = 0; $i < $var["size"]; $i++)
        fprintf($dHandle, "  <OPTION value='" . $var["name"] . "[" . $i .
                          "]'>" . $var["labels"][$i] . "</OPTION>" . LF);
  }

  fprintf($dHandle, "</SELECT>" . LF);

  fclose($dHandle);

?>