<?php

/**
* php functions for the ODI database implementation in SPENVIS. 
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noordwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 31 May, 2010 
*
* @version $Id: odi.library.php 294 2010-04-06 11:20:55Z daniel $
*
* @package ODI
* @subpackage SPENVIS
*/


  require_once(realpath(dirname( __FILE__ ).'/../other/php/putils.php'));

  require_once(realpath(dirname( __FILE__ ).'/../other/php/SpenvisProperties.php'));

  require_once(realpath(dirname( __FILE__ )."/dbtools.php"));

  $link = connect_to_database();

  $dHandle = putils_fopen($p_type="dbl", $p_ext="txt", $p_mode="w");

  fprintf($dHandle, "%s" . LF, "<CENTER><TABLE border='1' cellspacing='4' cellpadding='4'>");
  fprintf($dHandle, "%s" . LF, "<CAPTION><B>Available data sets</B></CAPTION>");

//
// Build a list of dataset metadata
//
  $query = "SELECT ds.name AS name, ds.sedat_name AS sedat_name, " .
           "ds.epoch_start AS tmin, ds.epoch_end AS tmax, " .
           "dm.value AS description, dd.value AS platform_type " .
           "FROM dataset AS ds LEFT JOIN dataset_metadata AS dm " .
           "ON (ds.id=dm.dataset_id) LEFT JOIN dataset_metadata AS dd " .
           "ON (ds.id=dd.dataset_id) WHERE " .
           "dm.attribute='Logical_source_description' AND " .
           "dd.attribute='platform_type' AND " .
           "ds.availability='public' ORDER BY ds.sedat_name ASC";
  $result = mysqli_query($link, $query);
  $lines = array();
  while ($row = mysqli_fetch_array($result, MYSQLI_ASSOC))
    $lines[] = $row;
  mysqli_free_result($result);

//
// Build a table with a list of datasets, separated into virtual
// instruments (indices) and 'real' instruments
//
  fprintf($dHandle, "<TR><TH colspan='4'>Magnetospheric and solar indices</TH></TR>" . LF);
  fprintf($dHandle, "%s" . LF, "<TR><TH colspan='2'>Dataset</TH><TH>Start Time</TH><TH>End Time</TH></TR>");
  foreach ($lines as $line)
  {
    if ($line["platform_type"] == "virtual")
    {
      $tmin = cdfepoch_to_date($line["tmin"]);
      $tmax = cdfepoch_to_date($line["tmax"]);
      fprintf($dHandle, "<TR><TD align='center'>" .
                        "<INPUT type='radio' name='PRIMARY' value='%s" .
                        "'></TD>" . LF, $line["name"]);
      fprintf($dHandle, "<TD align='left'>%s</TD>" . LF,
                        "<SPAN title='" . $line["description"] .
                        "'>" . $line["sedat_name"] . "</SPAN>");
      fprintf($dHandle, "<TD align='left' nowrap>%s</TD>" . LF, $tmin[0]);
      fprintf($dHandle, "<TD align='left' nowrap>%s</TD></TR>" . LF, $tmax[0]);
    }
  }

  fprintf($dHandle, "<TR><TH colspan='4'>Satellite data</TH></TR>" . LF);
  fprintf($dHandle, "%s" . LF, "<TR><TH colspan='2'>Dataset</TH><TH>Start Time</TH><TH>End Time</TH></TR>");
  foreach ($lines as $line)
  {
    if ($line["platform_type"] != "virtual")
    {
      $tmin = cdfepoch_to_date($line["tmin"]);
      $tmax = cdfepoch_to_date($line["tmax"]);
      fprintf($dHandle, "<TR><TD align='center'>" .
                        "<INPUT type='radio' name='PRIMARY' value='%s" .
                        "'></TD>" . LF, $line["name"]);
      fprintf($dHandle, "<TD align='left'>%s</TD>" . LF,
                        "<SPAN title='" . $line["description"] .
                        "'>" . $line["sedat_name"] . "</SPAN>");
      fprintf($dHandle, "<TD align='left' nowrap>%s</TD>" . LF, $tmin[0]);
      fprintf($dHandle, "<TD align='left' nowrap>%s</TD></TR>" . LF, $tmax[0]);
    }
  }

  fprintf($dHandle, "%s" . LF, "</TABLE></CENTER><P>");

  fclose($dHandle);

  mysqli_close($link);

?>