<?php

/**
* php functions for the ODI database implementation in SPENVIS. 
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noordwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 31 May, 2010 
*
* @version $Id: odi.library.php 294 2010-04-06 11:20:55Z daniel $
*
* @package ODI
* @subpackage SPENVIS
*/


// Set parameter values dependent on operating system
  if (strtoupper(substr(PHP_OS, 0, 3)) === "WIN")
  {
    define("LF", "\r\n");
  }
  else
  {
    define("LF", "\n");
  }

//////
// Connect to database.
//////
  function connect_to_database()
  {

    $prop = new SpenvisProperties($t_file="ODI.properties");

    $db_user = $prop->get_var("ODI_USER");
    if ($db_user === FALSE) putils_err_write("ODI_USER is not defined!");

    $db_pwd = $prop->get_var("ODI_PWD");
    if ($db_pwd === FALSE) putils_err_write("ODI_PWD is not defined!");

    $db_name = $prop->get_var("ODI_DB");
    if ($db_name === FALSE) putils_err_write("ODI_DB is not defined!");

    $db_host = $prop->get_var("ODI_HOST");
    if ($db_host === FALSE) putils_err_write("ODI_HOST is not defined!");

    $db_port = $prop->get_var("ODI_PORT");
    if ($db_port === FALSE) $db_port = NULL;

    $db_socket = $prop->get_var("ODI_SOCKET");
    if ($db_socket === FALSE) $db_socket = NULL;

    $conn = mysqli_init();
    if (!$conn) putils_err_write("Failed to initialize database connection");
    if (!$conn->real_connect($db_host, $db_user, $db_pwd, $db_name,
                             $port=$db_port, $socket=$db_socket))
      putils_err_write("Unable to connect to database");

    return $conn;
  }


  function date_to_cdfepoch($date, $msec)
  {
    $cdf_1970 = 62167219200000.0;

    $epoch = 1000.0 * gmmktime(substr($date, 11, 2), substr($date, 14, 2), substr($date, 17, 2),
                               substr($date, 5, 2), substr($date, 8, 2), substr($date, 0, 4));
    $epoch += $cdf_1970 + $msec;

    return $epoch;
  }
  

  function cdfepoch_to_date($cdf_epoch)
  {
    $epoch = $cdf_epoch - 62167219200000.0;
    $time = (int) ($epoch/1000.0);
    $ms = (int) ($epoch - $time * 1000.0);
    if ($ms < 0)
    {
      $time -= 1;
      $ms += 1000;
    }
    $date = gmdate("Y-m-d H:i:s", $time);
    return array($date, $ms);
  }


  function remove_quotes($s)
  {
    $s = strtr($s, array("'" => "", "\"" => ""));
    return $s;
  }


?>