function xformat, axis, index, value
;------------------------------------------------------------------------------
; This function returns the formatted labels for the X-scale in the format
; yyyy-mm-dd<CR>hh:mm:ss
;------------------------------------------------------------------------------
; PARAMETERS:
; -----------
; The function is called by specifying:
;   axis, ... ,xtickformat='xformat'
;------------------------------------------------------------------------------
  cdf_epoch,base,1950,1,1,0,0,0,0,/compute_epoch
  epoch=base+value*86.4d6
  cdf_epoch,epoch,year,month,day,hour,min,sec,/breakdown_epoch
  Return,string(year,'-',month,'-',day,'!C',hour,':',min,':',sec, $
         format="(I4.4,5(A,i2.2))")
end

PRO XYPlot, file_name, format, xystr, x, y, ylog=ylog, fmin=fmin, $
            ytitle=ytitle, plthdr=plthdr
;
; History: March 2010: created by D. Heynderickx (DH Consultancy)
;

set_device, File_Name, Format
nwloadct, 0

if (n_elements(fmin) gt 0) then vmin = fmin
scale,y,log=ylog,min_value=vmin,style=20,tick_get=ylevels,range=y_range,/nodata

!p.position = [0.15,0.1,0.95,0.95]
plot, x, y, ytype=ylog, xstyle=2, yrange=y_range, min_value=vmin, $
      ytickv=ylevels, ystyle=18, ytitle=ytitle, xtickformat='xformat', $
      xcharsize=0.8, /nodata

ind = where((y ge y_range(0)) and (y le y_range(1)))
if (ind(0) lt 0) then begin
  xyouts,0.55,0.525,'No values in range',align=0.5,/normal,width=vwidth
  xcsize = float(!D.X_CH_SIZE)/!D.X_VSIZE
  ycsize = float(!D.Y_CH_SIZE)/!D.Y_VSIZE
  plots,0.55+vwidth*[-.5,.5,.5,-.5,-.5]+xcsize*[-2,2,2,-2,-2], $
        0.525+ycsize*[-1,-1,1.5,1.5,-1],/normal
endif else begin
  nf = size(np)
  if (nf(0) eq 0) then begin
    nf = 1
    np = n_elements(x)
  endif else begin
    nf = nf(1)
  endelse
  nd = size(y)
  if (nd(0) eq 1) then nd = 0 else nd = nd(2) - 1
  if (n_elements(psym) eq 0) then begin
    psym = intarr(nd+1)
  endif else begin
    ns = size(psym)
    if (ns(0) eq 0) then psym = intarr(nd+1) + psym
  endelse
  for j=0,nd do begin
    z = y(*,j)
    ns = 0
    for i=0,nf-1 do begin
      nl = ns + np(i) - 1
      u = x(ns:nl)
      v = z(ns:nl)
      ind = where((v ge y_range(0)) and (v le y_range(1)))
      if (ind(0) ge 0) then oplot,u,v,min_value=vmin,psym=psym(j)
      ns = nl + 1
    endfor
  endfor
endelse

if (n_elements(plthdr) gt 0) then $
  xyouts,0.55,0.975,plthdr,/normal,align=0.5

PS_Annot, extract=xystr

!p.position = 0

close_device, File_Name, Format

END


PRO db_plot

  Namelist = read_nml('idl')

  IF (isDefined('FORMAT',Namelist)) THEN BEGIN
    format = STR_SEP(NMLValue('FORMAT',Namelist),'''')
    format = format[1]
  ENDIF ELSE $
    errorMessage,'Unknown graphical format'

  IF (isDefined('LOG',Namelist)) THEN $
    log = FIX(NMLValue('LOG',Namelist))

  IF (isDefined('VARNAME',Namelist)) THEN BEGIN
    varname = STR_SEP(NMLValue('VARNAME',Namelist),'''')
    varname = varname[1]
  ENDIF ELSE $
    errorMessage,'Plot variable not defined'

  var_read, ['MJD',varname], varstr, var, legend, np, $
            file='spenvis_dbo.txt'

  get_meta, varstr, 'PLT_HDR', plthdr


  XYPlot, 'dbplot', format, varstr, var[*,0], var[*,1], $
          ylog=log, fmin=fmin, ytitle=STRCOMPRESS(STRTRIM(legend[1], 2)), $
          plthdr=STRCOMPRESS(STRTRIM(plthdr, 2))

END