<?php
/**
* Parser function for real time ACE MAG data.
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noorwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 10 June, 2010 
*
* @version $Id: ACE_MAG_RT.php 326 2010-06-17 13:08:15Z peter $
*
* @package ODI
* @subpackage ParsersFunctions
*/


function dirscan_ACE_MAG_RT($basedir,$filepattern)
{
    // ACE SWEPAM data.
    $url = "http://www.swpc.noaa.gov/ftpdir/lists/ace/ace_mag_1m.txt";
    $dfiles = array();
    $dfiles[] = array( "ace_mag_1m_".date("H"), date("Y/m/d H:i:s"), $url );
    return $dfiles;
}


// Parse a data file and write formatted content to load.tmp
function parse_ACE_MAG_RT($rname, $fid)
{

    $ms = 0;
    $all_rows = file($rname);
    $tfile = fopen("load.tmp", "w");
    $result = FALSE;
    if ( $all_rows !== FALSE ) {
        $result = TRUE;
        foreach( $all_rows as $row) {
            if(!preg_match("/^[:#]/",$row)) {
                $token = preg_split("/\s+/",$row);
                $epoch = $token[0]."-".$token[1]."-".$token[2]." ";
                $epoch.= substr($token[3],0,2).":".substr($token[3],2,2).":00";
                $cdf_epoch = date_to_cdfepoch($epoch, $ms);
                fprintf($tfile, "%20.3f, %s, %d",
                            $cdf_epoch, $epoch, $ms);
                // Bx, By, Bz.
                for($i=7;$i<10;$i++) {
                    fprintf($tfile, ", %f", $token[$i]);
                }
                fprintf($tfile, ", %d%s",$fid, LF);
            }
        }
    }
    
    fclose($tfile);
    
    return $result;

}

?>