<?php
/**
* Parser function for AZUR.
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noorwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 2 December, 2009 
*
* @version $Id: AZUR_EI_88.php 252 2009-12-17 20:52:43Z peter $
*
* @package ODI
* @subpackage ParsersFunctions
*/

// Parse a data file and write formatted content to load.tmp
  function parse_AZUR_EI_88($rname, $fid)
  {
// Indices for re-ordering proton and alpha fluxes
    $pind = array(0, 2, 3, 4, 5, 6);
// Energy bin widths (MeV)
    $ewidth = array(1.2, 13.0, 2.5, 5.2, 11.6, 26.8, 55.2);
    $G = array();
// Geometric factors (cm^2 sr) for EI-88/1
    $G[] = array(0.0580, 0.0580, 0.0580, 0.0580, 0.0580, 0.0612, 0.0772);
// Geometric factors (cm^2 sr) for EI-88/2
    $G[] = array(0.0595, 0.0595, 0.0595, 0.0595, 0.0595, 0.0628, 0.0792);

    $bname = str_replace("CLEAN", "BL", $rname);
    $vHandle = fopen($rname, "r");
    $bHandle = fopen($bname, "r");
    $tHandle = fopen("load.tmp", "w");
    if (($vHandle !== FALSE) && ($bHandle !== FALSE) && ($tHandle !== FALSE))
    {
// Read header line
      $vdum = fgets($vHandle);
      while (!feof($vHandle))
      {
        $vdum = fgets($vHandle);
        $bdum = fgets($bHandle);
        if (strlen($vdum) > 100)
        {
          $vdum = preg_split("/[\s]+/", trim($vdum));
          $quality = $vdum[0];
          $year = $vdum[1];
          $day = $vdum[2];
          $ms = $vdum[3];
          $sec = intval(floor($ms/1000));
          $ms -= $sec * 1000;
          $dtime = gmmktime(0, 0, 0, 1, 1, $year) + ($day-1) * 86400 + $sec;
          $epoch = gmdate("Y-m-d H:i:s", $dtime);
          $cdf_epoch = date_to_cdfepoch($epoch, $ms);

          fprintf($tHandle, "%20.3f,%s,%3d,%1d", $cdf_epoch, $epoch, $ms, $quality);
// Add position and (B,L)
          $pos = implode(",", preg_split("/[\s]+/", trim(substr($bdum, 0, 51))));
          fprintf($tHandle, ",%s", $pos);
// Add measured magnetic field vector
          for ($i = 0; $i < 3; $i++) fprintf($tHandle, ",%s", $vdum[27+$i]);
// Convert and add fluxes
          for ($id = 0; $id < 2; $id++)
          {
            $pflux = array();
            for ($i = 0; $i < 6; $i++)
            {
              if ($vdum[31+8*$id+$pind[$i]] < 0)
                $pflux[] = -1.0e31;
              else
                $pflux[] = $vdum[31+8*$id+$pind[$i]] / $G[$id][$pind[$i]] / $ewidth[$pind[$i]] / 10.0;
            }
            if ($vdum[32+8*$id] < 0)
              $aflux = -1.0e31;
            else
              $aflux = $vdum[32+8*$id] / $G[$id][1] / $ewidth[1] / 10.0;
            foreach ($pflux as $flux)
              fprintf($tHandle, ",%14.6e", $flux);
            fprintf($tHandle, ",%14.6e", $aflux);
          }
          fprintf($tHandle, ",%d%s", $fid, LF);
        }
      }
      fclose($vHandle);
      fclose($bHandle);
      fclose($tHandle);

      $result = TRUE;
    }
    else
      $result = FALSE;

    return $result;
  }

?>