<?php
/**
* Parser function for real time GOES proton and electron data.
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noorwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 10 June, 2010 
*
* @version $Id: GOES_SEM_RT.php 326 2010-06-17 13:08:15Z peter $
*
* @package ODI
* @subpackage ParsersFunctions
*/


function dirscan_GOES_SEM_RT($basedir,$filepattern)
{
    // Primary GOES satellite for particle data.
    $url = "http://www.swpc.noaa.gov/ftpdir/lists/particle/Gp_part_5m.txt";
    $dfiles = array();
    $dfiles[] = array( "Gp_part_5m_".date("H"), date("Y/m/d H:i:s"), $url );
    return $dfiles;
}


// Parse a data file and write formatted content to load.tmp
function parse_GOES_SEM_RT($rname, $fid)
{
    
    $r_geo = 42171.0; // km.
    
    $ms = 0;
    $all_rows = file($rname);
    $tfile = fopen("load.tmp", "w");
    $sat_num = 0;
    $pos_x = 0.0;
    $pos_y = 0.0;
    $pos_z = 0.0;
    $result = FALSE;
    if ( $all_rows !== FALSE ) {
        $result = TRUE;
        foreach( $all_rows as $row) {
            if(preg_match("/^[:#]/",$row)) {
                if(preg_match("/#\s+Source:\s+GOES-(\d+)/",$row,$match)) {
                    $sat_num = $match[1];
                }
                if(preg_match("/#\s+Location:\s+W(\d+)/",$row,$match)) {
                    $longitude = deg2rad(360 - $match[1]);
                    $pos_x = $r_geo * cos($longitude);
                    $pos_y = $r_geo * sin($longitude);
                }
            }
            else {
                $token = preg_split("/\s+/",$row);
                $epoch = $token[0]."-".$token[1]."-".$token[2]." ";
                $epoch.= substr($token[3],0,2).":".substr($token[3],2,2).":00";
                $cdf_epoch = date_to_cdfepoch($epoch, $ms);
                fprintf($tfile, "%20.3f, %s, %d, %d, %f, %f, %f",
                            $cdf_epoch, $epoch, $ms, $sat_num, $pos_x, $pos_y, $pos_z);
                // First electrons ...
                for($i=12;$i<15;$i++) {
                    fprintf($tfile, ", %f", $token[$i]);
                }
                // ... then protons.
                for($i=6;$i<12;$i++) {
                    fprintf($tfile, ", %f", $token[$i]);
                }
                fprintf($tfile, ", %d%s",$fid, LF);
            }
        }
    }
    
    fclose($tfile);
    
    return $result;
    
}

?>