<?php

/**
* Parser function for HELIOS E7.
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noordwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 2 December, 2009 
*
* @version $Id: HELIOS_E7.php 224 2009-12-03 12:49:21Z peter $
*
* @package ODI
* @subpackage ParsersFunctions
*/

// Parse a data file and write formatted content to load.tmp
  function parse_HELIOS_E7($rname, $fid)
  {
    $tHandle = fopen($rname, "r");
    if ($tHandle !== FALSE)
    {
      $tfile = fopen("load.tmp", "w");
      while (!feof($tHandle))
      {
        $rec = fgets($tHandle);
        $rec = preg_split("/[\s]+/", trim($rec));
        if (count($rec) > 1)
        {
          $cdf_epoch = $rec[0];
          list($epoch, $ms) = cdfepoch_to_date($cdf_epoch);
          fprintf($tfile, "%20.1f, %s, %d", $cdf_epoch, $epoch, $ms);
// Convert to Cartesian coordinates
          $lon = deg2rad($rec[3]);
          $lat = deg2rad($rec[2]);
          $alt = $rec[1] + 6371.2;
          $x = $alt * cos($lon) * cos($lat);
          $y = $alt * sin($lon) * cos($lat);
          $z = $alt * sin($lat);
          fprintf($tfile, str_repeat(",%.6e", 3), $x, $y, $z);
// B, L, Pitch angle
          fprintf($tfile, ",%s,%s,%s", $rec[5], $rec[6], $rec[4]);
          for ($i = 0; $i < 17; $i++)
            fprintf($tfile, ",%s", $rec[7+$i]);

          fprintf($tfile, ", %d%s", $fid, LF);
        }
      }
      fclose($tfile);
      $result = TRUE;
    }
    else
      $result = FALSE;

    return $result;
  }

?>