<?php

/**
* Parser function for IMP8 CPME.
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noorwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 2 December, 2009 
*
* @version $Id: GOES_SEM.php 224 2009-12-03 12:49:21Z peter $
*
* @package ODI
* @subpackage ParsersFunctions
*/


// Parse a data file and write formatted content to load.tmp
  function parse_IMP8_CPME($rname, $fid)
  {
    $ms = 0;
// Most files are gzipped, but not all of them...
    if (substr($rname, -3) == ".gz")
    {
      exec("gzip -d -c " . $rname . " > tmp.txt");
      $cpme = file("tmp.txt");
    }
    else
      $cpme = file($rname);
    if ($cpme !== FALSE)
    {
      $tfile = fopen("load.tmp", "w");
      array_shift($cpme);
      $rec = array_shift($cpme);
      $ncol = count(explode("_fx", $rec)) - 1;

      foreach ($cpme as $rec)
      {
        list($year, $doy, $hour, $min, $sec) = sscanf($rec, " %4d %3d %2d %2d %2d");
        $ts = gmmktime($hour, $min, $sec, 1, 1, $year) + ($doy-1) * 86400;
        $epoch = gmdate("Y-m-d H:i:s", $ts);
        $cdf_epoch = date_to_cdfepoch($epoch, $ms);

        $vals = array();
        for ($j = 0; $j < $ncol+5; $j++)
          $vals[] = trim(substr($rec, 41+$j*10, 9));

        fprintf($tfile, "%20.3f, %s, %d", $cdf_epoch, $epoch, $ms);
        for ($i = 0; $i < $ncol+5; $i++)
          fprintf($tfile, ",%.4e", (($i > 4) && ($vals[$i] < 0.0)) ? -1.0e31 : $vals[$i]);
        fprintf($tfile, ", %d%s", $fid, LF);
      }
      fclose($tfile);
      @unlink("tmp.txt");

      $result = TRUE;
    }
    else
      $result = FALSE;

    return $result;
  }

?>