<?php

/**
* Parser function for IMP8 CRNC.
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noorwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 2 December, 2009 
*
* @version $Id: GOES_SEM.php 224 2009-12-03 12:49:21Z peter $
*
* @package ODI
* @subpackage ParsersFunctions
*/

// Parse a data file and write formatted content to load.tmp
  function parse_IMP8_CRNC($rname, $fid)
  {
    $ms = 0;
    $crnc = file($rname);
    if ($crnc !== FALSE)
    {
      $tfile = fopen("load.tmp", "w");
      foreach ($crnc as $rec)
      {
        $quality = substr($rec, 0, 2);
        $year = substr($rec, 2, 5);
        $month = substr($rec, 17, 3);
        $day = substr($rec, 20, 3);
        $secs = substr($rec, 11, 6);
        $mins = $secs / 60;
        $hour = intval(floor($mins/60.0));
        $min = $mins - $hour * 60;
        $epoch = sprintf("%04d-%02d-%02d %02d:%02d:00", $year, $month, $day, $hour, $min);
        $cdf_epoch = date_to_cdfepoch($epoch, $ms);

        if ($quality == 0)
          $vals = array_fill(0, 58, -999);
        else
        {
          $vals = array();
          for ($i = 0; $i < 13; $i++)
          {
// Time coverage
            $vals[$i] = substr($rec, 23+14*$i, 5);
// Rate counts
            $vals[13+$i] = substr($rec, 28+14*$i, 9);
          }
// PH event counts
          for ($i = 0; $i < 4; $i++) $vals[] = substr($rec, 205+5*$i, 5);
// Electrons
          $vals[] = substr($rec, 225, 5);
          $vals[] = substr($rec, 270, 5);
          $vals[] = substr($rec, 285, 5);
// Protons
          for ($i = 0; $i < 4; $i++) $vals[] = substr($rec, 230+5*$i, 5);
          $vals[] = substr($rec, 275, 5);
          for ($i = 0; $i < 6; $i++) $vals[] = substr($rec, 290+5*$i, 5);
// Helium
          for ($i = 0; $i < 4; $i++) $vals[] = substr($rec, 250+5*$i, 5);
          $vals[] = substr($rec, 280, 5);
          for ($i = 0; $i < 7; $i++) $vals[] = substr($rec, 320+5*$i, 5);
// CNO
          $vals[] = substr($rec, 355, 5);
// Sun_Earth_IMP angle
          $vals[] = substr($rec, 360, 4);
        }
        fprintf($tfile, "%20.3f, %s, %d, %d",
                        $cdf_epoch, $epoch, $ms, $quality);
        foreach ($vals as $val)
          fprintf($tfile, ",%9d", ($val < 0) ? -999 : $val);
        fprintf($tfile, ", %d%s", $fid, LF);
      }
      fclose($tfile);

      $result = TRUE;
    }
    else
      $result = FALSE;

    return $result;
  }

?>