<?php

/**
* Parser function for ISEE1.
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noorwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 2 December, 2009 
*
* @version $Id: ISEE1_MEPI.php 253 2009-12-18 14:07:30Z peter $
*
* @package ODI
* @subpackage ParsersFunctions
*/

// Parse a data file and write formatted content to load.tmp

  function parse_ISEE1_MEPI($rname, $fid)
  {
// Export the cdf file
    exec($_SERVER["ODI_HOME"] . "/parsers/cdfexport -batch text -settings ISEE1_MEPI.set -initial \"noheading,nooutput\" -text tmp.txt " . $rname);
    $tHandle = fopen("tmp.txt", "r");
    if ($tHandle !== FALSE)
    {
      $tfile = fopen("load.tmp", "w");
      while (!feof($tHandle))
      {
        $rec = fgets($tHandle);
        $rec = preg_split("/[\s]+/", trim($rec));
        if (count($rec) > 1)
        {
          $cdf_epoch = $rec[28];
          list($epoch, $ms) = cdfepoch_to_date($cdf_epoch);
          fprintf($tfile, "%20.1f, %s, %d", $cdf_epoch, $epoch, $ms);
// Convert to Cartesian coordinates
// Longitude and latitude have been interchanged in the cdf file!
          $lon = deg2rad($rec[8]);
          $lat = deg2rad($rec[7]);
          $alt = $rec[3] * 6371.2;
          $x = $alt * cos($lon) * cos($lat);
          $y = $alt * sin($lon) * cos($lat);
          $z = $alt * sin($lat);
          fprintf($tfile, str_repeat(",%.6e", 3), $x, $y, $z);
          fprintf($tfile, str_repeat(",%s", 3), $rec[8], $rec[7], $rec[3]);
// GSE coordinates
          fprintf($tfile, str_repeat(",%s", 3), $rec[4], $rec[5], $rec[6]);
// L,B values
          fprintf($tfile, ",%s,%s", $rec[1], $rec[2]);
// Magnetic field vector
          for ($i = 0; $i < 3; $i++)
            fprintf($tfile, ",%s", $rec[9+$i]);
// Electron and proton fluxes
          for ($i = 0; $i < 16; $i++)
            fprintf($tfile, ",%s", $rec[12+$i]);
// Bitrate flag
          fprintf($tfile, ",%s", $rec[0]);

          fprintf($tfile, ", %d%s", $fid, LF);
        }
      }
      fclose($tfile);
      @unlink("tmp.txt");
      $result = TRUE;
    }
    else
      $result = FALSE;

    return $result;
  }

?>