<?php

/**
* Parser function for ISEE2 KED.
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noorwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 2 December, 2009 
*
* @version $Id: GOES_SEM.php 224 2009-12-03 12:49:21Z peter $
*
* @package ODI
* @subpackage ParsersFunctions
*/

// Parse a data file and write formatted content to load.tmp
  function parse_ISEE2_KED($rname, $fid)
  {
// Export the cdf file
    exec($_SERVER["ODI_HOME"] . "/parsers/cdfexport -batch text -settings ISEE2_KED.set -initial \"noheading,nooutput\" -text tmp.txt " . $rname);
    $tHandle = fopen("tmp.txt", "r");
    if ($tHandle !== FALSE)
    {
      $tfile = fopen("load.tmp", "w");
      while (!feof($tHandle))
      {
        $rec = fgets($tHandle);
        $rec = preg_split("/[\s]+/", trim($rec));
        if (count($rec) > 1)
        {
          $cdf_epoch = $rec[70];
          list($epoch, $ms) = cdfepoch_to_date($cdf_epoch);
          fprintf($tfile, "%20.1f, %s, %d", $cdf_epoch, $epoch, $ms);
// Convert to Cartesian coordinates
          $lon = deg2rad($rec[0]);
          $lat = deg2rad($rec[1]);
          $alt = $rec[2] * 6371.2;
          $x = $alt * cos($lon) * cos($lat);
          $y = $alt * sin($lon) * cos($lat);
          $z = $alt * sin($lat);
          fprintf($tfile, str_repeat(",%.6e", 3), $x, $y, $z);
          fprintf($tfile, str_repeat(",%s", 3), $rec[0], $rec[1], $rec[2]);
// Extract magnetic field strengths (measured, local and 9 mirror points)
          fprintf($tfile, ",%s", $rec[3]);
          for ($i = 0; $i < 10; $i++)
            fprintf($tfile, ",%s", $rec[39+$i]);
// L values
          for ($i = 0; $i < 9; $i++)
            fprintf($tfile, ",%s", $rec[49+$i]);
// Invariant latitudes
          for ($i = 0; $i < 9; $i++)
            fprintf($tfile, ",%s", $rec[58+$i]);
// Indices
          fprintf($tfile, str_repeat(",%s", 5),
                          $rec[34], $rec[35], $rec[36], $rec[37], $rec[38]);
// Electron fluxes
          for ($i = 0; $i < 30; $i++)
            fprintf($tfile, ",%s", $rec[4+$i]);
// WAPS fluxes
          for ($i = 0; $i < 2; $i++)
            fprintf($tfile, ",%s", $rec[68+$i]);
// Quality flag
          fprintf($tfile, ",%s", $rec[67]);

          fprintf($tfile, ", %d%s", $fid, LF);
        }
      }
      fclose($tfile);
      @unlink("tmp.txt");
      $result = TRUE;
    }
    else
      $result = FALSE;

    return $result;
  }

?>