<?php
/**
* Parser function for KpAP.
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noorwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 2 December, 2009 
*
* @version $Id: KpAp.php 252 2009-12-17 20:52:43Z peter $
*
* @package ODI
* @subpackage ParsersFunctions
*/


  function dirscan_KpAp($basedir, $filepattern)
  {
// Scan files in data directory
    $dfiles = dirscan_generic($basedir, "%");

// Add the Potsdam last and current month tables and Lund forecast
// page to the file list for KpAp
    if (strpos($filepattern, "_3h") !== FALSE)
    {
      $dfiles[] = array("kpap_3h/potsdam/kpyymm.wdc", date("Y/m/d H:i:s"),
                        "http://www-app3.gfz-potsdam.de/kp_index/kpyymm.wdc");
      $dfiles[] = array("kpap_3h/potsdam/pqlyymm.wdc", date("Y/m/d H:i:s"),
                        "http://www-app3.gfz-potsdam.de/kp_index/pqlyymm.wdc");
      $dfiles[] = array("kpap_3h/lund/kp0.txt", date("Y/m/d H:i:s"),
                        "http://rwc.lund.irf.se/rwc/kp/kp0.txt");
    }
    else
    {
      $dfiles[] = array("kpap_1d/potsdam/kpyymm.wdc", date("Y/m/d H:i:s"),
                        "http://www-app3.gfz-potsdam.de/kp_index/kpyymm.wdc");
      $dfiles[] = array("kpap_1d/potsdam/pqlyymm.wdc", date("Y/m/d H:i:s"),
                        "http://www-app3.gfz-potsdam.de/kp_index/pqlyymm.wdc");
    }

    return $dfiles;
  }


// Parse a data file and write formatted content to load.tmp
  function parse_KpAp($rname, $fid, $sk_file)
  {
    $ms = 0;
    $kpaps = file($rname);
    $tfile = fopen("load.tmp", "w");
    if ($kpaps !== FALSE)
    {
      if ($sk_file == "KpAp_3h.skt")
      {
// Predicted Kp from Lund
        if (strpos($rname, "lund") !== FALSE)
        {
          $dum = preg_split("/\s+/", trim($kpaps[0]));
          $date = $dum[0];
          $hour = substr($dum[1], 0, 2);
          $epoch = $date . " " . $hour . ":00:00";
          $kpval = floatval($dum[2]);
          $cdf_epoch = date_to_cdfepoch($epoch, $ms);
          fprintf($tfile, "%20.3f, %s, %d, %f, %d,NULL,NULL, %d%s",
                          $cdf_epoch, $epoch, $ms, $kpval, 3, $fid, LF);
        }
// 3h Kp and Ap from NGDC and Potsdam
        else
        {
          if (strpos($rname, "qlyymm") !== FALSE)
            $flag = 2;
          else
            $flag = 0;
          foreach ($kpaps as $kpap)
          {
            $year = intval(substr($kpap, 0, 2));
            if ($year > 30) $year += 1900; else $year += 2000;
            $month = intval(substr($kpap, 2, 2));
            $day = intval(substr($kpap, 4, 2));
            for ($i = 0; $i < 8; $i++)
            {
              $hour = $i * 3;
              $epoch = sprintf("%04d-%02d-%02d %02d:00:00", $year, $month, $day, $hour);
              $cdf_epoch = date_to_cdfepoch($epoch, $ms);
              $kpd = intval(substr($kpap, 12+$i*2, 2));
              $kpq = floor($kpd/10);
              $kpr = $kpd % 10;
              switch ($kpr)
              {
                case 0:
                  $kp = $kpq;
                  break;
                case 3:
                  $kp = $kpq + 1.0 / 3.0;
                  break;
                case 7:
                  $kp = $kpq + 2.0 / 3.0;
                  break;
              }
              $ap = intval(substr($kpap, 31+$i*3, 3));
              fprintf($tfile, "%20.3f, %s, %d, %f, %d, %d, %d, %d%s",
                              $cdf_epoch, $epoch, $ms, $kp, $flag, $ap, $flag, $fid, LF);
            }
          }
        }
      }
      else
      {
// Daily values from NGDC and Potsdam
        if (strpos($rname, "qlyymm") !== FALSE)
          $flag = 2;
        else
          $flag = 0;
        foreach ($kpaps as $kpap)
        {
          $year = intval(substr($kpap, 0, 2));
          if ($year > 30) $year += 1900; else $year += 2000;
          $month = intval(substr($kpap, 2, 2));
          $day = intval(substr($kpap, 4, 2));
          $epoch = sprintf("%04d-%02d-%02d 00:00:00", $year, $month, $day);
          $cdf_epoch = date_to_cdfepoch($epoch, $ms);
          $ap = substr($kpap, 55, 3);
          $cp = substr($kpap, 58, 3);
          $c9 = substr($kpap, 61, 1);
          fprintf($tfile, "%20.3f, %s, %d, %s, %d, %s, %d, %s, %d",
                          $cdf_epoch, $epoch, $ms, $ap, $flag, $cp, $flag, $c9, $flag);
          if (strpos($rname, "potsdam") === FALSE)
            fprintf($tfile, ",%s, %d, %s, %s", substr($kpap, 62, 3), $flag,
                            substr($kpap, 65, 5), substr($kpap, 70, 1));
          else
            fprintf($tfile, ",NULL,NULL,NULL,NULL");
          fprintf($tfile, ",%d%s", $fid, LF);
        }
      }
      fclose($tfile);
      $result = TRUE;
    }
    else
      $result = FALSE;

    return $result;
  }

?>