<?php

/**
* Parser function for STRV1B.
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noordwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 2 December, 2009 
*
* @version $Id: ISEE1_MEPI.php 253 2009-12-18 14:07:30Z peter $
*
* @package ODI
* @subpackage ParsersFunctions
*/

// Parse a data file and write formatted content to load.tmp

  function parse_STRV1B_REM($rname, $fid)
  {
// Export the cdf files
    exec($_SERVER["ODI_HOME"] . "/parsers/cdfexport -batch text -settings A.set -initial \"noheading,nooutput\" -text tmpA.txt " . $rname);
    $Bname = substr_replace($rname, "sB", -12, 2);
    exec($_SERVER["ODI_HOME"] . "/parsers/cdfexport -batch text -settings B.set -initial \"noheading,nooutput\" -text tmpB.txt " . $Bname);
    $aHandle = fopen("tmpA.txt", "r");
    $bHandle = fopen("tmpB.txt", "r");
    if (($aHandle !== FALSE) && ($bHandle !== FALSE))
    {
      $fepoch = "19" . substr($rname, -10, 2) . "-" .
                substr($rname, -8, 2) . "-" . substr($rname, -6, 2) .
                " 00:00:00";
      $tfile = fopen("load.tmp", "w");
      while (!feof($aHandle))
      {
        $arec = fgets($aHandle);
        $arec = preg_split("/[\s]+/", trim($arec));
        if (count($arec) > 1)
        {
          $asec = array_shift($arec);
          if ($asec > 0.0)
          {
            $brec = fgets($bHandle);
            $brec = preg_split("/[\s]+/", trim($brec));
            $bsec = array_shift($brec);
            if ($bsec == $asec)
            {
              $cdf_epoch = date_to_cdfepoch($fepoch, 0) + $asec * 1000.0;
              list($epoch, $ms) = cdfepoch_to_date($cdf_epoch);
              fprintf($tfile, "%20.1f, %s, %3d", $cdf_epoch, $epoch, $ms);
// Accumulation time, channels, temperatures and voltages
              foreach($arec as $rec)
                fprintf($tfile, ",%.10e", $rec);
// DTCF
              for ($i = 0; $i < 2; $i++)
                fprintf($tfile, ",%.10e", $brec[$i]);
// Proton fluxes
              for ($i = 0; $i < 5; $i++)
                fprintf($tfile, ",%.10e", $brec[2*$i+2]);
// Proton errors
              for ($i = 0; $i < 5; $i++)
                fprintf($tfile, ",%.10e", $brec[2*$i+3]);
// Electron fluxes
              for ($i = 0; $i < 3; $i++)
                fprintf($tfile, ",%.10e", $brec[2*$i+12]);
// Electron errors
              for ($i = 0; $i < 3; $i++)
                fprintf($tfile, ",%.10e", $brec[2*$i+13]);
// Geographic and magnetic coordinates
              for ($i = 0; $i < 5; $i++)
                fprintf($tfile, ",%.10e", $brec[$i+18]);
              fprintf($tfile, ", %d%s", $fid, LF);
            }
          }
        }
      }
      fclose($tfile);
      @unlink("tmpA.txt");
      @unlink("tmpB.txt");
      $result = TRUE;
    }
    else
      $result = FALSE;

    return $result;
  }

?>