#!/usr/bin/php
<?php
/**
* Populate the ODI database.
* 
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noorwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 2 December, 2009 
*
* @version $Id: populate.php 253 2009-12-18 14:07:30Z peter $
*
* @package ODI
* @subpackage Admin
*/

require_once($_SERVER["ODI_HOME"]."/lib/odi.library.php");

//require_once("dirscan.php");

//require_once("parsefile.php");

// Load directory scan and file parse functions for individual datasets
$fdir = scandir("functions");
if (count($fdir) > 2)
{
    array_shift($fdir);
    array_shift($fdir);
    foreach ($fdir as $ffile)
    require_once("functions/" . $ffile);
}

$rawdir = getenv("ODI_RAWDATA");
if ($rawdir === FALSE)
{
    print "  Environment variable ODI_RAWDATA not set!" . LF;
    exit;
}

// Strip the command line arguments of the first entry, i.e. the script name
array_shift($argv);

print "Adding or updating data sets ..." . LF;
// Connect to the database at user level 1.
$conn_array = get_conn_array(1);
$conn = connect_to_database_array($conn_array);

$entries = file("datasets.txt");

foreach ($entries as $entry)
{
    $args = explode(";;;", $entry);
    $process = FALSE;
    if (in_array(trim($args[0], "# "), $argv))
      $process = TRUE;
    else
      if ((substr($entry, 0, 1) != "#") && (count($argv) == 0))
        $process = TRUE;

    if ($process)
    {
        $dataset_name = "dataset_" . trim($args[0], "# ");
        $basedir = $rawdir . trim($args[1]) . "/";
        $filepattern = trim($args[2]);
        $platform = trim($args[3]);
        $platform_type = trim($args[4]);
        $instrument = trim($args[5]);
        $sk_file = trim($args[6]);
        $parse_file = trim($args[7]);
        $availability = trim($args[8]);
        $sedat_name = trim($args[9]);
        $sedat_regions = trim($args[10]);
        $sedat_particles = trim($args[11]);
        $sedat_emin = trim($args[12]);
        $sedat_emax = trim($args[13]);
        
        // Name of function to list files in the raw data directory.
        $scanner = "dirscan_" . $parse_file;
        
        // Name of function to parse the raw data.
        $parser  = "parse_"   . $parse_file;
        
        if ( !function_exists($scanner) ) {
            $scanner = "dirscan_generic";
        }
        
        if ( !function_exists($parser) ) {
            echo "Function " . $parser . " does not exist." . LF;
            echo "Must be defined in parsers/functions/" . $parse_file . ".php" . LF;
            exit;
        }
        
        print "   Dirscan function: " . $scanner . LF;
        print "   Parser function:  " . $parser  . LF;
        
        chdir($platform);
        
        // Check if the dataset already exists.
        if (!exists_dataset($conn, $dataset_name))
        {
            skt2odi($conn, $dataset_name, $platform, $platform_type, $instrument, $sk_file, $availability,
                $sedat_name, $sedat_regions, $sedat_particles, $sedat_emin, $sedat_emax);
        }

        print "   Processing " . $dataset_name . " ..." . LF; 

        print "   Will use data from " . $basedir . LF;

        $nf = ingest_data($conn, $dataset_name, $basedir, $platform, $instrument, $filepattern, $sk_file,
                            $scanner, $parser);
        if ($nf > 0) print "      " . $nf . " files updated or added for " . $dataset_name . LF;
            chdir("../");
    }
}

mysqli_close($conn);

?>