#!/usr/bin/php
<?php
/**
* Check the epoch for a selected dataset.
* 
* Usage:
*
* <samp>
*  check_epoch.php dataset_name
* </samp>
*
* where <var>dataset_name</var> is the SEDAT dataset name.
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noorwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 23 February, 2010 
*
* @version $Id: check_epoch.php 282 2010-02-24 08:14:43Z peter $
*
* @package ODI
* @subpackage Tools
*/

require_once($_SERVER["ODI_HOME"].'/lib/odi.library.php');

// Get the input arguments.
$arg_array=array("sedat_name");
$args=get_cl_arguments($argv,$arg_array);

// Connect to the database at user level 1.
$conn_array=get_conn_array(1);
$conn=connect_to_database_array($conn_array);

// Check that the dataset metadata if it exists.
$q = 'SELECT * FROM dataset WHERE sedat_name='.quote($args["sedat_name"]);
if (!($res=$conn->query($q))) printf("Query: %s\nError: %s\n",$q,$conn->error);
if($row=$res->fetch_assoc()) {
    fwrite(STDOUT,"===================".LF.$args["sedat_name"].LF."------------".LF);
    $num_rec = $row["numrecs"];
    $epoch_start = $row["epoch_start"];
    $epoch_end = $row["epoch_end"];
    $dataset_name = $row["name"];
    $dataset_id = $row["id"];
    list($date,$ms) = cdfepoch_to_date($epoch_start);
    $date_start = $date.".".$ms;
    list($date,$ms) = cdfepoch_to_date($epoch_end);
    $date_end = $date.".".$ms;
    fwrite(STDOUT,"Number of records:    ".$num_rec.LF);
    fwrite(STDOUT,"First epoch:          ".$epoch_start." (".$date_start.")".LF);
    fwrite(STDOUT,"Last epoch:           ".$epoch_end." (".$date_end.")".LF);
    
    // Find median time step.    
    $q = "SELECT cdf_epoch FROM ".$dataset_name." ORDER BY cdf_epoch";
    if (!($res=$conn->query($q))) printf("Query: %s\nError: %s\n",$q,$conn->error);
    $delta_num = array(); // array ( $delta => $number_of_delta )
    $row = $res->fetch_assoc();
    $e1 = $row["cdf_epoch"];
    while($row=$res->fetch_assoc()) {
        $e2 = $row["cdf_epoch"];
        $d = strval($e2 - $e1);
        $e1 = $e2;
        if(array_key_exists($d,$delta_num)) {
            $delta_num[$d]++;
        }
        else {
            $delta_num[$d] = 1;
        }
    }
    // Sort so that the entry with largest $delta_num comes first.
    arsort($delta_num);
    // The keys are the actual $delta value.
    $delta = array_keys($delta_num);
    // Get the first $delta => median $delta.
    $delta_median = $delta[0];
    printf("Median time step (s): %0.3f".LF,$delta_median/1000);
    fwrite(STDOUT,"------------".LF);
    
    // Find times with time steps different from the median.
    $res->data_seek(0);
    $row = $res->fetch_assoc();
    $e1 = $row["cdf_epoch"];
    $all_equal = TRUE;
    while($row=$res->fetch_assoc()) {
        $e2 = $row["cdf_epoch"];
        $d = $e2 - $e1;
        if($d != $delta_median) {
            list($date,$ms) = cdfepoch_to_date($e1);
            $date_str = $date.".".$ms;
            $ds = $d/1000;
            printf("Time step = %0.3f s ",$ds);
            if($ds>60) {
                if($ds>3600) {
                    if($ds>86400) {
                        printf("(%0.1f days)",$ds/86400);
                    }
                    else {
                        printf("(%0.1f hours)",$ds/3600);
                    }
                }
                else {
                    printf("(%0.1f minutes)",$ds/60);
                }
            }
            printf(" at %s (%s)".LF,$e1,$date_str);
            $all_equal = FALSE;
        }
        $e1 = $e2;
    }
    if($all_equal) {
        fwrite(STDOUT,"All time steps are equal".LF);
    }
    fwrite(STDOUT,"------------".LF);
}
else {
    fwrite(STDOUT,$args["sedat_name"]." does not exist!\n");
    exit(0);
}
?>
