#!/usr/bin/php
<?php
/**
* Create a template XML file to be used with the <samp>export_to_xml</samp> program.
*
* This program is used to prepare an export format file to be used by the {@link export_to_xml.php}
* program. This program creates an XML file based on the metadata for a given dataset in the ODI
* database. The XML file name will be the SEDAT dataset name with the <samp>.set.xml</samp>
* extension. The <samp>.set.xml</samp> file may then be edited to produce the desired exported data.
*
* Usage:
*
* <samp>
*  export_build_xml.php dataset_name
* </samp>
*
* where <var>dataset_name</var> is the SEDAT dataset name.
* 
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noorwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 2 December, 2009 
*
* @version $Id: export_build_xml.php 224 2009-12-03 12:49:21Z peter $
*
* @package ODI
* @subpackage Tools
*/

require_once($_SERVER["ODI_HOME"].'/lib/odi.library.php');

// Get the input arguments.
$arg_array=array("dataset");
$args=get_cl_arguments($argv,$arg_array);
$dataset=$args["dataset"];

// Connect to the database at user level 3.
$conn_array=get_conn_array(3);
$conn=connect_to_database_array($conn_array);

// Open the XML output file.
$file_name = $dataset.".set.xml";
$file = fopen($file_name,"w");

// XML head.
fwrite($file,'<?xml version="1.0"?>'.LF);
fwrite($file,'<odi>'.LF);
fwrite($file,"<creator>Your Name (email)</creator>".LF);
fwrite($file,"<export_file>".$dataset.".xml</export_file>".LF);
fwrite($file,"<dataset>".$dataset."</dataset>".LF);

// Get the end epoch. Set start epoch = end epoch - one day.
$q = "SELECT epoch_end FROM dataset WHERE sedat_name=".quote($dataset);
if (!($res=$conn->query($q))) printf("Query: %s\nError: %s\n",$q,$conn->error);
if ($row=$res->fetch_assoc()) {
    $epoch_end_cdf = $row["epoch_end"];
}
else {
    fwrite(STDOUT,"Dataset does not exist, or no data in dataset: ".$dataset.LF);
    exit;
}
$epoch_start_cdf = $epoch_end_cdf - 24*60*60*1000;
list($epoch_start,$ms_start) = cdfepoch_to_date($epoch_start_cdf);
fwrite($file,"<epoch_start>".$epoch_start.".".$ms_start."</epoch_start>".LF);
list($epoch_end,$ms_end) = cdfepoch_to_date($epoch_end_cdf);
fwrite($file,"<epoch_end>".$epoch_end.".".$ms_end."</epoch_end>".LF);

fwrite($file,"<max_num_rec>1000</max_num_rec>".LF);

// Get the variable names for this dataset.
$q = "SELECT vm.name AS name FROM variable_metadata AS vm";
$q.= " LEFT JOIN dataset AS ds ON vm.dataset_id=ds.id";
$q.= " WHERE vm.record_variance='T'";
$q.= " AND ds.sedat_name=".quote($dataset);
if (!($res=$conn->query($q))) printf("Query: %s\nError: %s\n",$q,$conn->error);
while($row=$res->fetch_assoc()) {
    fwrite($file,"<variable>".$row["name"]."</variable>".LF);
}

fwrite($file,'</odi>'.LF);

// Close XML file.
fclose($file);

?>