#!/usr/bin/php
<?php
/**
* Get the SEDAT name, ODI name, number of records and epoch range for the datasets
* that have been updated with files in a given date range
*
* Usage:
*
* <samp>
* show_updates.php date_start date_end
* </samp>
*
* where <samp>date_start</samp> and <samp>date_end</samp> specify the range for the
* ingestion dates of the input files. Dates containing blanks should be enclosed in
* double quotes ("2000-01-01 00:00:00")!
* 
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noorwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 2 December, 2009 
*
* @version $Id: show_updates.php 275 2010-01-22 12:00:42Z daniel $
*
* @package ODI
* @subpackage Tools
*/

require_once($_SERVER["ODI_HOME"].'/lib/odi.library.php');

// Get the input arguments.
if(count($argv) == 3) {
    $date1=$argv[1];
    $date2=$argv[2];
}
else {
    fwrite(STDOUT,"Error: Incorrect number of input arguments" . LF);
    fwrite(STDOUT,"Usage: show_datasets.php \"<date_start>\" \"<date_end>\"" . LF);
    exit;
}

// Connect to the database at user level 3.
$conn_array=get_conn_array(3);
$conn=connect_to_database_array($conn_array);

// Get the epoch range.
$q = "SELECT ds.name, ds.sedat_name FROM dataset AS ds LEFT JOIN dataset_file AS df " .
     "ON (ds.id=df.dataset_id) WHERE df.ingestion_date>= '" . $date1 .
     "' AND df.ingestion_date<= '" . $date2 . "' GROUP BY df.dataset_id";
if (!($res=$conn->query($q))) printf("Query: %s" . LF . "Error: %s" . LF,$q,$conn->error);
while($row=$res->fetch_assoc()) {
    $q = "SELECT MIN(ds.cdf_epoch) AS t1, MAX(ds.cdf_epoch) AS t2, " .
         "COUNT(cdf_epoch) AS numrecs FROM " . $row["name"] .
         " AS ds LEFT JOIN dataset_file AS df ON (ds.dataset_file_id=df.id) " .
         " WHERE df.ingestion_date>= '" . $date1 .
         "' AND df.ingestion_date<= '" . $date2 . "'";
    if (!($dres=$conn->query($q))) printf("Query: %s" . LF . "Error: %s" . LF,$q,$conn->error);
        $line=$dres->fetch_assoc();
        fwrite(STDOUT,$row["sedat_name"]." (".$row["name"].") ");
        fwrite(STDOUT,$line["numrecs"]." records");
        if ($line["numrecs"] > 0)
        {
            list($date,$ms) = cdfepoch_to_date($line["t1"]);
            fwrite(STDOUT,": ".$date.".".str_pad($ms,3,"0",STR_PAD_LEFT)." to ");
            list($date,$ms) = cdfepoch_to_date($line["t2"]);
            fwrite(STDOUT,$date.".".str_pad($ms,3,"0",STR_PAD_LEFT));
        }
        fwrite(STDOUT,LF);
}

?>